C************************************************************************
      program dust_flux
C************************************************************************
C
C     J. Zmuidzinas  5-20-95
C
C     Calculates emergent dust spectrum of arbitrary spherical
C     cloud
C
C************************************************************************
C
      character*80 paramfile, dustfile,
     +                 outfile
      common/filenames/
     +                 paramfile, dustfile,
     +                 outfile
C
      write(*,*) ' File with initial model parameters and fluxes ? '
10    format(A)
      read(*,10) paramfile
      write(*,*) ' Output file ? '
      read(*,10) outfile
C
      call getparam(paramfile)     ! read in parameters, fluxes
      call calcobserv(outfile)
C
      stop
      end
C
C************************************************************************
      subroutine getparam(fname)
C************************************************************************
C
      character*(*) fname
      character*100 textline
C
      include 'grid.inc'
      include 'obsdust.inc'
      include 'cgs_constants.inc'
C
      real*8 wav, bw, flux
      integer ir, ib
C
      character*80 paramfile, dustfile,
     +                 outfile
      common/filenames/
     +                 paramfile, dustfile,
     +                 outfile
C
      open(UNIT=1, NAME=fname, STATUS='UNKNOWN')
C
10    format(A)
C
      read(1,10) dustfile
      ib = index(dustfile, ' ') -1     ! delete trailing blanks
      dustfile = dustfile(1:ib)
C
      read(1,*) thickratio        ! thickness ratio, first:last shells
C
      read(1,*) rscalecm
      if(rscalecm.LE.0) then
         write(*,*) ' Error in parameter file - rscalecm <= 0 !!'
         close(1)
         stop
      endif
C
      read(1,*) rmin, rmax
      if(rmin.GE.rmax) then
         write(*,*) ' Error in parameter file - rmin >= rmax !!'
         close(1)
         stop
      endif
C
      read(1,*) nshell
      if(nshell.LT.4) then
         write(*,*) ' Error in parameter file - '
         write(*,*) ' Need at least 4 radial shells!'
         write(*,*) ' Found: ', nshell
         close(1)
         stop
      else if(nshell.GT.MAXGRID) then
         write(*,*) ' Error in parameter file - '
         write(*,*) ' Too many radial shells: ', nshell
         write(*,*) ' MAXGRID = ', MAXGRID
         write(*,*)
     +   ' Reduce number of shells or recompile with larger MAXGRID'
         close(1)
         stop
       endif
C
      read(1,*) h2den0, h2den1, h2denradius, h2denpower
      if(h2den0.LT.0. .OR. h2den1.LT.0 .OR. h2denradius.LT.0.) then
         write(*,*) ' Error in parameter file - h2den law invalid'
         close(1)
         stop
      endif
C
      read(1,*) Tdust0, Tdust1, Tdustradius, Tdustpower, Tdustmin
      if(Tdust0.LT.0. .OR. Tdust1.LT.0 .OR. Tdustradius.LT.0.) then
         write(*,*) ' Error in parameter file - Tdust law invalid'
         close(1)
         stop
      endif
C
      read(1,*) colfg, tempfg        ! foreground dust
      if(colfg.LT.0. .OR. tempfg.LT.0) then
         write(*,*) ' Error in parameter file - foreground dust invalid'
         close(1)
         stop
      endif
C
C     Fake the other parameters, since not needed here
C
      Tgas0 = Tdust0
      Tgas1 = Tdust 1
      Tgasradius = Tdustradius
      Tgaspower = Tdustpower
      Tgasmin = Tdustmin
      abun0 = 1.
      abun1 = 0.
      abunradius = 1.
      abunpower = -1.
      numobs = 0
C
      read(1,10, END=50) textline
      read(textline,*) distance        ! get distance to source in
C                                      ! rscalecm units (usually pc)
20    continue
         read(1,10, END=50) textline
         read(textline,*) wav, bw, flux
         numobs = numobs+1
         beamsize(numobs) = bw
         d_wavl(numobs) = wav
         obsflux(numobs) = flux
      if(numobs.LT.MAXOBS) then
         goto 20
      endif
C
50    continue
      close(1)
C
C     Calculate grid
C
      call calcgrid
C
C     Normalize shell radii now
C
      do 300 ir=1, nshell
         rshell(ir) = rshell(ir)/rmax
300   continue
C
C     get dust parameters...
C
      call getdust(dustfile)                ! dust parameters
C
      return
      end
C
C**********************************************************************
      subroutine getdust(infile)
C**********************************************************************
C
C     reads dust parameters from disk
C
      include 'dust.inc'
C
      character*80 infile
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
      read(1,*) Qdust0
      read(1,*) wavdust0
      read(1,*) slopedust
      read(1,*) adust
      read(1,*) rhodust
      read(1,*) Rgasdust
      read(1,*) gasmu
      close(1)
      return
      end
C
C**********************************************************************
      real*8 function sigmadust(wavlen)
C**********************************************************************
C
C     Calculates dust cross-section (in cm-2) per H2 molecule
C
      real*8 wavlen                     ! wavelength in microns
C
      include 'dust.inc'
      include 'cgs_constants.inc'       ! for amicron = 1.E-4
C
      real*8 Qdust, result
C
      Qdust = Qdust0*(wavdust0/wavlen)**slopedust    ! efficiency
      result = 3*gasmu*Qdust/(4.*adust*amicron*rhodust*Rgasdust)
      sigmadust = result
      return
      end
C
C********************************************************************
      subroutine pathlen(r, nr, xp, ds, mr)
C********************************************************************
C
C     ***calculates pathlength thru shells 
C
C        Inputs:
C        r               - array of radii, in ASCENDING ORDER
C        nr              - no. of radii
C        xp              - impact parameter
C
C        Output:
C        ds(nr)          - pathlengths
C        mr              - index of minimum radius penetrated by xp
C********************************************************************
C
      real*8 r(nr), xp
      real*8 ds(nr)
      integer mr
C
      integer ir
      real*8 x1, x2
C
C     ***calculate mr
C
      ir = 1                  ! start from smallest radius
C
10    continue
      if(r(ir).LT.xp .AND. ir.LT.nr) then
         ir = ir+1            ! impact ip doesn't penetrate radius ir
         goto 10              ! so go to next larger radius
      endif
C
      if(r(ir).GE.xp) then
         mr = ir          ! smallest radius that ip penetrates
      else
         mr = nr+1        ! impact out of range - ip doesn't 
      endif               ! penetrate any radii
C
C     Calculate pathlengths
C
      ir = mr
      if(ir.LE.nr) then        ! this ray actually passes thru cloud
         x2 = xp/r(ir)
         x2 = x2*x2            ! calculate pathlength from ray center
C                              ! to first intersecting shell
         ds(ir) = r(ir)*sqrt(1.-x2)
C
         do 50 ir=mr+1, nr    ! now do pathlengths between shells
            x1 = r(ir)/r(ir-1)
            x1 = x1*x1
            x2 = xp/r(ir-1)
            x2 = x2*x2
            ds(ir) = r(ir-1)*(x1-1.)
     +               /(sqrt(x1-x2) + sqrt(1.-x2))
50       continue
      endif
C
      return
      end
C
C********************************************************************
      subroutine calcobserv(fname)
C********************************************************************
C
C     solve the radiative transfer for each observation and write
C     spectrum to output file
C
      character*(*) fname
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'intensity.inc'
      include 'cgs_constants.inc'
      include 'obsdust.inc'
C
      real*8 sigmadust
      external sigmadust
C
      character*80 paramfile, dustfile,
     +                 outfile
      common/filenames/
     +                 paramfile, dustfile,
     +                 outfile
C
      real*8 chisq
      common/fit/chisq
C
      integer ishell, iobs
C
C
      real*8 sdust(MAXGRID)              ! dust source function
      real*8 dustopacity(MAXGRID)        ! dust opacity
C
      real*8 wavelen                     ! wavelength in cm
      real*8 xsectdust                   ! dust absorption cross-section
C
      real*8 x1, tph
      real*8 sp
      real*8 d_freq, spectrum, flux, Omega
      real*8 taufg, sfg
C
C     ***Open output file
C
      open(UNIT=2, FILE=fname, STATUS='UNKNOWN')
C
1     format(A)
      write(2,1) dustfile
      write(2,1) paramfile
      write(2,1) outfile
      write(2,*)
      write(2,*) ' Wavelength, beamsize,',
     +           ' TA*, model flux, observed flux,',
     +           ' fit error, mean radius:'
      write(*,1) dustfile
      write(*,1) paramfile
      write(*,1) outfile
      write(*,*)
      write(*,*) ' Wavelength, beamsize,',
     +           ' TA*, model flux, observed flux,',
     +           ' fit error, mean radius:'
C
C     Loop over observations...
C
      iobs = 1
      chisq = 0.
111   continue
C
C        ***1/e beam radius in rscalecm units
C
         ebeamrad = beamsize(iobs)*distance*(PI/(180.*60.*60.))
     +              /(2.*sqrt(log(2.)))
         bmoffs = 0.
         xx0 = bmoffs/ebeamrad
C
C        ***Beam solid angle in sr from arcsec FWHM
C
         Omega = PI*((beamsize(iobs)*PI/(180.*60.*60.))**2.)
     +           /(4.*log(2.))
C
C        ***get wavelength in cm
C
         wavelen = d_wavl(iobs)/1.e4     ! d_wavl is in microns
C
C        ***calculate frequency in GHz
C
         d_freq = clight/(wavelen*1000.*amegahz)
C
C        ***dust absorption cross-section at this wavelength
C
         xsectdust = sigmadust(wavelen/amicron)
C
C        ***now calculate opacities in radial shells
C
         do 20 ishell=1, nshell
            dustopacity(ishell) = xsectdust*h2den(ishell)
     +                            *rmax*rscalecm
20       continue
C
C        ***now calculate dust source function
C
         tph = d_freq*1000.*amegahz*hplanck/kboltz
C
         do 30 ishell=1, nshell
            x1 = tph/(exp(tph/Tdust(ishell))-1.)
            sdust(ishell) = x1
C
30       continue
C
C
         do 50 ishell=1, nshell
            opacity(ishell) =  dustopacity(ishell)
50       continue
C
C           ***now calculate total source function
C
         do 60 ishell=1, nshell
            sourcefn(ishell) = sdust(ishell)
60       continue
C
C        ***we're ready to do the radiative transfer
C
         call intensity(spectrum, sp)
C
C        ***correct for foreground dust, if any
C
         sfg = tph/(exp(tph/tempfg)-1.)
         taufg = colfg*xsectdust
         taufg = exp(-taufg)
         spectrum = spectrum*taufg + sfg*(1.-taufg)
C
C        ***spectrum is antenna temp in beam
C           use this to calculate flux in Jy
C
         flux = 2*kboltz*Omega*spectrum/(jansky*wavelen**2)
         x1 = ((flux-abs(obsflux(iobs)))/obsflux(iobs))**2.
C
         write(2,63) d_wavl(iobs), beamsize(iobs),
     +              spectrum, flux, obsflux(iobs), x1,
     +              sp*ebeamrad/spectrum
63       format(' ', 2(F8.2, 2X), F9.4, 2X, 2(F10.3, 2X),
     +          E8.3, 2X, F10.5)
         write(*,63) d_wavl(iobs), beamsize(iobs),
     +              spectrum, flux, obsflux(iobs), x1,
     +              sp*ebeamrad/spectrum
C
         if(obsflux(iobs).GT.0) then
            chisq = chisq + x1
         endif
         iobs = iobs+1
      if(iobs.LE.numobs) then
         goto 111
      endif
C
      write(2,*) ' Chi square = ', chisq
      write(*,*) ' Chi square = ', chisq
      close(2)
C
      return
      end
c
C*******************************************************************
      subroutine intensity(out1, out2)
C*******************************************************************
C
      real*8 out1, out2
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'intensity.inc'
      include 'cgs_constants.inc'
C
      real*8 res1, res2, a, b, c
      real*8 integrand1
      external integrand1
      real*8 integrand2
      external integrand2
C
      a = 0.
      c = rshell(nshell)*rmax/ebeamrad
      b = xx0
C
      if(b.LT.c) then
         call qromb(integrand1, a, b, res1)
         call qromb(integrand1, b, c, res2)
         out1 = 2.*(res1+res2)
         call qromb(integrand2, a, b, res1)
         call qromb(integrand2, b, c, res2)
         out2 = 2.*(res1+res2)
      else
         call qromb(integrand1, a, c, res1)
         out1 = 2.*res1
         call qromb(integrand2, a, c, res1)
         out2 = 2.*res1
      endif
C
      return
      end
C
C*******************************************************************
      real*8 function integrand1(xx)
C*******************************************************************
C
      real*8 xx
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'intensity.inc'
      include 'cgs_constants.inc'
C
      real*8 outinten, xp, result
      real*8 bi0exp
      external bi0exp
C
      xp = xx*ebeamrad/rmax
      if(xp.LT.rshell(nshell)) then
         call doray(rshell, nshell, xp,
     +              opacity, sourcefn, outinten)
      else
         outinten = 0.
      endif
      result = outinten*xx*exp(-(xx-xx0)**2)*bi0exp(2.*xx*xx0)
C
      integrand1 = result
      return
      end
C
C*******************************************************************
      real*8 function integrand2(xx)
C*******************************************************************
C
      real*8 xx
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'intensity.inc'
      include 'cgs_constants.inc'
C
      real*8 outinten, xp, result
      real*8 bi0exp
      external bi0exp
C
      xp = xx*ebeamrad/rmax
      if(xp.LT.rshell(nshell)) then
         call doray(rshell, nshell, xp,
     +              opacity, sourcefn, outinten)
      else
         outinten = 0.
      endif
      result = outinten*xx*xx*exp(-(xx-xx0)**2)*bi0exp(2.*xx*xx0)
C
      integrand2 = result
      return
      end
C
C
C*******************************************************************
      subroutine doray(r,nr,xp,chi,ss,out)
C*******************************************************************
c
c     1-4-94  J. Zmuidzinas
c
c     Radiative transfer solver for spherical geometry
c     Based directly on 1-st order transfer eqn.
c     Solves for intensity along ray of given impact parameters
c     Assumes that opacity and source function vary linearly
c     between radial shells.
c
c     input:
c       r     - normalized shell radii (rgrid(1)=0, rgrid(nr)=1)
c       nr    - number of shells
c       xp    - impact parameter
c       chi   - absorption coefficient
c       ss    - source function
c     output:
c       out   - emergent intensity
c
c       **all are real*8 
c
c     All physical quantities (chi, s) are evaluated AT THE SHELL RADII.
c
c     The routine calculates the emergent intensity at the
c     outermost shell.
c     
C*******************************************************************
c
      implicit real*8 (a-h,o-z)
C
      include 'grid.inc'            ! for MAXGRID
C
C
C     Inputs:
      integer nr
      real*8 r(nr), xp, chi(nr), ss(nr)
C
C     Output:
      real*8 out
C
C     local variables:
C
      integer  mr
      real*8 dtau(MAXGRID), ifor(MAXGRID), iback(MAXGRID),
     +       e(MAXGRID), f(MAXGRID), g(MAXGRID),
     +       ds(MAXGRID)
      real*8 x1
C
      parameter (cut=1.e-2)
C
c     calculate pathlengths
c
      call pathlen(r, nr, xp, ds, mr)
c
c     calculate optical depth grid; e, f, g, h coefficients
c
      ifor(nr) = 0.               ! no radiation yet...
C
      ir = mr
c
      if(ir.LE.nr) then           ! ray actually passes thru cloud,
c                                 ! so go ahead...
c
      x1 = ds(ir)*chi(ir)         ! assume chi is constant in
      dtau(ir) = x1               ! innermost shell
      e(ir) = exp(-x1)
      if(abs(x1).LE.cut) then
         g(ir) = (x1/2.)*(1.-x1/3.*(1.-x1/4.*(1.-x1/5.*
     +                   (1.-x1/6.*(1.-x1/7.*(1.-x1/8.*
     +                   (1.-x1/9.*(1.-x1/10.))))))))
         f(ir) = x1*(1.-g(ir))
      else
         f(ir) = 1.-e(ir)
         g(ir)  =1. - f(ir)/x1
      endif
      do ir=mr+1, nr
         x1 = ds(ir)*(chi(ir)+chi(ir-1))*0.5
         dtau(ir) = x1
         e(ir) = exp(-x1)
         if(abs(x1).LE.cut) then
            g(ir) = (x1/2.)*(1.-x1/3.*(1.-x1/4.*(1.-x1/5.*
     +                      (1.-x1/6.*(1.-x1/7.*(1.-x1/8.*
     +                      (1.-x1/9.*(1.-x1/10.))))))))
            f(ir) = x1*(1.-g(ir))
         else
            f(ir) = 1.-e(ir)
            g(ir)  =1. - f(ir)/x1
         endif
      enddo
c
c     do the transfer
c
c     first, loop backwards to get ingoing ray
c
      iback(nr) = 0.              ! no background radiation incident
      do ir=nr-1, mr, -1
         iback(ir) = e(ir+1)*iback(ir+1)
     +               + f(ir+1)*ss(ir+1)
     +               + g(ir+1)*(ss(ir)-ss(ir+1))
      enddo
c
c     now loop forwards to get outgoing ray
c
      ir = mr
      x1 = e(ir)*e(ir)
      ifor(ir) = x1*iback(ir) + (1.-x1)*ss(ir)
      do ir=mr+1, nr
         ifor(ir) = e(ir)*ifor(ir-1)
     +              + f(ir)*ss(ir-1)
     +              + g(ir)*(ss(ir)-ss(ir-1))
      enddo
C
      endif                 ! if(mr.LE.nr)
C
      out = ifor(nr)        ! this is the emergent intensity
C
      return
      end
C

C*******************************************************************
      SUBROUTINE qromb(func,a,b,ss)
C*******************************************************************
      INTEGER JMAX,JMAXP,K,KM
      REAL*8 a,b,func,ss,EPS
      EXTERNAL func
      PARAMETER (EPS=1.e-4, JMAX=20, JMAXP=JMAX+1, K=5, KM=K-1)
CU    USES polint,trapzd
      INTEGER j
      REAL*8 dss,h(JMAXP),s(JMAXP)
      h(1)=1.
      do 11 j=1,JMAX
        call trapzd(func,a,b,s(j),j)
        if (j.ge.K) then
          call polint(h(j-KM),s(j-KM),K,0.,ss,dss)
          if (abs(dss).le.EPS*abs(ss)) return
        endif
        s(j+1)=s(j)
        h(j+1)=0.25*h(j)
11    continue
      pause 'too many steps in qromb'
      END
C
C*******************************************************************
      SUBROUTINE trapzd(func,a,b,s,n)
C*******************************************************************
      INTEGER n
      REAL*8 a,b,s,func
      EXTERNAL func
      INTEGER it,j
      REAL*8 del,sum,tnm,x
      if (n.eq.1) then
        s=0.5*(b-a)*(func(a)+func(b))
      else
        it=2**(n-2)
        tnm=it
        del=(b-a)/tnm
        x=a+0.5*del
        sum=0.
        do 11 j=1,it
          sum=sum+func(x)
          x=x+del
11      continue
        s=0.5*(s+(b-a)*sum/tnm)
      endif
      return
      END
C
C*******************************************************************
      SUBROUTINE polint(xa,ya,n,x,y,dy)
C*******************************************************************
      INTEGER n,NMAX
      REAL*8 dy,x,y,xa(n),ya(n)
      PARAMETER (NMAX=10)
      INTEGER i,m,ns
      REAL*8 den,dif,dift,ho,hp,w,c(NMAX),d(NMAX)
      ns=1
      dif=abs(x-xa(1))
      do 11 i=1,n
        dift=abs(x-xa(i))
        if (dift.lt.dif) then
          ns=i
          dif=dift
        endif
        c(i)=ya(i)
        d(i)=ya(i)
11    continue
      y=ya(ns)
      ns=ns-1
      do 13 m=1,n-1
        do 12 i=1,n-m
          ho=xa(i)-x
          hp=xa(i+m)-x
          w=c(i+1)-d(i)
          den=ho-hp
          if(den.eq.0.)pause 'failure in polint'
          den=w/den
          d(i)=hp*den
          c(i)=ho*den
12      continue
        if (2*ns.lt.n-m)then
          dy=c(ns+1)
        else
          dy=d(ns)
          ns=ns-1
        endif
        y=y+dy
13    continue
      return
      END
C
C*******************************************************************
      real*8 FUNCTION bi0exp(x)
C*******************************************************************
      REAL*8 x
      REAL*8 ax, result
      real*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9
      DATA p1,p2,p3,p4,p5,p6,p7/1.0d0,3.5156229d0,3.0899424d0,
     *1.2067492d0,0.2659732d0,0.360768d-1,0.45813d-2/
      DATA q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,0.1328592d-1,
     *0.225319d-2,-0.157565d-2,0.916281d-2,-0.2057706d-1,0.2635537d-1,
     *-0.1647633d-1,0.392377d-2/
      ax=abs(x)
      if (ax.lt.3.75) then
        y=(x/3.75)**2
        result=p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7)))))
        result = result*exp(-ax)
      else
        y=3.75/ax
        result=(1./sqrt(ax))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     +         (q7+y*(q8+y*q9))))))))
      endif
      bi0exp = result
      return
      END
C
C************************************************************************
      real*8 function h2denfun(rad)
C************************************************************************
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      result = h2den0 + h2den1*((rad/h2denradius)**h2denpower)
      h2denfun = result
      return
      end
C
C************************************************************************
      real*8 function Tgasfun(rad)
C************************************************************************
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      result = Tgas0 + Tgas1*((rad/Tgasradius)**Tgaspower)
      if(result.LT.Tgasmin) then
         result = Tgasmin
      endif
      Tgasfun = result
      return
      end
C
C************************************************************************
      real*8 function Tdustfun(rad)
C************************************************************************
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      result = Tdust0 + Tdust1*((rad/Tdustradius)**Tdustpower)
      if(result.LT.Tdustmin) then
         result=Tdustmin
      endif
      Tdustfun = result
      return
      end
C
C************************************************************************
      real*8 function abunfun(rad)
C************************************************************************
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      result = abun0 + abun1*((rad/abunradius)**abunpower)
      abunfun = result
      return
      end
C
C************************************************************************
      real*8 function columnfun(rad)
C************************************************************************
C
C     Calculates radial column density from r=rmin to r=rad
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      if(h2denppower .NE. -1.) then
         result = h2den0*(rad-rmin)
     +       + (h2den1/((h2denradius**h2denpower)*(h2denpower+1.)))
     +       *(rad**(h2denpower+1.) - rmin**(h2denpower+1.))
      else
         result = h2den0*(rad-rmin)
     +       + (h2den1*h2denradius)*dlog(rad/rmin)
      endif
C
      result = result*rscalecm     ! put distance scale in
                                   ! so column density is in cm-2
      columnfun = result
      return
      end
C
C
C************************************************************************
      subroutine shellcount(rad, sh, dsh)
C************************************************************************
C
C     Calculates continuous "shell count" function and derivative
C     Shell radii determined by solving sh = k, k=2, 3, ..., nshell-1
C     This one gives grids spaced logarithmically in col. density
C
      real*8 rad
      real*8 sh, dsh
C
      include 'grid.inc'
C
      real*8 colmax         ! column density at rmax
      real*8 shell          ! gets subtracted for root-finding
C
      real*8 alpham, lnalpha
C
      common/shellcnt/ colmax, shell, alpham, lnalpha
C
      real*8 columnfun
      external columnfun
C
      real*8 x1
C
      x1 = 1.-alpham*columnfun(rad)/colmax
      sh = 1. + log(x1)/lnalpha
      sh = sh - shell
      dsh = -alpham*h2denfun(rad)*rscalecm/colmax
     +      /(lnalpha*x1)
      return
      end
C
C************************************************************************
      subroutine calcgrid
C************************************************************************
C
C     Calculates a radial grid which has logarithmic column
C     density intervals
C
      real*8 ACC
      parameter (ACC=1.E-6)     ! accuracy for root-finder
C
      include 'grid.inc'
C
      integer ir
C
      real*8 h2denfun
      real*8 Tgasfun
      real*8 Tdustfun
      real*8 abunfun
      real*8 columnfun
C
      external h2denfun
      external Tgasfun
      external Tdustfun
      external abunfun
      external columnfun
C
      external shellcount
C
      real*8 rtsafe              ! Numerical Recipes root-finder
      external rtsafe
C
      real*8 colmax              ! column density at rmax
      real*8 shell               ! gets subtracted for root-finding
      real*8 alpham, lnalpha
C
      common/shellcnt/ colmax, shell, alpham, lnalpha
C
      colmax = columnfun(rmax)   ! get maximum column density
      lnalpha = log(thickratio)/(2.-nshell)
      alpham = 1. - exp(lnalpha*(nshell-1.))
C
      rshell(1) = rmin           ! first radius at rmin
      rshell(nshell) = rmax      ! last radius at rmax
C
      do 10 ir=2, nshell-1
         shell = ir              ! this is the shell we're solving for
         rshell(ir) = rtsafe(shellcount, rshell(ir-1), rmax, ACC)
10    continue
C
C     calculate physical parameters...
C
      do 20 ir=1, nshell
         h2den(ir) = h2denfun(rshell(ir))
         Tgas(ir) = Tgasfun(rshell(ir))
         Tdust(ir) = Tdustfun(rshell(ir))
         abun(ir) = abunfun(rshell(ir))
         Ncol(ir) = columnfun(rshell(ir))
20    continue
C
      return
      end
C
C
C     Numerical Recipes routines...
C
C************************************************************************
      REAL*8 FUNCTION rtsafe(funcd,x1,x2,xacc)
C************************************************************************
      INTEGER MAXIT
      REAL*8 result,x1,x2,xacc
      EXTERNAL funcd
      PARAMETER (MAXIT=100)
      INTEGER j
      REAL*8 df,dx,dxold,f,fh,fl,temp,xh,xl
C
      call funcd(x1,fl,df)
      call funcd(x2,fh,df)
      if((fl.gt.0..and.fh.gt.0.).or.(fl.lt.0..and.fh.lt.0.))pause
     *'root must be bracketed in rtsafe'
      if(fl.eq.0.)then
        rtsafe=x1
        return
      else if(fh.eq.0.)then
        rtsafe=x2
        return
      else if(fl.lt.0.)then
        xl=x1
        xh=x2
      else
        xh=x1
        xl=x2
      endif
      result=.5*(x1+x2)
      dxold=abs(x2-x1)
      dx=dxold
      call funcd(result,f,df)
      do 11 j=1,MAXIT
        if(((result-xh)*df-f)*((result-xl)*df-f).ge.0..or. abs(2.*
     *f).gt.abs(dxold*df) ) then
          dxold=dx
          dx=0.5*(xh-xl)
          result=xl+dx
          if(xl.eq.result) then
             rtsafe = result
             return
          endif
        else
          dxold=dx
          dx=f/df
          temp=result
          result=result-dx
          if(temp.eq.result) then
             rtsafe = result
             return
          endif
        endif
        if(abs(dx).lt.xacc) then
           rtsafe = result
           return
        endif
        call funcd(result,f,df)
        if(f.lt.0.) then
          xl=result
        else
          xh=result
        endif
11    continue
      pause 'rtsafe exceeding maximum iterations'
      return
      END
