C
C********************************************************************
      program lte_sphere2
C********************************************************************
C
C     J. Zmuidzinas   5-9-95
C
C     calculate lte line intensities for uniform sphere
C********************************************************************
C
      include 'spectrum.inc'
      include 'obs2.inc'
      include 'cgs_constants.inc'
C
      real*8 srcfun
      real*8 fpop(MAXLEV)
C
      real*8 radmin, radmax, radstep
C
      real*8 Tx, rad, xfrac, linewidth
      real*8 x1, wb

      real*8 tau(MAXOBS), den(MAXOBS), colden(MAXOBS), bmcol(MAXOBS)
      real*8 Qpart, wavelen
C
      character*80 specfile, obsfile, outfile
      integer itran, levi, levf, ilevel
      integer iobs
C
      real*8 efn
      external efn
C
      real*8 parsec
      parameter(parsec=3.0856E18)      ! in cm
C
      write(*,*) ' Molecular transition data file ? '
      read(*,10) specfile
10    format(A)
      write(*,*) ' File giving observation list ? '
      read(*,10) obsfile
      write(*,*) ' Output file ? '
      read(*,10) outfile
C
C
      write(*,*) ' Radius needed to calculate beam filling factors'
      write(*,*) ' and column density (given n, X) '
      write(*,*) ' Radius min, max, step (pc)'
      read(*,*) radmin, radmax, radstep
C
      write(*,*) ' Tx needed to calculate LTE level populations'
      write(*,*) ' Excitation temperature (K)'
      read(*,*) Tx
      write(*,*) ' Fractional abundance ? '
      read(*,*) xfrac
      write(*,*) ' Line FWHM (km/s; for optical depth) ? '
      read(*,*) linewidth
C
      call getspec(specfile)
      call getobserv(obsfile)
C
      open(UNIT=3, FILE=outfile, status='UNKNOWN')
!      write(3,10) specfile
!      write(3,10) obsfile
C
      do 15 iobs=1, numobs
         itran=transnum(iobs)
         levi = linitial(itran)
         levf = lfinal(itran)
         write(3,16) itran,
     +               elabel(levi), elabel(levf),
     +               frequency(itran), ta(iobs)
16        format(I5, 1X, A10,'-',A10, 4x, F11.2, ' Ta = ', F11.3)
15    continue
C
      write(3,*) ' Radius: ', radmin, radmax, radstep, ' (pc)'
      write(3,*) ' Tx : ', Tx, ' (K)'
      write(3,*) ' Frac abundance : ', xfrac,
     +           '   Linewidth : ', linewidth, '( km/s)'
      write(3,*)
      write(3,*) ' Radius, (optical depth, density, column density,',
     +           ' beam avg col. den.):'
      write(3,*)
C
C
C     sum partition function
C
            Qpart = 0.
            do 30 ilevel=1, numlev
               Qpart = Qpart + degen(ilevel)*exp(-energy(ilevel)/Tx)
30          continue
C
C     get fractional populations
C
            do 40 ilevel=1, numlev
               fpop(ilevel) = degen(ilevel)*exp(-energy(ilevel)/Tx)
     +                        /Qpart
40          continue
C
C
      rad=radmin
20    continue
C
C     calculate LTE column densities, optical depths, etc.
C
            do 50 iobs=1, numobs
               itran=transnum(iobs)
               wavelen = clight/(frequency(itran)*amegahz)
               levi = linitial(itran)
               levf = lfinal(itran)
               x1 = exp(tphoton(itran)/Tx)-1.
               srcfun = tphoton(itran)/x1
               wb = beamsize(iobs)/(2.*sqrt(log(2.)))
               tau(iobs) = ta(iobs)/(srcfun*efn((rad/wb)**2))
               if(tau(iobs).GE.1) then
                  tau(iobs) = -1.
               else if(tau(iobs).LT.0.) then
                  tau(iobs) = -1.
               else
                  tau(iobs) = -log(1.-tau(iobs))
               endif
               colden(iobs) = tau(iobs)*8*PI*akm*linewidth/
     +                  ((wavelen**3)*acoeff(itran)
     +                   *fpop(levi)*x1)
C
C     volume density
C
               den(iobs) = colden(iobs)/(2.*rad*parsec*xfrac)
               colden(iobs) = colden(iobs)/xfrac
               bmcol(iobs) = colden(iobs)*efn((rad/wb)**2)
               if(den(iobs).LE.0.) then
                  tau(iobs) = 1.e-20
                  den(iobs) = 1.e-20
                  colden(iobs) = 1.e-20
                  bmcol(iobs) = 1.e-20
               endif
50          continue
C
            write(3,55) rad, 
     +                  (tau(iobs), den(iobs), colden(iobs),
     +                  bmcol(iobs), iobs=1, numobs) 
55          format(1X, E11.4, 48(1X, E11.4, :))
C
C     Loop over radius...
C
        rad = rad+radstep
      if(rad.LE.radmax) go to 20
      close(3)
      stop
      end
C
C********************************************************************
      subroutine getspec(infile)
C********************************************************************
C
C     read energy levels, A coefficients from disk
C
      include 'spectrum.inc'
      include 'cgs_constants.inc'
C
      character*(*) infile
      character*120 textline
      integer i, j, li, lf
      real*4 x
C
1     format(A)
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
      read(1,*)                 ! skip comment line
      read(1,*) numlev
      if(numlev.GT.MAXLEV) then
         write(*,*)
     +     ' Too many energy levels in molecular transitions file'
         write(*,*) ' Number of levels = ', numlev
         write(*,*) ' You may need to recompile with larger MAXLEV'
         close(1)
         stop
      endif
C
      read(1,*)                 ! skip comment line
      do 20 i=1, numlev
         read(1,1) elabel(i)
20    continue
C
      read(1,*)                 ! skip comment line
C
      do 30 i=1, numlev
         read(1,1) textline
         read(textline,*) j, energy(i), degen(i)
30    continue
C
      read(1,*)                 ! skip comment line
C
C
C     Clear A-coefficient matrix
C
      do 50 i=1, MAXTRAN
         acoeff(i) = 0.
50    continue
C
      numtran = 0                            ! no transitions yet
C
60    continue
         read(1,1,END=70) textline
         read(textline, *) li, lf, x

         if(li.LT.0 .OR. li.GT.numlev .OR.    ! subscripts out of bounds
     +      lf.LT.0 .OR. lf.GT.numlev) then
C
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Bad energy level number:'
            write(*,1) textline
            close(1)
            stop
         endif
C
         if(energy(li).LE.energy(lf)) then
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Cannot have spontaneous upwards transitions'
            write(*,1) textline
            close(1)
            stop
         endif
C
         numtran = numtran + 1     
         acoeff(numtran) = x         ! looks OK, so stuff it
         linitial(numtran) = li
         lfinal(numtran) = lf
         x = energy(li) - energy(lf)
         tphoton(numtran) = x            ! photon temperature in K
         x = x*kboltz/(hplanck*amegahz)  ! convert to MHz
         frequency(numtran) = x
C
      if(numtran .LT. MAXTRAN) goto 60   ! process next transition
      write(*,*) ' Warning: may have too many transitions in file'
      write(*,*) ' Maximum number is ', MAXTRAN
C
70    continue
C
C     now done reading transitions...
C
      close(1)
C
      return
      end
C
C************************************************************************
      subroutine getobserv(fname)
C************************************************************************
C
      character*(*) fname
      character*100 textline
C
      include 'spectrum.inc'
      include 'obs2.inc'
      include 'cgs_constants.inc'
C
      integer itran, li, lf
      real*8 bw, bo
C
      open(UNIT=1, NAME=fname, STATUS='UNKNOWN')
C
10    format(A)
C
      numobs = 0
20    continue
         read(1,10, END=50) textline
         read(textline,*) li, lf, bw, bo
         do 30 itran=1,numtran                 ! see if initial, final
            if(li.EQ.linitial(itran) .AND.     ! levels match up with a
     +         lf.EQ.lfinal(itran)) then       ! known transition
                  numobs = numobs+1
                  transnum(numobs) = itran     ! yes, it's a good one...
                  beamsize(numobs) = bw
                  ta(numobs) = bo
                  goto 40
            endif
30       continue
         write(*,*)
     +         ' WARNING - skipping unknown transition:  ',
     +         textline
40       continue
      if(numobs.LT.MAXOBS) then
         goto 20
      endif
50    continue
C
      close(1)
      return
      end
C
C
C********************************************************************
      real*8 function efn(x)
C********************************************************************
C
         real*8 x
C
         real*8 cut, result
         parameter(cut=1.e-2)
C
         if(abs(x).LE.cut) then
            result = x*(1.-x/2*(1.-x/3.*(1.-x/4.*(1.-x/5.
     +                *(1.-x/6.*(1.-x/7.*(1.-x/8.*(1.-x/9.
     +                *(1.-x/10.)))))))))
         else
            result = 1.-exp(-x)
         endif
C
         efn=result
C
         return
         end
C********************************************************************
