C**********************************************************************
      subroutine getcollrate
C**********************************************************************
C
C     reads collisional rate table from disk
C
      include 'collrate.inc'
C
      character*80 collfile
      character*200 textline
      integer i, levi, levf
C
      write(*,*) ' Collisional excitation rates file ? '
      read(*,10) collfile
10    format(A)
C
      open(UNIT=1, FILE=collfile, STATUS='OLD')
C
C     zero out collisional rates
C
      do 40 i=1, MAXT
         tcoll(i) = 0.
         do 30 levi=1, MAXL
            do 20 levf=1, MAXL
               collrate(i, levi, levf) = 0.
20          continue
30       continue
40    continue
C
      read(1,*) numtcoll
      if(numtcoll .GT. MAXT) then
         write(*,*) ' Too many temperatures for collisional rates'
         write(*,*) ' Number of Temps = ', numtcoll
         write(*,*) ' Recompile with larger MAXT if necessary'
         close(1)
         stop
      endif
C
      do 50 i=1, numtcoll
         read(1,*) tcoll(i)
50    continue
C
      ncollev = 0
60    continue
C
C
         read(1,10, END=70) textline
         read(textline,*) levi, levf
         if(ncollev.LT.levi)  ncollev = levi
         if(ncollev.LT.levf)  ncollev = levf
         if(ncollev.GT.MAXL) then
            write(*,*)
     +         ' Too many energy levels in collisional rates file'
            write(*,*) ' Number of levels = ', ncollev
            write(*,*) ' Encountered in rate file on line '
            write(*,*) '*** ', textline, ' ***'
            write(*,*) ' Recompile with larger MAXL if necessary'
            close(1)
            stop
         endif
         if(levi.LT.1 .OR. levf.LT.1) then
            write(*,*) ' Error in collisional rate file'
            write(*,*) ' Energy level labels must be >= 1'
            write(*,*) '*** ', textline, ' ***'
            close(1)
            stop
         endif
C
C        All seems to be OK, so read the rates
C
         read(textline, *)
     +        levi, levf, (collrate(i,levi,levf), i=1, numtcoll)
      goto 60
C
70    continue
      close(1)
      return
      end
C
C**********************************************************************
      real*4 function collratecalc(li, lf, temp)
C**********************************************************************
C
C     interpolates collisional rate table
C
C     li = initial level
C     lf = final level
C     temp = temperature
C
      integer li, lf
      real*4 temp
C
      include 'collrate.inc'
C
      integer indx
      real result, x, x1, x2, y1, y2
C
      if(li.GT.ncollev .OR. lf.GT.ncollev) then
         result = 0.0
         goto 900
      endif
C
      if(temp.LE.tcoll(1)) then
         result = collrate(1, li, lf)
         goto 900
      endif
C
      if(temp.GE.tcoll(numtcoll)) then
         result = collrate(numtcoll, li, lf)
         goto 900
      endif
C
      indx = 1
10    continue
         indx = indx+1
      if(temp.GT.tcoll(indx) .AND. indx.LT.numtcoll) goto 10
C
C     now do a power law interpolation
C
      x = log(temp)
      x1 = log(tcoll(indx-1))
      x2 = log(tcoll(indx))
      y1 = log(collrate(indx-1, li, lf))
      y2 = log(collrate(indx, li, lf))
C
      result = (y1*(x2-x) + y2*(x-x1))/(x2-x1)
      result = exp(result)
C
900   continue
      collratecalc = result
      return
      end
C
