C************************************************************************
      program levpop
C************************************************************************
C
C     J. Zmuidzinas  1-4-94
C
C     creates a file of radius vs. level population for plotting
C
C
C************************************************************************
C
      character*80 gridfile, specfile, collfile, dustfile,
     +                 molndfile, outfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 molndfile, outfile
C
      integer levelnum
C
      write(*,*) ' Name of file created by molnd ? '
10    format(A)
      read(*,10) molndfile
      write(*,*) ' Energy level number ? '
      read(*,*) levelnum
      write(*,*) ' Output file ? '
      read(*,10) outfile
C
      call process(molndfile, outfile, levelnum)  
C
      stop
      end
C
C************************************************************************
      subroutine process(ifname, ofname, level)
C************************************************************************
C
      character*(*) ifname, ofname
      integer level
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer_obs.inc'
      include 'cgs_constants.inc'
C
      character*80 gridfile, specfile, collfile, dustfile,
     +                 molndfile, outfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 molndfile, outfile
      character*80 textline
C
      integer ir, ilev, ii
C
      open(UNIT=3, NAME=ifname, STATUS='UNKNOWN')
C
      read(3,10) gridfile
10    format(A)
      read(3,10) specfile
      read(3,10) collfile
      read(3,10) dustfile
      read(3,10) textline
C
      call getgrid(gridfile)                ! read in radial grid
      call getspec(specfile)                ! read in energy levels, etc.
C
      read(3,10) textline                   ! skip nshell, numlev...
      do 100 ir=1, nshell
         read(3,10) textline                ! skip blank line
         read(3,10) textline                ! skip rshell(ir)
         do 70 ilev=1, numlev
            read(3,10) textline
            read(textline,*) ii, fracpop(ir, ilev)
70       continue
100   continue
C
      close(3)
C
      open(UNIT=3, NAME=ofname, STATUS='UNKNOWN')
      write(3,10) ifname
      write(3,*) level, ' ', elabel(level)
      do 200 ir=1, nshell
         write(3,*) rshell(ir)*rmax, fracpop(ir, level)
200   continue
      close(3)
      return
      end
C
C************************************************************************
      subroutine getgrid(infile)
C************************************************************************
C
      include 'grid.inc'
C
      character*(*) infile
      integer ir
C
      open(UNIT=1, FILE=infile, STATUS='UNKNOWN')
C
      read(1,*)
      read(1,*) igridver               ! grid version number
      if(igridver.NE.2) then           ! not version 2 like we need
         write(*,*) ' Incorrect version number in radial grid file',
     +              infile
         write(*,*) ' Need version 2; got version no.', igridver
         stop
      endif
C
      read(1,*)
      read(1,*)                        ! skip grid parameters file name
C
      read(1,*)
      read(1,*) rscalecm
      read(1,*) rmin, rmax
      read(1,*) nshell
      read(1,*) linewidth               ! this is FWHM in km/s
C
C     calculate 1/e half-width of line
C
      ewidth = linewidth/(2.*sqrt(log(2.)))
C
      read(1,*) numvel
      read(1,*)  h2den0, h2den1, h2denradius, h2denpower
      read(1,*)  Tgas0, Tgas1, Tgasradius, Tgaspower
      read(1,*)  Tdust0, Tdust1, Tdustradius, Tdustpower
      read(1,*)  abun0, abun1, abunradius, abunpower
      read(1,*)
      read(1,*)
C
C     Read in parameters as a function of radius
C     Note that radius is not normalized to 1 yet
C
      do 200 ir=1, nshell
         read(1,*)  rshell(ir), h2den(ir),
     +              Tgas(ir), Tdust(ir), abun(ir), Ncol(ir)
200   continue
C
      close(1)
C
C     Normalize shell radii now
C
      do 300 ir=1, nshell
         rshell(ir) = rshell(ir)/rmax
300   continue
C
      return
      end
C
C********************************************************************
      subroutine getspec(infile)
C********************************************************************
C
C     read energy levels, A coefficients from disk
C
      include 'spectrum.inc'
      include 'cgs_constants.inc'
C
      character*(*) infile
      character*120 textline
      integer i, j, li, lf
      real*4 x
C
1     format(A)
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
      read(1,*)                 ! skip comment line
      read(1,*) numlev
      if(numlev.GT.MAXLEV) then
         write(*,*)
     +     ' Too many energy levels in molecular transitions file'
         write(*,*) ' Number of levels = ', numlev
         write(*,*) ' You may need to recompile with larger MAXLEV'
         close(1)
         stop
      endif
C
      read(1,*)                 ! skip comment line
      do 20 i=1, numlev
         read(1,1) elabel(i)
20    continue
C
      read(1,*)                 ! skip comment line
C
      do 30 i=1, numlev
         read(1,1) textline
         read(textline,*) j, energy(i), degen(i)
30    continue
C
      read(1,*)                 ! skip comment line
C
C     Clear A-coefficient matrix
C
      do 50 i=1, MAXTRAN
         acoeff(i) = 0.
50    continue
C
      numtran = 0                            ! no transitions yet
C
60    continue
         read(1,1,END=70) textline
         read(textline, *) li, lf, x

         if(li.LT.0 .OR. li.GT.numlev .OR.    ! subscripts out of bounds
     +      lf.LT.0 .OR. lf.GT.numlev) then
C
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Bad energy level number:'
            write(*,1) textline
            close(1)
            stop
         endif
C
         if(energy(li).LE.energy(lf)) then
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Cannot have spontaneous upwards transitions'
            write(*,1) textline
            close(1)
            stop
         endif
C
         numtran = numtran + 1     
         acoeff(numtran) = x         ! looks OK, so stuff it
         linitial(numtran) = li
         lfinal(numtran) = lf
         x = energy(li) - energy(lf)
         tphoton(numtran) = x            ! photon temperature in K
         x = x*kboltz/(hplanck*amegahz)  ! convert to MHz
         frequency(numtran) = x
C
      if(numtran .LT. MAXTRAN) goto 60   ! process next transition
      write(*,*) ' Warning: may have too many transitions in file'
      write(*,*) ' Maximum number is ', MAXTRAN
C
70    continue
C
C     now done reading transitions...
C
      close(1)
C
C     so sort them by ascending frequency, including
C     the initial and final state indices
C
C     call PIKSR3(numtran, frequency, linitial, lfinal)
C
      return
      end
C
C********************************************************************
      SUBROUTINE PIKSR3(N,ARR,BRR,CRR)
C********************************************************************
      REAL*8 ARR(N), A
      INTEGER BRR(N), CRR(N), B, C
      DO 12 J=2,N
        A=ARR(J)
        B=BRR(J)
        C=CRR(J)
        DO 11 I=J-1,1,-1
          IF(ARR(I).LE.A)GO TO 10
          ARR(I+1)=ARR(I)
          BRR(I+1)=BRR(I)
          CRR(I+1)=CRR(I)
11      CONTINUE
        I=0
10      ARR(I+1)=A
        BRR(I+1)=B
        CRR(I+1)=C
12    CONTINUE
      RETURN
      END
C
