C     On ALPHA  (medoc):
C     f77 -fast -fpe1 -o molnd_heat molnd_heat.f
C
C************************************************************************
      program molnd_heat
C************************************************************************
C
C     J. Zmuidzinas  1-1-94
C
C     Radiative transfer and statistical equilibrium program
C     for arbitrary spherical distributions of gas and dust
C
C     Version 1 - use first-order radiative transfer solver()
C                 instead of 2nd-order Feautrier algorithm.
C
C     _heat version: calculate net cooling rate vs. radius
C
C************************************************************************
C
      character*80 gridfile, specfile, collfile, dustfile,
     +             savefile, molndfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 savefile, molndfile
C
      character*1 ans
C
      real*8 cnveps
      integer maxiter, accon
      common/convcrit/cnveps, maxiter, accon 
C
      integer moreiter
      logical firstime
C
      real*8 abunfac, dustfac
      common/xfact/abunfac, dustfac
C
      write(*,*) ' Start new (N) or continue old (O) calculation ?'
      read(*,10) ans
10    format(A)
C
      if(ans .EQ. 'N' .OR. ans .EQ. 'n') then
         write(*,*) ' Starting NEW calculation...'
         write(*,*) ' Input file name for radial grid ? '
         read(*,10) gridfile
         write(*,*) ' Molecular transition data file ? '
         read(*,10) specfile
         write(*,*) ' Collisional excitation rates file ? '
         read(*,10) collfile
         write(*,*) ' Dust parameters file ? '
         read(*,10) dustfile
C
         write(*,*) ' Molecular abundance factor (normally 1.0) ? '
         read(*,*) abunfac
         write(*,*) ' Dust absorption factor (normally 1.0) ? '
         read(*,*) dustfac
C
         write(*,*) ' Output file ? '          ! get file name before
         read(*,10) savefile                   ! grinding through
C
         call getgrid(gridfile)                ! read in radial grid
         call getspec(specfile)                ! read in energy levels, etc.
         call getcollrate(collfile)            ! collision rates
         call getdust(dustfile)                ! dust parameters
         firstime = .TRUE.                     ! first set of iterations
C
      else
         write(*,*) ' Continuing OLD calculation...'
         write(*,*) ' Name of file written by molnd ? '
         read(*,10) molndfile
         write(*,*) ' Output file ? '          ! get file name before
         read(*,10) savefile                   ! grinding through
C
         write(*,*) ' Molecular abundance factor (normally 1.0) ? '
         read(*,*) abunfac
         write(*,*) ' Dust absorption factor (normally 1.0) ? '
         read(*,*) dustfac
C
         call getmolnd(molndfile)              ! read level populations
         firstime = .FALSE.                    ! continuing old calculation
      endif
C
C
20    continue
C
      write(*,*)
     +  ' Use accelerated (1) or normal (0) lambda-iteration ?'
      read(*,*) accon
      write(*,*) ' Relative fractional population change '
      write(*,*) ' threshold for convergence (i.e. 1.E-3) ? '
      read(*,*) cnveps
      write(*,*) ' Maximum number of iterations (i.e. 30) ? '
      read(*,*) maxiter
C
      call selfconsistent(firstime)          ! iterate level populations
C                                          ! and radiative transfer
      firstime = .FALSE.
      write(*,*) ' Continue iterating (1) or quit (0) ? '
      read(*,*) moreiter
      if(moreiter.NE.0) goto 20
C
      call saveresults(savefile)           ! write results to disk
      stop
      end
C
C
C************************************************************************
      subroutine getmolnd(fname)
C************************************************************************
C
      character*(*) fname
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
      include 'cgs_constants.inc'
C
      character*80 gridfile, specfile, collfile, dustfile,
     +             savefile, molndfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 savefile, molndfile
C
      character*80 textline
C
      integer ir, ilev, ii
C
      open(UNIT=3, NAME=fname, STATUS='UNKNOWN')
C
      read(3,10) gridfile
10    format(A)
      read(3,10) specfile
      read(3,10) collfile
      read(3,10) dustfile
      read(3,10) textline
C
      call getgrid(gridfile)                ! read in radial grid
      call getspec(specfile)                ! read in energy levels, etc.
      call getcollrate(collfile)            ! collision rates
      call getdust(dustfile)                ! dust parameters
C
      read(3,10) textline                   ! skip nshell, numlev...
      do 100 ir=1, nshell
         read(3,10) textline                ! skip blank line
         read(3,10) textline                ! skip rshell(ir)
         do 70 ilev=1, numlev
            read(3,10) textline
            read(textline,*) ii, fracpop(ir, ilev)
70       continue
100   continue
C
      close(3)
      return
      end
C
C************************************************************************
      subroutine saveresults(fname)
C************************************************************************
C
      character*(*) fname
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
      include 'cgs_constants.inc'
C
      character*80 gridfile, specfile, collfile, dustfile,
     +             savefile, molndfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 savefile, molndfile
C
      integer ir, ilev, itran, li, lf
C
      real*8 abunfac, dustfac
      common/xfact/abunfac, dustfac
C
      real*8 lumcool, lumheat, vol, diff
      real*8 sumcool, sumheat
C
      open(UNIT=2, NAME=fname, STATUS='UNKNOWN')
      open(UNIT=4, NAME='heatncool.dat', STATUS='UNKNOWN')
C
      write(2,10) gridfile
10    format(A)
      write(2,10) specfile
      write(2,10) collfile
      write(2,10) dustfile
      write(2,10) savefile
C
      write(4,10) gridfile
      write(4,10) specfile
      write(4,10) collfile
      write(4,10) dustfile
      write(4,10) savefile
C
      write(2,*) nshell, numlev, abunfac, dustfac
      write(4,*) nshell, numlev, abunfac, dustfac
      write(4,*) ' Heating, cooling in ergs/(s cm**3)'
      write(4,*) ' Lheat, Lcool are in solar luminosities',
     +           ' per radial shell'
      write(4,*)
     +          '  Radius       Cooling     Heating      ',
     +          ' Lcool        Lheat       Lcool-Lheat'
C
      sumheat = 0.
      sumcool = 0.
C
      do 100 ir=1, nshell
         write(2,*)
         write(2,*) rshell(ir)
         do 70 ilev=1, numlev
            write(2,65) ilev, fracpop(ir, ilev), elabel(ilev)
65          format(I5, 1X, E12.5, 3X, A10)
70       continue
C
C        ***also write heating & cooling
C           put into units of ergs/(s cm**3)
C
         heat(ir) = heat(ir)*kboltz*abun(ir)*abunfac
         cool(ir) = cool(ir)*kboltz*abun(ir)*abunfac
C
C        Now calculate total heating & cooling for this shell,
C        in solar luminosities
C
         if(ir.EQ.1) then
            vol = 4*PI*(rshell(1)**3.)/3.
         else
            vol = 4*PI*(rshell(ir)**3.-rshell(ir-1)**3.)/3.
         endif
         vol = vol*((rmax*rscalecm)**3.)
         lumheat = heat(ir)*vol/lsolar
         lumcool = cool(ir)*vol/lsolar
         diff = lumcool-lumheat
         sumheat = sumheat + lumheat
         sumcool = sumcool + lumcool
C        
         write(4,75) rshell(ir)*rmax, cool(ir), heat(ir),
     +               lumcool, lumheat, diff
75       format(6(2X, E11.4))
100   continue
C
      diff = sumcool - sumheat
      write(4,105) sumcool, sumheat, diff
105   format(' Totals for cloud:', 21X, 3(2X, E11.4))
C
      close(2)
      close(4)
      return
      end
C
C************************************************************************
      subroutine getgrid(infile)
C************************************************************************
C
      include 'grid.inc'
C
      character*(*) infile
      integer ir
C
      open(UNIT=1, FILE=infile, STATUS='UNKNOWN')
C
      read(1,*)
      read(1,*) igridver               ! grid version number
      if(igridver.NE.2) then           ! not version 2 like we need
         write(*,*) ' Incorrect version number in radial grid file',
     +              infile
         write(*,*) ' Need version 2; got version no.', igridver
         stop
      endif
C
      read(1,*)
      read(1,*)                        ! skip grid parameters file name
C
      read(1,*)
      read(1,*) rscalecm
      read(1,*) rmin, rmax
      read(1,*) nshell
      read(1,*) linewidth               ! this is FWHM in km/s
C
C     calculate 1/e half-width of line
C
      ewidth = linewidth/(2.*sqrt(log(2.)))
C
      read(1,*) numvel
      read(1,*)  h2den0, h2den1, h2denradius, h2denpower
      read(1,*)  Tgas0, Tgas1, Tgasradius, Tgaspower
      read(1,*)  Tdust0, Tdust1, Tdustradius, Tdustpower
      read(1,*)  abun0, abun1, abunradius, abunpower
      read(1,*)
      read(1,*)
C
C     Read in parameters as a function of radius
C     Note that radius is not normalized to 1 yet
C
      do 200 ir=1, nshell
         read(1,*)  rshell(ir), h2den(ir),
     +              Tgas(ir), Tdust(ir), abun(ir), Ncol(ir)
200   continue
C
      close(1)
C
C     Normalize shell radii now
C
      do 300 ir=1, nshell
         rshell(ir) = rshell(ir)/rmax
300   continue
C
      return
      end
C
C
C**********************************************************************
      subroutine getcollrate(infile)
C**********************************************************************
C
C     reads collisional rate table from disk
C
      include 'collrat8.inc'
C
      character*(*) infile
      character*200 textline
      integer i, levi, levf
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
C     zero out collisional rates
C
      do 40 i=1, MAXT
         tcoll(i) = 0.
         do 30 levi=1, MAXL
            do 20 levf=1, MAXL
               collrate(i, levi, levf) = 0.
20          continue
30       continue
40    continue
C
      read(1,*) numtcoll
      if(numtcoll .GT. MAXT) then
         write(*,*) ' Too many temperatures for collisional rates'
         write(*,*) ' Number of Temps = ', numtcoll
         write(*,*) ' Recompile with larger MAXT if necessary'
         close(1)
         stop
      endif
C
      do 50 i=1, numtcoll
         read(1,*) tcoll(i)
50    continue
C
      ncollev = 0
60    continue
C
C
         read(1,62, END=70) textline
62       format(A)
         read(textline,*) levi, levf
         if(ncollev.LT.levi)  ncollev = levi
         if(ncollev.LT.levf)  ncollev = levf
         if(ncollev.GT.MAXL) then
            write(*,*)
     +         ' Too many energy levels in collisional rates file'
            write(*,*) ' Number of levels = ', ncollev
            write(*,*) ' Encountered in rate file on line '
            write(*,*) '*** ', textline, ' ***'
            write(*,*) ' Recompile with larger MAXL if necessary'
            close(1)
            stop
         endif
         if(levi.LT.1 .OR. levf.LT.1) then
            write(*,*) ' Error in collisional rate file'
            write(*,*) ' Energy level labels must be >= 1'
            write(*,*) '*** ', textline, ' ***'
            close(1)
            stop
         endif
C
C        All seems to be OK, so read the rates
C
         read(textline, *)
     +        levi, levf, (collrate(i,levi,levf), i=1, numtcoll)
      goto 60
C
70    continue
      close(1)
      return
      end
C
C**********************************************************************
      real*8 function collratecalc(li, lf, temp)
C**********************************************************************
C
C     interpolates collisional rate table
C
C     li = initial level
C     lf = final level
C     temp = temperature
C
      integer li, lf
      real*8 temp
C
      include 'collrat8.inc'
C
      integer indx
      real*8 result, x, x1, x2, y1, y2
C
      if(li.GT.ncollev .OR. lf.GT.ncollev) then
         result = 0.0
         goto 900
      endif
C
      if(temp.LE.tcoll(1)) then
         result = collrate(1, li, lf)
         goto 900
      endif
C
      if(temp.GE.tcoll(numtcoll)) then
         result = collrate(numtcoll, li, lf)
         goto 900
      endif
C
      indx = 1
10    continue
         indx = indx+1
      if(temp.GT.tcoll(indx) .AND. indx.LT.numtcoll) goto 10
C
C     now do a power law interpolation
C
      x = log(temp)
      x1 = log(tcoll(indx-1))
      x2 = log(tcoll(indx))
      y1 = log(collrate(indx-1, li, lf))
      y2 = log(collrate(indx, li, lf))
C
      result = (y1*(x2-x) + y2*(x-x1))/(x2-x1)
      result = exp(result)
C
900   continue
      collratecalc = result
      return
      end
C
C********************************************************************
      subroutine getspec(infile)
C********************************************************************
C
C     read energy levels, A coefficients from disk
C
      include 'spectrum.inc'
      include 'cgs_constants.inc'
C
      character*(*) infile
      character*120 textline
      integer i, j, li, lf
      real*8 x
C
1     format(A)
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
      read(1,*)                 ! skip comment line
      read(1,*) numlev
      if(numlev.GT.MAXLEV) then
         write(*,*)
     +     ' Too many energy levels in molecular transitions file'
         write(*,*) ' Number of levels = ', numlev
         write(*,*) ' You may need to recompile with larger MAXLEV'
         close(1)
         stop
      endif
C
      read(1,*)                 ! skip comment line
      do 20 i=1, numlev
         read(1,1) elabel(i)
20    continue
C
      read(1,*)                 ! skip comment line
C
      do 30 i=1, numlev
         read(1,1) textline
         read(textline,*) j, energy(i), degen(i)
30    continue
C
      read(1,*)                 ! skip comment line
C
C
C     Clear A-coefficient matrix
C
      do 50 i=1, MAXTRAN
         acoeff(i) = 0.
50    continue
C
      numtran = 0                            ! no transitions yet
C
60    continue
         read(1,1,END=70) textline
         read(textline, *) li, lf, x

         if(li.LT.0 .OR. li.GT.numlev .OR.    ! subscripts out of bounds
     +      lf.LT.0 .OR. lf.GT.numlev) then
C
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Bad energy level number:'
            write(*,1) textline
            close(1)
            stop
         endif
C
         if(energy(li).LE.energy(lf)) then
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Cannot have spontaneous upwards transitions'
            write(*,1) textline
            close(1)
            stop
         endif
C
         numtran = numtran + 1     
         acoeff(numtran) = x         ! looks OK, so stuff it
         linitial(numtran) = li
         lfinal(numtran) = lf
         x = energy(li) - energy(lf)
         tphoton(numtran) = x            ! photon temperature in K
         x = x*kboltz/(hplanck*amegahz)  ! convert to MHz
         frequency(numtran) = x
C
      if(numtran .LT. MAXTRAN) goto 60   ! process next transition
      write(*,*) ' Warning: may have too many transitions in file'
      write(*,*) ' Maximum number is ', MAXTRAN
C
70    continue
C
C     now done reading transitions...
C
      close(1)
C
      return
      end
C
C**********************************************************************
      subroutine getdust(infile)
C**********************************************************************
C
C     reads dust parameters from disk
C
      include 'dust.inc'
C
      character*80 infile
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
      read(1,*) Qdust0
      read(1,*) wavdust0
      read(1,*) slopedust
      read(1,*) adust
      read(1,*) rhodust
      read(1,*) Rgasdust
      read(1,*) gasmu
      close(1)
      return
      end
C
C**********************************************************************
      real*8 function sigmadust(wavlen)
C**********************************************************************
C
C     Calculates dust cross-section (in cm-2) per H2 molecule
C
      real*8 wavlen                     ! wavelength in microns
C
      include 'dust.inc'
      include 'cgs_constants.inc'       ! for amicron = 1.E-4
C
      real*8 Qdust, result
C
      Qdust = Qdust0*(wavdust0/wavlen)**slopedust    ! efficiency
      result = 3*gasmu*Qdust/(4.*adust*amicron*rhodust*Rgasdust)
      sigmadust = result
      return
      end
C
C
C********************************************************************
      subroutine selfconsistent(first)
C********************************************************************
C
C     Use accelerated lambda iteration method of
C     Rybicki and Hummer, A+A 245, 171  (1991)
C     to simultaneously solve for statistical equilibrium and
C     radiative transfer.
C
C********************************************************************
C
      logical done, first
      integer itercount
      real*8 cnveps
      integer maxiter, accon
      common/convcrit/cnveps, maxiter, accon
C
      call initialize
      done = .FALSE.                ! convergence flag
      itercount = 0
C
10    continue
         if(.NOT.first) then        ! don't do transfer first time
            write(*,*) ' Solving transfer for iteration #', itercount
            call solvetransfer(accon)
         endif
C
         write(*,*) ' Solving rates for iteration #', itercount
         call solverates            ! solve for fractional populations
C
         if(.NOT.first) then        ! don't check convergence 1st time
            call checkconvergence(done)
         else
            first = .FALSE.
         endif
C
         itercount = itercount+1
      if(.NOT.done .AND. itercount.LT.maxiter) goto 10
C
      return
      end
C
C********************************************************************
      subroutine initialize
C********************************************************************
C
C     Initialize radiative transfer calculation
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
C
      integer ishell, itran, ip
C
C     ***first, zero Jeff and set lambda-* to 1
C
      do 20 itran=1, numtran
         do 10 ishell = 1, nshell
            Jeff(ishell, itran) = 0.
            lamstar(ishell, itran) = 1.   ! turn off radiative rates
                                          ! in first iteration to get LTE
10       continue
20    continue
C
C     ***then calculate abscissas and weights for Gauss-Hermite integration
C        Since the line is symmetric, we only do positive velocities
C
      call gauher(xinteg, winteg, 2*numvel)
C
C     ***choose impact parameters to be same as shell radii
C
      nimpact = nshell+1
      impact(1) = 0.
      do 30 ip=2, nimpact
         impact(ip) = rshell(ip-1)
30    continue
C
C     ***now get angular weights and shell pathlengths
C
      call angweight(rshell, nshell, MAXGRID,
     +               impact, nimpact, MAXIMPACT,
     +               minrad, maximp, angwgt, dshell)
C
      return
      end
C
C********************************************************************
      subroutine angweight(r, nr, nrp, xp, np, npp,
     +                     mr, mp, dmu, ds)
C********************************************************************
C
C     ***calculates the weights for angular integration
C        and also pathlength thru shells at various impacts
C
C        Inputs:
C        r               - array of radii, in ASCENDING ORDER
C        nr              - no. of radii
C        nrp             - phys. dimension of r()
C        xp              - array of impact parameters, in ASCENDING ORDER
C        np              - no. of impacts
C        npp             - phys. dimension of xp()
C
C        Outputs:
C        mr(npp)         - mr(k) gives the index of smallest radial shell
C                          penetrated by ray w/ impact xp(k)
C        mp(nrp)         - mp(k) gives the index of the largest impact
C                          parameter which penetrates shell r(k)
C        dmu(npp, nrp)   - angular weights
C        ds(npp, nrp)    - pathlengths
C
      real*8 r(nrp), xp(npp)
      real*8 dmu(npp, nrp)
      real*8 ds(npp, nrp)
      integer mr(npp), mp(nrp)
C
      integer ir, ip
      real*8 x1, x2
C
C     ***calculate mr() and mp()
C
      do 20 ip=1, np             ! loop over impacts
         ir = 1                  ! start from smallest radius
C
10       continue
         if(r(ir).LT.xp(ip) .AND. ir.LT.nr) then
            ir = ir+1            ! impact ip doesn't penetrate radius ir
            goto 10              ! so go to next larger radius
         endif
C
         if(r(ir).GE.xp(ip)) then
            mr(ip) = ir          ! smallest radius that ip penetrates
         else
            mr(ip) = nr+1        ! impact out of range - ip doesn't 
         endif                   ! penetrate any radii
20    continue
C
      do 40 ir= 1, nr            ! loop over radii
         ip = np                 ! check impacts from largest to smallest
C
30       continue
         if(xp(ip).GT.r(ir) .AND. ip.GT.1) then
            ip = ip-1
            goto 30
         endif
C
         if(xp(ip).LE.r(ir)) then
            mp(ir) = ip
         else
            mp(ir) = 0            ! none of the impacts penetrates ir
         endif
40    continue
C
C     Calculate pathlengths
C
      do 60 ip=1, np              ! loop over impacts
         ir = mr(ip)
         if(ir.LE.nr) then        ! this ray actually passes thru cloud
            x2 = xp(ip)/r(ir)
            x2 = x2*x2            ! calculate pathlength from ray center
C                                 ! to first intersecting shell
            ds(ip,ir) = r(ir)*sqrt(1.-x2)
C
            do 50 ir=mr(ip)+1, nr    ! now do pathlengths between shells
               x1 = r(ir)/r(ir-1)
               x1 = x1*x1
               x2 = xp(ip)/r(ir-1)
               x2 = x2*x2
               ds(ip, ir) = r(ir-1)*(x1-1.)
     +                      /(sqrt(x1-x2) + sqrt(1.-x2))
50          continue
         endif
60    continue
C
C     Calculate angular weights
C
      do 80 ir=1, nr              ! loop over radii
         ip = mp(ir)
C
         if(ip.EQ.1) then         ! only one ray through ir
            dmu(ip,ir) = 1.       ! so this carries the full weight
C
         else if(ip.GT.1) then    ! at least two rays thru ir
C
C           weight of first ray...
C
            x1 = xp(1)/r(ir)
            x2 = xp(2)/r(ir)
            x1 = x1*x1
            x2 = x2*x2
            dmu(1,ir) = 0.5*( x1/(1.+sqrt(1.-x1))
     +                      + x2/(1.+sqrt(1.-x2)) )
C
C           weight of last ray...
C
            x1 = xp(ip)/r(ir)
            x2 = xp(ip-1)/r(ir)
            x1 = x1*x1
            x2 = x2*x2
            dmu(ip,ir) = 0.5*(sqrt(1.-x1) + sqrt(1.-x2) )
C
C           weights of rays in between
C
            do 70 ip=2, mp(ir)-1
               x1 = xp(ip+1)/r(ir)
               x2 = xp(ip-1)/r(ir)
               x1 = x1*x1
               x2 = x2*x2
               dmu(ip,ir) = 0.5*( x1/(1.+sqrt(1.-x1))
     +                          - x2/(1.+sqrt(1.-x2)) )
70          continue
         endif
80    continue
C
      return
      end
C
C********************************************************************
      subroutine solverates
C********************************************************************
C
C     ***solve statistical equilibrium equations for each
C        radial shell, given lambda-* and Jeff from radiative
C        transfer calculation
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
      include 'cgs_constants.inc'
C
      real*8 abunfac, dustfac
      common/xfact/abunfac, dustfac
C
      real*8 collratecalc              ! function to calculate coll rates
      external collratecalc
C
      real*8 rates(MAXLEV, MAXLEV)     ! rate matrix
      real*8 cpyrates(MAXLEV, MAXLEV)  ! copy of rate matrix
      real*8 fpop(MAXLEV)              ! temporary storage for frac. pop.
      real*8 rhs(MAXLEV)               ! right-hand side of rate eqn
      real*8 ratesdet                  ! for determinant sign
      integer rateindex(MAXLEV)        ! for LU decomposition
C
      integer ishell, itran, ilev, jlev
      real*8 xtmp, cr, hr
      real*8 maxcool, maxheat, shcool, shheat
      real*8 mcr, mhr
      integer icool, jcool, iheat, jheat
C
      do 100 ishell=1, nshell        ! solve for each shell
C
C        ***zero heating & cooling rates
C
         heat(ishell) = 0.
         cool(ishell) = 0.
         maxcool = 0.
         maxheat = 0.
         icool = 0
         jcool = 0
         iheat = 0
         jheat = 0
         mcr = 0.
         mhr = 0.
C
C        ***zero rate matrix
C
         do 20 ilev=1, numlev
            do 10 jlev=1, numlev
               rates(ilev, jlev) = 0.
10          continue
20       continue
C
C        ***now add in collisional rates
C
         do 40 ilev=1, numlev
            do 30 jlev=1, ilev-1
               if(jlev.NE.ilev) then
                  xtmp = collratecalc(ilev, jlev, Tgas(ishell))
C                 write(*,*) ishell, ilev, jlev, Tgas(ishell), xtmp
                  hr = xtmp
                  xtmp = xtmp*h2den(ishell)
                  rates(ilev, ilev) = rates(ilev, ilev) + xtmp
                  rates(jlev, ilev) = rates(jlev, ilev) - xtmp
C
                  shheat =
     +                xtmp*h2den(ishell)*fracpop(ishell,ilev)
     +                *(energy(ilev) - energy(jlev))
                  heat(ishell) = heat(ishell) + shheat
                  if(shheat.GT.maxheat) then
                     maxheat = shheat
                     iheat = ilev
                     jheat = jlev
                     mhr = hr
                  endif
C
                  xtmp = xtmp*degen(ilev)
     +                   *exp((energy(jlev)-energy(ilev))/Tgas(ishell))
     +                   /degen(jlev)
                  cr = hr*degen(ilev)
     +                   *exp((energy(jlev)-energy(ilev))/Tgas(ishell))
     +                   /degen(jlev)
                  rates(jlev, jlev) = rates(jlev, jlev) + xtmp
                  rates(ilev, jlev) = rates(ilev, jlev) - xtmp
C
                  shcool =
     +                xtmp*h2den(ishell)*fracpop(ishell,jlev)
     +                *(energy(ilev) - energy(jlev))
                  cool(ishell) = cool(ishell) + shcool
                  if(shcool.GT.maxcool) then
                     maxcool = shcool
                     icool = ilev
                     jcool = jlev
                     mcr = cr
                  endif
C
               endif
30          continue
40       continue
C
C        write(*,*) ' Shell: ', ishell
         maxcool = maxcool*kboltz*abun(ishell)*abunfac
         maxheat = maxheat*kboltz*abun(ishell)*abunfac
C        write(*,*) ' Max cool: ', icool, jcool, maxcool, mcr
C        write(*,*) ' Max heat: ', iheat, jheat, maxheat, mhr
C
C        ***now add in radiative rates
C
         do 50 itran=1, numtran           ! loop over all transitions
C
            ilev = linitial(itran)        ! get initial and final levels
            jlev = lfinal(itran)
C
C           ***do corrected spontaneous emission rate
C
            xtmp = acoeff(itran)*(1.-lamstar(ishell, itran))
            rates(ilev, ilev) = rates(ilev, ilev) + xtmp
            rates(jlev, ilev) = rates(jlev, ilev) - xtmp
C
C           ***next, do corrected stimulated emission rate
C              notice that Jeff is in units of Kelvin since it
C              is related to the antenna temperature; thus we
C              divide by the photon temp to get rate in units of Aij
C
            xtmp = acoeff(itran)*Jeff(ishell, itran)/tphoton(itran)
            rates(ilev, ilev) = rates(ilev, ilev) + xtmp
            rates(jlev, ilev) = rates(jlev, ilev) - xtmp
C
C           ***finally, do corrected absorption rate
C
            xtmp = xtmp*degen(ilev)/degen(jlev)
            rates(jlev, jlev) = rates(jlev, jlev) + xtmp
            rates(ilev, jlev) = rates(ilev, jlev) - xtmp
C
50       continue
C
C        ***the rate equations are redundant, since rates sum to zero
C           However, we need to impose normalization condition, so
C           we overwrite last row of rate matrix with 1's
C           We also need to make up the right-hand side of eqn
C           and to make a copy of the rate matrix.
C
         do 70 ilev=1, numlev
            rates(numlev, ilev) = 1.
            rhs(ilev) = 0.
            fpop(ilev) = 0.
            do 60 jlev=1, numlev
               cpyrates(jlev, ilev) = rates(jlev,ilev)
60          continue
70       continue
         rhs(numlev) = 1.       ! fractional populations add up to 1.
         fpop(numlev) = 1.
C
C        ***Now solve the linear system
C
         call ludcmp(cpyrates, numlev, MAXLEV, rateindex, ratesdet)
         call lubksb(cpyrates, numlev, MAXLEV, rateindex, fpop)
C
C        ***and polish up solution a bit
C
         call mprove(rates, cpyrates, numlev, MAXLEV,
     +               rateindex, rhs, fpop)
C
C        ***now save the solution
C
         do 90 ilev=1, numlev
            oldpop(ishell, ilev) = fracpop(ishell, ilev)
            fracpop(ishell, ilev) = fpop(ilev)
C           write(*,*) 'Pops: ish ', ishell, 'lev', ilev, fpop(ilev)
90       continue
C
C     ***go on to next radial shell
C
100   continue
C
      return
      end
C
C********************************************************************
      subroutine solvetransfer(accelerate)
C********************************************************************
C
C     solve the radiative transfer for each transition
C
      integer accelerate
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
      include 'cgs_constants.inc'
C
      real*8 sigmadust
      external sigmadust
C
      integer itran, ishell, ivel, levi, levf
C
      real*8 lamvel(MAXGRID, MAXVEL)     ! angle-averaged lambda-*
      real*8 uvel(MAXGRID, MAXVEL)       ! and Feautrier u before
C                                        ! velocity averaging
C
      real*8 opacity(MAXGRID, MAXVEL)    ! total opacity
      real*8 gasopacity(MAXGRID, MAXVEL) ! gas opacity
      real*8 sourcefn(MAXGRID, MAXVEL)   ! gas+dust source function
C
      real*8 sgas(MAXGRID)               ! gas source function
      real*8 sdust(MAXGRID)              ! dust source function
      real*8 dustopacity(MAXGRID)        ! dust opacity
C
      real*8 velprof(MAXVEL)             ! normalized velocity profile
C
      real*8 wavelen                     ! wavelength in cm
      real*8 xsectdust                   ! dust absorption cross-section
      real*8 xsectgas                    ! gas absorption cross-section
C
      real*8 x1, x2, xi, xf, gi, gf, tph, denom
C
      real*8 tiny
      parameter(tiny=1.e-10)             ! smallest fractional population
C
      real*8 abunfac, dustfac
      common/xfact/abunfac, dustfac
C
C
C     ***Calculate normalized velocity profile function in cgs units
C
      x1 = sqrt(PI)*ewidth*akm           ! ewidth is km/s; convert to cm/s
      do 10 ivel=1, numvel
         velprof(ivel) = exp(-xinteg(ivel)**2)/x1
10    continue
C
C     ***Loop over transitions
C
      do 100 itran=1, numtran
C
         levi = linitial(itran)
         levf = lfinal(itran)
         gi = degen(levi)
         gf = degen(levf)
C
C        ***transition wavelength in cm
C
         wavelen = clight/(frequency(itran)*amegahz)
C
C        ***dust absorption cross-section at this wavelength
C
         xsectdust = sigmadust(wavelen/amicron)*dustfac
C
C        ***now calculate opacities in radial shells
C
         do 30 ishell=1, nshell
            dustopacity(ishell) = xsectdust*h2den(ishell)
     +                            *rmax*rscalecm
C
            xi = fracpop(ishell, levi)
            xf = fracpop(ishell, levf)
C
C           ***transition absorption cross-section per H2 molecule,
C              in cm-2 * Hz   (i.e. integrated over frequency)
C
            xsectgas = wavelen*wavelen*acoeff(itran)/(8.*PI)
            xsectgas = xsectgas*abun(ishell)
     +                 *(1./gf)*(gi*xf - gf*xi)
     +                 * abunfac
            if(xsectgas.LT.0.) then
C              write(*,*) ' Warning: population inversion for ',
C    +                    elabel(levi), ' - ', elabel(levf),
C    +                    ' in shell ', ishell
               xsectgas = 0.        ! force to zero 
            endif
C
C           ***now multiply by frequency profile
C
            do 20 ivel=1, numvel
               gasopacity(ishell, ivel) = xsectgas*rmax*rscalecm
     +               *h2den(ishell)*velprof(ivel)*wavelen
C
C              ***and add to get total opacity
C
               opacity(ishell, ivel) = gasopacity(ishell, ivel) 
     +                                 + dustopacity(ishell)
20          continue
30       continue
C
C        ***now calculate gas and dust source functions
C
         tph = tphoton(itran)
C
         do 40 ishell=1, nshell
            x1 = tph/(exp(tph/Tdust(ishell))-1.)
            sdust(ishell) = x1
C
            xi = fracpop(ishell, levi)
            xf = fracpop(ishell, levf)
C
            denom = gi*xf-gf*xi
            if(abs(denom).LE.tiny) then
               x1 = 0.     ! force to zero
            else
               x1 = tph*gf*xi/denom
            endif
            if(x1.LT.0.) then
C              write(*,*) ' Warning: Gas source fn < 0. for ',
C    +                    elabel(levi), ' - ', elabel(levf),
C    +                    ' in shell ', ishell
               x1 = 0.    ! force to zero...
            endif
            sgas(ishell) = x1
C
40       continue
C
C        ***now calculate total source function
C
         do 60 ishell=1, nshell
            x1 = dustopacity(ishell)*sdust(ishell)
            do 50 ivel=1, numvel
               x2 = gasopacity(ishell, ivel)*sgas(ishell)
               sourcefn(ishell, ivel) = (x1 + x2)
     +                                  /opacity(ishell, ivel)
50          continue
60       continue
C
C        ***we're ready to do the radiative transfer
C
         do 70 ivel=1, numvel
            call solver(rshell, nshell, MAXGRID,
     +                 impact, nimpact, MAXIMPACT,
     +                 minrad, maximp, angwgt, dshell,
     +                 opacity(1,ivel), sourcefn(1,ivel),
     +                 uvel(1,ivel), lamvel(1,ivel))
70       continue
C
C        ***now sum intensities and lambda over velocity
C           note that lambda-operator gets multiplied by the
C           ratio of gas to total opacity, as in R+H, Eqn. 2.25
C
         do 90 ishell=1, nshell
C
            x1 = 0.
            x2 = 0.
            do 80 ivel=1, numvel
               x1 = x1 + winteg(ivel)*uvel(ishell,ivel)
               x2 = x2 + winteg(ivel)*lamvel(ishell, ivel)
     +             *gasopacity(ishell,ivel)/opacity(ishell,ivel)
80          continue
C
C           ***Normalize and save results
C              factor of 2 comes from doing only positive velocities
C               sqrt(PI) comes from normalization of exp(-x**2)
C
            Jtot(ishell, itran) = x1*2/sqrt(PI)
C
            if(accelerate.NE.0) then
C
C              ***we're using the accelerated lambda-iteration
C
               x2 = x2*2./sqrt(PI)
C
C              ***we'd better watch out for spurious values of lamstar
C                 Masers, for instance, might cause trouble here
C
               if(x2.GT.1.0) then
                  x2 = 1.0
                  write(*,*) ' Warning: transition ', itran,
     +                    '(', elabel(levi), ' - ',
     +                     elabel(levf), ')',
     +                    ' in shell', ishell, ' has lambda > 1 !'
               endif
C
               if(x2.LT.0.) then
C
C                 ***lambda < 0 ; may be masing
C                    so treat this transition in "unaccelerated fashion"
C
                  lamstar(ishell, itran) = 0.
                  Jeff(ishell, itran) = Jtot(ishell, itran) 
               else
C
                  lamstar(ishell, itran) = x2
C
C                 ***Subtract off the part of intensity which comes
C                    from the diagonal part of lambda to get Jeff
C                    (R+H, eqn. 2.26)
C
                  Jeff(ishell, itran) = Jtot(ishell, itran) -
     +                           lamstar(ishell,itran)*sgas(ishell)
               endif
C
            else
C
C              ***we're doing an ordinary unaccelerated iteration
C                 make sure lamstar set to zero
C                 and use full intensity, not Jeff
C
               Jeff(ishell, itran) = Jtot(ishell, itran) 
               lamstar(ishell, itran) = 0.
            endif
C
C            write(*,*) 'transfer:', ishell, itran,
C     +                 Jeff(ishell, itran),
C     +                 lamstar(ishell, itran)
C
90       continue
C
100   continue
C
      return
      end
c
C*******************************************************************
      subroutine solver(r,nr,nrp,xp,np,npp,mr,mp,dmu,ds,
     +                  chi,ss,avu,avl)
C*******************************************************************
c
c     1-2-94  J. Zmuidzinas
c
c     Radiative transfer solver for spherical geometry
c     Based directly on 1-st order transfer eqn.
c     Solves for intensity along rays of given impact parameters
c     Assumes that opacity and source function vary linearly
c     between radial shells.
c
c     input:
c       r     - normalized shell radii (rgrid(1)=0, rgrid(nr)=1)
c       nr    - number of shells
c       nrp   - physical dimension of r() (really for dmu+ds)
c       xp    - impact parameters
c       np    - number of impacts
c       npp   - physical dim of xp() (again for dmu+ds)
c       mr    - index of minimum radius penetrated vs. impact
c       mp    - index of maximum impact to peneterate vs. radius 
c       dmu   - weights for angular integration (from angweight())
c       ds    - shell pathlengths for various impacts (from angweight())
c       chi   - absorption coefficient
c       ss    - source function
c     output:
c       avu   - angle averaged u
c       avl   - angle averaged lambda operator
c
c       **all are real*8 !!!EXCEPT!!! dmu and ds WHICH ARE real*8
c
c     All input and output physical quantities (chi, s, avu, avl)
c     are evaluated AT THE SHELL RADII.
c
c     The routine calculates the angle-averaged intensity
c     and the angle-averaged diagonal part of the lambda
c     (i.e. transfer) operator (avl) evaluated AT THE SHELL RADII.
c     
C*******************************************************************
c
      implicit real*8 (a-h,o-z)
C
      include 'grid.inc'            ! for MAXGRID
C
C
C     Inputs:
      integer nr, nrp, np, npp
      integer  mr(np), mp(nr)
      real*8 r(nr), xp(np)
      real*8 dmu(npp,nrp), ds(npp,nrp)     ! must use physical dim here
      real*8 chi(nr), ss(nr)
C
C     Outputs:
      real*8 avu(nr),avl(nr)
C
C     local variables:
C
      real*8 dtau(MAXGRID), ifor(MAXGRID), iback(MAXGRID),
     +       e(MAXGRID), f(MAXGRID), g(MAXGRID), h(MAXGRID),
     +       rl(MAXGRID)
      real*8 x1
C
      parameter (cut=1.e-2)
C
c     initialize variables
c
      do ir=1,nr
         avu(ir)=0.
         avl(ir)=0.
      enddo
c
c     loop over impacts
      do ip=1,np
c
c     calculate optical depth grid; e, f, g, h coefficients
c
         ir = mr(ip)
c
         if(ir.LE.nr) then           ! ray actually passes thru cloud,
c                                    ! so go ahead...
c
         x1 = ds(ip,ir)*chi(ir)      ! assume chi is constant in
         dtau(ir) = x1               ! innermost shell
         h(ir) = 2.*x1
         e(ir) = exp(-x1)
         if(abs(x1).LE.cut) then
            g(ir) = (x1/2.)*(1.-x1/3.*(1.-x1/4.*(1.-x1/5.*
     +                      (1.-x1/6.*(1.-x1/7.*(1.-x1/8.*
     +                      (1.-x1/9.*(1.-x1/10.))))))))
            f(ir) = x1*(1.-g(ir))
         else
            f(ir) = 1.-e(ir)
            g(ir)  =1. - f(ir)/x1
         endif
         do ir=mr(ip)+1, nr
            x1 = ds(ip,ir)*(chi(ir)+chi(ir-1))*0.5
            dtau(ir) = x1
            h(ir) = h(ir-1) + 2.*x1
            e(ir) = exp(-x1)
            if(abs(x1).LE.cut) then
               g(ir) = (x1/2.)*(1.-x1/3.*(1.-x1/4.*(1.-x1/5.*
     +                         (1.-x1/6.*(1.-x1/7.*(1.-x1/8.*
     +                         (1.-x1/9.*(1.-x1/10.))))))))
               f(ir) = x1*(1.-g(ir))
            else
               f(ir) = 1.-e(ir)
               g(ir)  =1. - f(ir)/x1
            endif
         enddo
c
c     do the transfer
c
c     first, loop backwards to get ingoing ray and diagonal of lambda
c
         iback(nr) = 0.              ! no background radiation incident
         rl(nr) = 0.5*g(nr)          ! on the cloud
         do ir=nr-1, mr(ip), -1
            iback(ir) = e(ir+1)*iback(ir+1)
     +                  + f(ir+1)*ss(ir+1)
     +                  + g(ir+1)*(ss(ir)-ss(ir+1))
            rl(ir) = 0.5*(g(ir) + g(ir+1)*(1.+exp(-h(ir))))
         enddo
c
c     now loop forwards to get outgoing ray
c
         ir = mr(ip)
         x1 = e(ir)*e(ir)
         ifor(ir) = x1*iback(ir) + (1.-x1)*ss(ir)
         do ir=mr(ip)+1, nr
            ifor(ir) = e(ir)*ifor(ir-1)
     +                 + f(ir)*ss(ir-1)
     +                 + g(ir)*(ss(ir)-ss(ir-1))
         enddo
c
c     calculate angle averages
c
         do ir=mr(ip), nr
            avu(ir) = avu(ir) +
     +                0.5*(ifor(ir)+iback(ir))*dmu(ip,ir)
            avl(ir) = avl(ir) + rl(ir)*dmu(ip,ir)
         enddo
C
      endif                 ! if(mr(ip).LE.nr)
C
      enddo                 ! end of loop over impact parameters
C
      return
      end
C
C********************************************************************
      subroutine checkconvergence(done)
C********************************************************************
C
C     check for convergence of the level populations
C
      logical done
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
C
      integer ilev, ishell, ilevmax, ishmax
      real*8 maxchange, change, rmschange, rmsweight, x1
      real*8 cnveps
      integer maxiter, accon
      common/convcrit/cnveps, maxiter, accon
C
      parameter(EPS=1.E-6)     ! for calculating fractional change
C
C     find maximum relative change
C
      maxchange = 0.
      rmschange = 0.
      rmsweight = 0.
      do 20 ishell = 1, nshell
         do 10 ilev = 1, numlev
            change = abs(fracpop(ishell, ilev) - oldpop(ishell, ilev))
            x1 = abs(fracpop(ishell, ilev))           ! take abs
            rmschange = rmschange + change*change*x1  ! in case <0 !!
            rmsweight = rmsweight + x1                ! so wgt OK
C
            change = change/(abs(fracpop(ishell, ilev)) + EPS)
            if(maxchange.LT.change) then
               maxchange = change
               ishmax = ishell
               ilevmax = ilev
            endif
10       continue
20    continue
C
C     Convergence test
C
      write(*,*) ' Maximum fractional population change is',
     +           maxchange
      write(*,*) ' This occurs for level', ilevmax,
     +           '(', elabel(ilevmax), ')',
     +           ' in radial shell', ishmax, 
     +           '( /', nshell, ')'
      write(*,*) ' Old population: ', oldpop(ishmax, ilevmax)
      write(*,*) ' New population: ', fracpop(ishmax, ilevmax)
      rmschange = sqrt(rmschange/rmsweight)
      write(*,*) ' Population-weighted r.m.s pop. change is:',
     +           rmschange
      done = (maxchange.LT.cnveps)
C
      return
      end
C
C********************************************************************
      SUBROUTINE gauher(x,w,n)
C********************************************************************
      INTEGER n,MAXIT
      REAL*8 w(n),x(n)
      REAL*8 EPS,PIM4
      PARAMETER (EPS=3.D-14,PIM4=.7511255444649425D0,MAXIT=10)
      INTEGER i,its,j,m
      REAL*8 p1,p2,p3,pp,z,z1
      m=(n+1)/2
      do 13 i=1,m
        if(i.eq.1)then
          z=sqrt(float(2*n+1))-1.85575*(2*n+1)**(-.16667)
        else if(i.eq.2)then
          z=z-1.14*n**.426/z
        else if (i.eq.3)then
          z=1.86*z-.86*x(1)
        else if (i.eq.4)then
          z=1.91*z-.91*x(2)
        else
          z=2.*z-x(i-2)
        endif
        do 12 its=1,MAXIT
          p1=PIM4
          p2=0.d0
          do 11 j=1,n
            p3=p2
            p2=p1
            p1=z*sqrt(2.d0/j)*p2-sqrt(dble(j-1)/dble(j))*p3
11        continue
          pp=sqrt(2.d0*n)*p2
          z1=z
          z=z1-p1/pp
          if(abs(z-z1).le.EPS)goto 1
12      continue
        pause 'too many iterations in gauher'
1       x(i)=z
        x(n+1-i)=-z
        w(i)=2.d0/(pp*pp)
        w(n+1-i)=w(i)
13    continue
      return
      END
C
C********************************************************************
      SUBROUTINE ludcmp(a,n,np,indx,d)
C********************************************************************
      INTEGER n,np,indx(n),NMAX
      REAL*8 d,a(np,np),TINY
      PARAMETER (NMAX=500,TINY=1.0e-20)
      INTEGER i,imax,j,k
      REAL*8 aamax,dum,sum,vv(NMAX)
      d=1.
      do 12 i=1,n
        aamax=0.
        do 11 j=1,n
          if (abs(a(i,j)).gt.aamax) aamax=abs(a(i,j))
11      continue
        if (aamax.eq.0.) pause 'singular matrix in ludcmp'
        vv(i)=1./aamax
12    continue
      do 19 j=1,n
        do 14 i=1,j-1
          sum=a(i,j)
          do 13 k=1,i-1
            sum=sum-a(i,k)*a(k,j)
13        continue
          a(i,j)=sum
14      continue
        aamax=0.
        do 16 i=j,n
          sum=a(i,j)
          do 15 k=1,j-1
            sum=sum-a(i,k)*a(k,j)
15        continue
          a(i,j)=sum
          dum=vv(i)*abs(sum)
          if (dum.ge.aamax) then
            imax=i
            aamax=dum
          endif
16      continue
        if (j.ne.imax)then
          do 17 k=1,n
            dum=a(imax,k)
            a(imax,k)=a(j,k)
            a(j,k)=dum
17        continue
          d=-d
          vv(imax)=vv(j)
        endif
        indx(j)=imax
        if(a(j,j).eq.0.)a(j,j)=TINY
        if(j.ne.n)then
          dum=1./a(j,j)
          do 18 i=j+1,n
            a(i,j)=a(i,j)*dum
18        continue
        endif
19    continue
      return
      END
C
C********************************************************************
      SUBROUTINE lubksb(a,n,np,indx,b)
C********************************************************************
      INTEGER n,np,indx(n)
      REAL*8 a(np,np),b(n)
      INTEGER i,ii,j,ll
      REAL*8 sum
      ii=0
      do 12 i=1,n
        ll=indx(i)
        sum=b(ll)
        b(ll)=b(i)
        if (ii.ne.0)then
          do 11 j=ii,i-1
            sum=sum-a(i,j)*b(j)
11        continue
        else if (sum.ne.0.) then
          ii=i
        endif
        b(i)=sum
12    continue
      do 14 i=n,1,-1
        sum=b(i)
        do 13 j=i+1,n
          sum=sum-a(i,j)*b(j)
13      continue
        b(i)=sum/a(i,i)
14    continue
      return
      END
C
C********************************************************************
      SUBROUTINE mprove(a,alud,n,np,indx,b,x)
C********************************************************************
      INTEGER n,np,indx(n),NMAX
      REAL*8 a(np,np),alud(np,np),b(n),x(n)
      PARAMETER (NMAX=500)
CU    USES lubksb
      INTEGER i,j
      REAL*8 r(NMAX)
      DOUBLE PRECISION sdp
      do 12 i=1,n
        sdp=-b(i)
        do 11 j=1,n
          sdp=sdp+dble(a(i,j))*dble(x(j))
11      continue
        r(i)=sdp
12    continue
      call lubksb(alud,n,np,indx,r)
      do 13 i=1,n
        x(i)=x(i)-r(i)
13    continue
      return
      END
