C
C    Calculate rotational spectrum of a diatomic molecule
C
C    Ver 2 - calculate frequencies as well.
C
      integer maxlev
      parameter (maxlev = 20)
C
      real*8 E(maxlev), A(maxlev), fr(maxlev)
      integer g(maxlev)
      character*80 outfile
C
      real megahertz
      parameter (megahertz=1.E6)
C
      integer Jmax
      real B, D, H, L, M
      real dipole
      character*80 constsfile
C
      integer J
      real x, freq
C
      include 'cgs_constants.inc'
C
      write(*,*) ' Molecular constants file ? '
      read(*,1) constsfile
1     format(A)
      write(*,*) ' Output file ? '
      read(*,1) outfile
C
C     get molecular constants
C
      open(UNIT=1, FILE=constsfile, STATUS='OLD')
C
      read(1,*) Jmax   ! maximum J to calculate E
      read(1,*) dipole ! dipole moment in Debye
      read(1,*) B      ! rotational constants in MHz
      read(1,*) D
      read(1,*) H
      read(1,*) L
      read(1,*) M
      close(1)
C
      if(Jmax.GT.maxlev) then
         Jmax = maxlev
      endif
C
      open(UNIT=1, FILE=outfile, STATUS='UNKNOWN')

      do 10 J=1, Jmax
         x = J*(J+1)
C
C        calculate energy in hertz
C
         E(J) = (((((M*x-L)*x +H)*x -D)*x +B)*x )*megahertz
         g(J) = 2*J+1     ! degeneracy
C
C        Calculate Einstein A-coefficient
C
         if(J.GT.1) then
            freq = E(J)-E(J-1)
         else
            freq = E(1)
         endif
         fr(J) = freq/(megahertz*1000.)
C
         A(J) = 32*PI*PI*PI*J/(3*g(J))
         A(J) = A(J)*(dipole*debye*freq/(echarge*clight))**2
         A(J) = A(J)*freq/alphafs
10    continue
C
C     write out results
C
      write(1,*) ' Number of levels: '
      write(1,*) Jmax+1
      write(1,*) ' Level labels: '
      do 15 J=0, Jmax
         write(1,*) ' J =', J
15    continue
      write(1,*) ' J+1  Level energy (K)   degeneracy : '
      write(1,*) ' 1   0.00     1'
      do 20 J=1, Jmax
         write(1,*) J+1, '  ', E(J)*hplanck/kboltz, '  ', g(J)
20    continue
C
C     write out A-coefficients
C
      write(1,*) ' Ji+1   Jf+1   A(Ji-Jf) (s-1) '
      do 40 J=1, Jmax
         write(1,*) J+1, '   ', J, '   ', A(J), fr(J)
40    continue
      close(1)
      stop
      end
