C*************************************************************************
C      Subroutines to calculate three-j symbols
C
C      J. Zmuidzinas, Dec. 5 1993
C
C      Copied from 'Angular Momentum' by R. Zare, Appendix A
C      with minor mods
C
C*************************************************************************
      subroutine setup
C*************************************************************************
C
C     ***calculates and stores factorials
C
      implicit real*8 (A-H, O-Z)
      parameter (Nmax=400)
      dimension facl(Nmax)
      common/factor/facl
C
      facl(1)=0.D0
      do 100 i=2, Nmax
         i1 = i-1
         facl(i) = facl(i1) + dlog(dfloat(i1))
100   continue
      return
      end
C
C*************************************************************************
      real*8 function frontl(x1, x2, x3, y1, y2, y3)
C*************************************************************************
C
C     ***used in three-j calculation
C
      implicit real*8 (A-H, O-Z)
      parameter (Nmax=400)
      dimension fl(Nmax)
      common/factor/fl
C
      l1 = ifix(sngl(x1+x2-x3+1.1))
      l2 = ifix(sngl(x2+x3-x1+1.1))
      l3 = ifix(sngl(x3+x1-x2+1.1))
      l4 = ifix(sngl(x1+x2+x3+2.1))
      l5 = ifix(sngl(x1+y1+1.1))
      l6 = ifix(sngl(x1-y1+1.1))
      l7 = ifix(sngl(x2+y2+1.1))
      l8 = ifix(sngl(x2-y2+1.1))
      l9 = ifix(sngl(x3+y3+1.1))
      l10 = ifix(sngl(x3-y3+1.1))
C
      frontl = fl(l1) + fl(l2) + fl(l3) - fl(l4) + fl(l5) +fl(l6)
      frontl = frontl + fl(l7) + fl(l8) + fl(l9) + fl(l10)
      frontl = frontl/2.D0
C
      return
      end
C
C*************************************************************************
      real*4 function real3j(rj1, rj2, rj3, rm1, rm2, rm3)
C*************************************************************************
C
C     ***calculates 3-j coefficient
C        real3j - each real argument is actual value
C
C*************************************************************************
C
      implicit real*8 (A-H, O-Z)
      parameter (Nmax=400)
      dimension fl(Nmax)
      common/factor/fl
      real*8 frontl
      external frontl
C
      real*4 rj1, rj2, rj3, rm1, rm2, rm3
C
      j1d = int(2*rj1)
      j2d = int(2*rj2)
      j3d = int(2*rj3)
      m1d = int(2*rm1)
      m2d = int(2*rm2)
      m3d = int(2*rm3)
C
      x1 = dfloat(j1d)/2.D0
      x2 = dfloat(j2d)/2.D0
      x3 = dfloat(j3d)/2.D0
      y1 = dfloat(m1d)/2.D0
      y2 = dfloat(m2d)/2.D0
      y3 = dfloat(m3d)/2.D0
C
      if(iabs(m1d).GT.j1d) goto 9998
      if(iabs(m2d).GT.j2d) goto 9998
      if(iabs(m3d).GT.j3d) goto 9998
C
      if(j1d+j2d-j3d.LT.0) goto 9998
      if(j2d+j3d-j1d.LT.0) goto 9998
      if(j3d+j1d-j2d.LT.0) goto 9998
C
      if(m1d+m2d+m3d.NE.0) goto 9998
C
      kmin = (j3d-j1d-m2d)/2
      kmin1 = kmin
      kmin2 = (j3d-j2d+m1d)/2
      if(kmin2.LT.kmin) kmin = kmin2
      kmin = (-1)*kmin
C
      kmax = ifix(sngl(x1 + x2 - x3 + 0.1))
      kmax1 = kmax
      kmax2 = ifix(sngl(x1 - y1 + 0.1))
      kmax3 = ifix(sngl(x2 + y2 + 0.1))
      if(kmax2.LT.kmax) kmax = kmax2
      if(kmax3.LT.kmax) kmax = kmax3
C
      if(kmin.LT.0) kmin=0
      if(kmin.GT.kmax) goto 9998
C
      jmin = kmin+1
      jmax = kmax+1
      term1 = frontl(x1, x2, x3, y1, y2, y3)
      msign = (-1)**((j1d-j2d-m3d)/2)
      sum = 0.0D0
C
      do 10 i1=jmin, jmax
         i = i1-1
         term2 = fl(i1) + fl(kmin1+i1) + fl(kmin2+i1)
         term2 = term2 + fl(kmax1-i+1) + fl(kmax2-i+1) + fl(kmax3-i+1)
         term = dexp(term1-term2)
         term = term*msign*((-1)**i)
         sum = sum + term
10    continue
C
      goto 9999
C
9998  continue
      sum = 0.0
9999  continue
      real3j = sum
C
      return
      end
C
C*************************************************************************
      real*4 function thrj(jj1, jj2, jj3, mm1, mm2, mm3)
C*************************************************************************
C
C     ***calculates 3-j coefficient
C        thrj - each integer argument is 2x actual value
C
C*************************************************************************
C
      real*4 real3j
      external real3j
      real*4 result, xj1, xj2, xj3, xm1, xm2, xm3
C
      xj1 = jj1/2.0D0
      xj2 = jj2/2.0D0
      xj3 = jj3/2.0D0
      xm1 = mm1/2.0D0
      xm2 = mm2/2.0D0
      xm3 = mm3/2.0D0
      result = real3j(xj1, xj2, xj3, xm1, xm2, xm3)
      thrj = result
      return
      end
C
C*************************************************************************
      real*4 function vint3j(ij1, ij2, ij3, im1, im2, im3)
C*************************************************************************
C
C     ***calculates 3-j coefficient
C        vint3j - each integer argument is actual value (no 1/2 ints)
C
C*************************************************************************
C
      real*4 real3j
      external real3j
      real*4 result, xj1, xj2, xj3, xm1, xm2, xm3
C
      xj1 = ij1
      xj2 = ij2
      xj3 = ij3
      xm1 = im1
      xm2 = im2
      xm3 = im3
      result = real3j(xj1, xj2, xj3, xm1, xm2, xm3)
      vint3j = result
      return
      end
