C**********************************************************************
C
C     Dust model for SgrB2
C
C     R-2 + const. density law
C
C**********************************************************************
      program dusty
C**********************************************************************
C
C     General parameters and conversion factors to cgs
C
      real*8 PI
      real*8 Msol, pc, mu
      real mm, micron
      parameter (PI=3.1415926535)
      parameter (Msol=2E33, pc=3.09E18, mu=4.58E-24)
      parameter (mm=0.1, micron=1E-4 ) 
C
C     Angle (arcsec) subtended by 1pc at SgrB2 (D=8.5kpc)
C
      real theta1
      parameter (theta1=180*60*60/(8500*PI))
C
      real bw, Tantenna, flux
      real b1, lambd1
      common/TAint/b1, lambd1
C
      real TA
      external TA
C
      write(*,*) ' FWHM Gaussian beam width in arcsec ?'
      read(*,*) bw
      write(*,*) ' Wavelength (microns) ?'
      read(*,*) lambd1
      write(*,*) ' Lambda = ', lambd1, ' microns'
      write(*,*) ' beam width  = ', bw, ' arcsec (FWHM)'
C
      Tantenna = Tbeam( bw/(theta1*2*sqrt(alog(2.))) )
      flux = 7.354*((100./lambd1)**2)*Tantenna*bw*bw
C
      write(*,*) ' Tantenna = ', Tantenna, ' K'
      write(*,*) ' Flux = ', flux, ' Jy'
      stop
      end
C
C**********************************************************************
      real function TAinteg(z)
C**********************************************************************
C
C     This calculates the integrand needed in the integration
C     through the cloud determine the antenna temperature at
C     wavelength *** lambd1 *** and at impact parameter *** b1 ***.
C     The integrand is evaluated at some depth *** z ***
C     into the cloud.
C
      real z, b1, lambd1
      common/TAint/b1, lambd1
C
C     variables we need to calculate
C
      real radius, Tdust, planck
      real  density, Ncol1, Ncol2
      real  Qdust, sigdust, tau
      real integrand
C
C     General parameters and conversion factors to cgs
C
      real*8 PI
      real*8 Msol, pc, mu
      real mm, micron
      parameter (PI=3.1415926535)
      parameter (Msol=2E33, pc=3.09E18, mu=4.58E-24)
      parameter (mm=0.1, micron=1E-4 ) 
C
C     dust parameters
C
      real adust, rhodust, Rgasdust
      real Q0dust, lam0dust, slopedust
      parameter(adust=0.1, rhodust=3, Rgasdust=100)
      parameter(Q0dust=2.E-5, lam0dust=1300, slopedust=1.5)
C
C     SgrB2-specific parameters
C
C     dust temperature profile (Lis&G, ApJ 356, 195, 1990)
C
      real Tdust0, rdust0, slopetemp
      parameter (Tdust0=398. , rdust0=0.01, slopetemp=-0.5)
C
C     Density law (after Lis&G, ApJ 337, 704, 1989 C18O data)
C
      real n1, n2, r1, Rmax
      parameter (n1=5.5E4, n2=2.2E3, r1=1.25, Rmax=22.5)
C
C     Stuff for Planck function
C
      real lambdaT
      parameter (lambdaT=11605.9*1.23963) ! in micron-K
C
C
C     Start calculating stuff
C
C     radius, temperature, Planck function, density:
C
      radius = sqrt(b1*b1+z*z)
      Tdust = Tdust0*(radius/rdust0)**slopetemp
      planck = lambdaT/(lambd1*(exp(lambdaT/(lambd1*Tdust))-1))
      density = n1*(r1/radius)**2 + n2
C
C     dust absorption cross-section per H2 molecule
C
      Qdust=Q0dust*(lam0dust/lambd1)**slopedust
      sigdust = 3*mu*Qdust/(4*adust*micron*rhodust*Rgasdust)
C
C     H2 column densities of the n1 and n2 density components
C
      Ncol1 = n1*((r1*pc)**2)/(b1*pc)
      Ncol1 = Ncol1*(atan(z/b1) + acos(b1/Rmax))
      Ncol2 = n2*Rmax*pc
      Ncol2 = Ncol2*(z/Rmax + sqrt(1-(b1/Rmax)**2))
C
C     Dust optical depth
C
      tau = (Ncol1+Ncol2)*sigdust
C
C     Put this all together in the integrand
C
      integrand = planck*sigdust*density*pc*exp(-tau)
      TAinteg = integrand
      return
      end
C
C
C**********************************************************************
      real function TA(b)
C**********************************************************************
C
      real TAinteg
      external TAinteg, QROMB1, MIDPT1
C
      real b, result, zmin, zmax
      real b1, lambd1
      common/TAint/b1, lambd1
C
C     Only need this for Rmax
C
      real n1, n2, r1, Rmax
      parameter (n1=5.5E4, n2=2.2E3, r1=1.25, Rmax=22.5)
C
C     stuff the impact parameter b into the common block
C
      b1 = b
C
C     do the integration
C
      zmax = sqrt(Rmax*Rmax-b1*b1)
      zmin = - zmax
      call QROMB1(TAinteg, zmin, zmax, result, MIDPT1)
      TA = result
      return
      end
C
C**********************************************************************
      real function Tbeaminteg(y)
C**********************************************************************
C
      real y
      real TA
      external TA
C
      real beamwid
      common/beam/beamwid
C
      real result
C
      result = beamwid*sqrt(-alog(y))
      result = TA(result)
      Tbeaminteg = result
      return
      end
C
C**********************************************************************
      real function Tbeam(beamwidth)
C**********************************************************************
C
      real beamwidth
      real Tbeaminteg
      external Tbeaminteg, QROMB2, MIDPT2
C
      real beamwid
      common/beam/beamwid
C
      real result, ymin, ymax
C
      beamwid = beamwidth
      ymin = 0.0001
      ymax = 1.-ymin
      call QROMB2(Tbeaminteg, ymin, ymax, result, MIDPT2)
      Tbeam = result
      return
      end
C
C**********************************************************************
      SUBROUTINE QROMB1(FUNC,A,B,SS,CHOOSE)
C**********************************************************************
      PARAMETER (EPS=1.E-5,JMAX=14,JMAXP=JMAX+1,KM=4,K=KM+1)
      DIMENSION S(JMAXP),H(JMAXP)
      REAL FUNC
      EXTERNAL FUNC, CHOOSE
      H(1)=1.
      DO 11 J=1,JMAX
        CALL CHOOSE(FUNC,A,B,S(J),J)
        IF (J.GE.K) THEN
          CALL POLINT(H(J-KM),S(J-KM),K,0.0,SS,DSS)
          IF (ABS(DSS).LT.EPS*ABS(SS)) RETURN
        ENDIF
        S(J+1)=S(J)
        H(J+1)=H(J)/9.
11    CONTINUE
      PAUSE 'Too many steps.'
      END
C
C**********************************************************************
      SUBROUTINE QROMB2(FUNC,A,B,SS,CHOOSE)
C**********************************************************************
      PARAMETER (EPS=1.E-5,JMAX=14,JMAXP=JMAX+1,KM=4,K=KM+1)
      DIMENSION S(JMAXP),H(JMAXP)
      REAL FUNC
      EXTERNAL FUNC, CHOOSE
      H(1)=1.
      DO 11 J=1,JMAX
        CALL CHOOSE(FUNC,A,B,S(J),J)
        IF (J.GE.K) THEN
          CALL POLINT(H(J-KM),S(J-KM),K,0.0,SS,DSS)
          IF (ABS(DSS).LT.EPS*ABS(SS)) RETURN
        ENDIF
        S(J+1)=S(J)
        H(J+1)=H(J)/9.
11    CONTINUE
      PAUSE 'Too many steps.'
      END
C
C**********************************************************************
      SUBROUTINE MIDSQU(FUNK,AA,BB,S,N)
C**********************************************************************
      REAL FUNK
      EXTERNAL FUNK
      FUNC(X) = 2.*X*FUNK(BB-X**2)
      B=SQRT(BB-AA)
      A=0.
      IF (N.EQ.1) THEN
        S=(B-A)*FUNC(0.5*(A+B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/(3.*TNM)
        DDEL=DEL+DEL
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DDEL
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=(S+(B-A)*SUM/TNM)/3.
        IT=3*IT
      ENDIF
      RETURN
      END
C
C**********************************************************************
      SUBROUTINE MIDSQL(FUNK,AA,BB,S,N)
C**********************************************************************
      REAL FUNK
      EXTERNAL FUNK
      FUNC(X) = 2.*X*FUNK(AA+X**2)
      B=SQRT(BB-AA)
      A=0.
      IF (N.EQ.1) THEN
        S=(B-A)*FUNC(0.5*(A+B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/(3.*TNM)
        DDEL=DEL+DEL
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DDEL
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=(S+(B-A)*SUM/TNM)/3.
        IT=3*IT
      ENDIF
      RETURN
      END
C
C**********************************************************************
      SUBROUTINE MIDPT1(FUNC,A,B,S,N)
C**********************************************************************
      REAL FUNC, A, B, S
      INTEGER N
      EXTERNAL FUNC
      IF (N.EQ.1) THEN
        S=(B-A)*FUNC(0.5*(A+B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/(3.*TNM)
        DDEL=DEL+DEL
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DDEL
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=(S+(B-A)*SUM/TNM)/3.
        IT=3*IT
      ENDIF
      RETURN
      END
C**********************************************************************
      SUBROUTINE MIDPT2(FUNC,A,B,S,N)
C**********************************************************************
      REAL FUNC, A, B, S
      INTEGER N
      EXTERNAL FUNC
      IF (N.EQ.1) THEN
        S=(B-A)*FUNC(0.5*(A+B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/(3.*TNM)
        DDEL=DEL+DEL
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DDEL
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=(S+(B-A)*SUM/TNM)/3.
        IT=3*IT
      ENDIF
      RETURN
      END
C
C**********************************************************************
      SUBROUTINE POLINT(XA,YA,N,X,Y,DY)
C**********************************************************************
      PARAMETER (NMAX=10) 
      DIMENSION XA(NMAX),YA(NMAX),C(NMAX),D(NMAX)
      NS=1
      DIF=ABS(X-XA(1))
      DO 11 I=1,N 
        DIFT=ABS(X-XA(I))
        IF (DIFT.LT.DIF) THEN
          NS=I
          DIF=DIFT
        ENDIF
        C(I)=YA(I)
        D(I)=YA(I)
11    CONTINUE
      Y=YA(NS)
      NS=NS-1
      DO 13 M=1,N-1
        DO 12 I=1,N-M
          HO=XA(I)-X
          HP=XA(I+M)-X
          W=C(I+1)-D(I)
          DEN=HO-HP
          IF(DEN.EQ.0.)PAUSE
          DEN=W/DEN
          D(I)=HP*DEN
          C(I)=HO*DEN
12      CONTINUE
        IF (2*NS.LT.N-M)THEN
          DY=C(NS+1)
        ELSE
          DY=D(NS)
          NS=NS-1
        ENDIF
        Y=Y+DY
13    CONTINUE
      RETURN
      END
C
