C
C********************************************************************
      program colden
C********************************************************************
C
C     J. Zmuidzinas   4-17-95
C
C     calculate ratio of line integrated intensity to upper state
C     column density for optically thin emission
C
      include 'spectrum.inc'
      include 'cgs_constants.inc'
C
      real*8 nupper(MAXTRAN)
      real*8 max, x1, srcfun
      real*8 fpop(MAXLEV)
      real*8 Tx, Qpart, wavelen
      character*80 specfile, outfile
      integer itran, levi, levf, ilevel, imax
C
      write(*,*) ' Molecular transition data file ? '
      read(*,10) specfile
10    format(A)
C
      write(*,*) ' Excitation temperature ?'
      read(*,*) Tx
      write(*,*) ' Output file ? '
      read(*,10) outfile
C
      call getspec(specfile)
C
      open(UNIT=3, FILE=outfile, status='UNKNOWN')
      write(3,10) specfile
      write(3,*) ' Tx = ', Tx
      write(3,*)
C
C     sum partition function
C
      Qpart = 0.
      do 20 ilevel=1, numlev
         Qpart = Qpart + degen(ilevel)*exp(-energy(ilevel)/Tx)
20    continue
      write(3,*) ' Partition function = ', Qpart
C
C     get fractional populations
C
      do 30 ilevel=1, numlev
         fpop(ilevel) = degen(ilevel)*exp(-energy(ilevel)/Tx)
     +                  /Qpart
30    continue
C
C     Now calculate column densities
C
      max = 0.
      imax = 0
      do 40 itran=1,numtran
         wavelen = clight/(frequency(itran)*amegahz)
         nupper(itran) = 8*PI*kboltz*akm/((wavelen**3)*hplanck
     +             *frequency(itran)*amegahz*acoeff(itran))
         levi = linitial(itran)
         levf = lfinal(itran)
         x1 = fpop(levi)/nupper(itran)
         if(x1.GT.max) then
            max = x1
         endif
40    continue
C
      write(3,41) 1./max
41    format(' Integral T dV = relative intensity',
     +       ' * total column density',
     +       ' / ', E12.5)
      write(3,*)
      write(3,31)
31    format(' frequen ', ' deg', ' energy ',
     +       '  col density', ' src fun', '   frac pop ',
     +       '  rel inten ', '   transition')
C
      do 50 itran=1,numtran
         srcfun = tphoton(itran)/(exp(tphoton(itran)/Tx)-1.)
         levi = linitial(itran)
         levf = lfinal(itran)
         x1 = fpop(levi)/nupper(itran)
         x1 = x1/max
         write(3, 45) frequency(itran)/1000.,
     +                degen(levi),
     +                energy(levi)-energy(1),
     +                nupper(itran), srcfun,
     +                fpop(levi),
     +                x1,
     +                elabel(levi), elabel(levf)
45       format(1X, F8.2, F4.0, 1X, F7.2, 1X,
     +          E11.4, 1X, F6.2, 1x, E11.4,
     +          1x, E11.4, 2X, A8, '-', A8)
50    continue
C
      close(3)
      stop
      end
C
C********************************************************************
      subroutine getspec(infile)
C********************************************************************
C
C     read energy levels, A coefficients from disk
C
      include 'spectrum.inc'
      include 'cgs_constants.inc'
C
      character*(*) infile
      character*120 textline
      integer i, j, li, lf
      real*4 x
C
1     format(A)
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
      read(1,*)                 ! skip comment line
      read(1,*) numlev
      if(numlev.GT.MAXLEV) then
         write(*,*)
     +     ' Too many energy levels in molecular transitions file'
         write(*,*) ' Number of levels = ', numlev
         write(*,*) ' You may need to recompile with larger MAXLEV'
         close(1)
         stop
      endif
C
      read(1,*)                 ! skip comment line
      do 20 i=1, numlev
         read(1,1) elabel(i)
20    continue
C
      read(1,*)                 ! skip comment line
C
      do 30 i=1, numlev
         read(1,1) textline
         read(textline,*) j, energy(i), degen(i)
30    continue
C
      read(1,*)                 ! skip comment line
C
C
C     Clear A-coefficient matrix
C
      do 50 i=1, MAXTRAN
         acoeff(i) = 0.
50    continue
C
      numtran = 0                            ! no transitions yet
C
60    continue
         read(1,1,END=70) textline
         read(textline, *) li, lf, x

         if(li.LT.0 .OR. li.GT.numlev .OR.    ! subscripts out of bounds
     +      lf.LT.0 .OR. lf.GT.numlev) then
C
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Bad energy level number:'
            write(*,1) textline
            close(1)
            stop
         endif
C
         if(energy(li).LE.energy(lf)) then
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Cannot have spontaneous upwards transitions'
            write(*,1) textline
            close(1)
            stop
         endif
C
         numtran = numtran + 1     
         acoeff(numtran) = x         ! looks OK, so stuff it
         linitial(numtran) = li
         lfinal(numtran) = lf
         x = energy(li) - energy(lf)
         tphoton(numtran) = x            ! photon temperature in K
         x = x*kboltz/(hplanck*amegahz)  ! convert to MHz
         frequency(numtran) = x
C
      if(numtran .LT. MAXTRAN) goto 60   ! process next transition
      write(*,*) ' Warning: may have too many transitions in file'
      write(*,*) ' Maximum number is ', MAXTRAN
C
70    continue
C
C     now done reading transitions...
C
      close(1)
C
      return
      end
C
