C
C     grid.inc
C
C     Defines all parameters relating to the radial grid
C
      parameter(MAXGRID=200)          ! Maximum number of radial shells
      parameter(MAXVEL=30)            ! Maximum number of velocities
C
      real*8 rscalecm                 ! radius scale factor in cm
                                      ! = unit for all distances
C
      real*8 rmin, rmax               ! minimum and maximum radii
C
C     thickratio is the ratio of column densities of the innermost
C     shell (between r(2) and r(3)) and the outermost shell
C     (between r(nshell-1) and r(nshell). This is used for
C     logarithmic grids only.
C
      real*8 thickratio
C
      integer nshell                  ! number of radial shells
C
C     We don't allow for a velocity field;
C     we also assume that the line width is constant with radius
C
      real*8 linewidth                ! FWHM spectral line width in km/s
      real*8 ewidth                   ! 1/e half-width in km/s
      integer numvel                  ! and # of velocities across line
C
C     We allow a radial dependence of
C
C          a + b*(r/r0)**power
C
C     for the H2 density, gas temperature, dust temperature,
C     and molecular abundance:
C
      real*8 h2den0, h2den1, h2denradius, h2denpower
      real*8 Tgas0, Tgas1, Tgasradius, Tgaspower, Tgasmin
      real*8 Tdust0, Tdust1, Tdustradius, Tdustpower, Tdustmin
      real*8 abun0, abun1, abunradius, abunpower
      real*8 colfg, tempfg            ! foreground gas & dust
C
C
      real*8 rshell(MAXGRID)          ! shell radii normalized
                                      ! to lie between 0 and 1
C
      real*8 h2den(MAXGRID)           ! values of these parameters
      real*8 Tgas(MAXGRID)            ! are evaluated midway
      real*8 Tdust(MAXGRID)           ! between two shell radii
      real*8 abun(MAXGRID)            !
      real*8 Ncol(MAXGRID)            !
C
      common/grid/rscalecm, rmin, rmax, thickratio, nshell,
     +            linewidth, ewidth, numvel, rshell,
     +            h2den, Tgas, Tdust, abun, Ncol
C
      common/h2dendef/ h2den0, h2den1, h2denradius, h2denpower
      common/Tgasdef/  Tgas0, Tgas1, Tgasradius, Tgaspower, Tgasmin
      common/Tdustdef/ Tdust0, Tdust1, Tdustradius, Tdustpower,
     +                 Tdustmin
      common/abundef/  abun0, abun1, abunradius, abunpower
      common/fg/colfg, tempfg
C
