C************************************************************************
      program makegrid
C************************************************************************
C
C     J. Zmuidzinas  12-9-93
C
C     Prepares a radial grid for input to the radiative transfer
C     program
C
C************************************************************************
C
      call getgridinfo
      call calcgrid
      call savegrid
      stop
      end
C
C************************************************************************
      subroutine getgridinfo
C************************************************************************
C
      include 'grid.inc'
C
      character*80 infile
C
      write(*,*) ' Input file containing parameters defining the grid ?'
      read(*,10) infile
10    format(A)
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
      read(1,*) rscalecm
      if(rscalecm.LE.0) then
         write(*,*) ' Error in parameter file - rscalecm <= 0 !!'
         close(1)
         stop
      endif
C
      read(1,*) rmin, rmax
      if(rmin.GE.rmax) then
         write(*,*) ' Error in parameter file - rmin >= rmax !!'
         close(1)
         stop
      endif
C
      read(1,*) nshell
      if(nshell.LT.3) then
         write(*,*) ' Error in parameter file - '
         write(*,*) ' Need at least 3 radial shells!'
         write(*,*) ' Found: ', nshell
         close(1)
         stop
      else if(nshell.GT.MAXGRID) then
         write(*,*) ' Error in parameter file - '
         write(*,*) ' Too many radial shells: ', nshell
         write(*,*) ' MAXGRID = ', MAXGRID
         write(*,*)
     +   ' Reduce number of shells or recompile with larger MAXGRID'
         close(1)
         stop
       endif
C
      read(1,*) linewidth, numvel
      if(linewidth .LE.0) then
         write(*,*) ' Error in parameter file - '
         write(*,*) ' Linewidth is <= 0. !! '
         close(1)
         stop
       endif
      if(numvel.LT.2) then
         write(*,*) ' Error in parameter file - '
         write(*,*) ' Need >= 2 velocities across line '
         close(1)
         stop
       endif
      if(numvel.GT.MAXVEL) then
         write(*,*) ' Error in parameter file - '
         write(*,*) ' More than', MAXVEL, ' velocities across line'
         write(*,*) ' Reduce or recompile with larger MAXVEL'
         close(1)
         stop
       endif
C
      read(1,*) h2den0, h2den1, h2denradius, h2denpower
      if(h2den0.LT.0. .OR. h2den1.LT.0 .OR. h2denradius.LT.0.) then
         write(*,*) ' Error in parameter file - h2den law invalid'
         close(1)
         stop
      endif
C
      read(1,*) Tgas0, Tgas1, Tgasradius, Tgaspower
      if(Tgas0.LT.0. .OR. Tgas1.LT.0 .OR. Tgasradius.LT.0.) then
         write(*,*) ' Error in parameter file - Tgas law invalid'
         close(1)
         stop
      endif
C
      read(1,*) Tdust0, Tdust1, Tdustradius, Tdustpower
      if(Tdust0.LT.0. .OR. Tdust1.LT.0 .OR. Tdustradius.LT.0.) then
         write(*,*) ' Error in parameter file - Tdust law invalid'
         close(1)
         stop
      endif
C
      read(1,*) abun0, abun1, abunradius, abunpower
      if(abun0.LT.0. .OR. abun1.LT.0 .OR. abunradius.LT.0.) then
         write(*,*) ' Error in parameter file - abundance law invalid'
         close(1)
         stop
      endif
C
      close(1)
      return
      end
C
C************************************************************************
      subroutine savegrid
C************************************************************************
C
      include 'grid.inc'
C
      character*80 outfile
      integer ir
C
      write(*,*) ' Output file name for saving grid ? '
      read(*,10) outfile
10    format(A)
C
      open(UNIT=1, FILE=outfile, STATUS='UNKNOWN')
C
      write(1,*) ' Defining parameters for grid: '
      write(1,20) rscalecm,
     +           '! rscalecm'
20    format(1X, E12.5, T60, A)
      write(1,30) rmin, rmax,
     +           '! rmin, rmax'
30    format(1X, E12.5, 1X, E12.5, T60, A)
      write(1,40) nshell,
     +           '! nshell'
40    format(1X, I10, T60, A)
      write(1,20) linewidth, '! FWHM linewidth'
      write(1,40) numvel, '! # velocities'
      write(1,50) h2den0, h2den1, h2denradius, h2denpower,
     +           '! h2den'
      write(1,50) Tgas0, Tgas1, Tgasradius, Tgaspower,
     +           '! Tgas'
      write(1,50) Tdust0, Tdust1, Tdustradius, Tdustpower,
     +           '! Tdust'
      write(1,50) abun0, abun1, abunradius, abunpower,
     +           '! abundance'
50    format(1X, E12.5, 1X, E12.5, 1X, E12.5, 1X, E12.5,
     +       T60, A)
      write(1,*)
C
      write(1,55) 'rshell', 'h2den', 'Tgas', 'Tdust',
     +            'abun', 'Ncol'
55    format(3X, A, T17, A, T30, A, T43, A, T56, A, T69, A)
C
C     Write out parameters as a function of radius
C     Note that radius is not normalized to 1 yet
C
      do 200 ir=1, nshell-1
         write(1,60) rshell(ir), h2den(ir),
     +              Tgas(ir), Tdust(ir), abun(ir), Ncol(ir)
60    format(1X, E12.5, 1X, E12.5, 1X, E12.5, 1X, E12.5,
     +       1X, E12.5, 1X, E12.5)
200   continue
C
      close(1)
      return
      end
C
C
C************************************************************************
      real*8 function h2denfun(rad)
C************************************************************************
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      result = h2den0 + h2den1*((rad/h2denradius)**h2denpower)
      h2denfun = result
      return
      end
C
C************************************************************************
      real*8 function Tgasfun(rad)
C************************************************************************
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      result = Tgas0 + Tgas1*((rad/Tgasradius)**Tgaspower)
      Tgasfun = result
      return
      end
C
C************************************************************************
      real*8 function Tdustfun(rad)
C************************************************************************
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      result = Tdust0 + Tdust1*((rad/Tdustradius)**Tdustpower)
      Tdustfun = result
      return
      end
C
C************************************************************************
      real*8 function abunfun(rad)
C************************************************************************
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      result = abun0 + abun1*((rad/abunradius)**abunpower)
      abunfun = result
      return
      end
C
C************************************************************************
      real*8 function columnfun(rad)
C************************************************************************
C
C     Calculates radial column density from r=rmin to r=rad
C
      real*8 rad
      real*8 result
C
      include 'grid.inc'
C
      if(h2denppower .NE. -1.) then
         result = h2den0*(rad-rmin)
     +       + (h2den1/((h2denradius**h2denpower)*(h2denpower+1.)))
     +       *(rad**(h2denpower+1.) - rmin**(h2denpower+1.))
      else
         result = h2den0*(rad-rmin)
     +       + (h2den1*h2denradius)*dlog(rad/rmin)
      endif
C
      result = result*rscalecm     ! put distance scale in
                                   ! so column density is in cm-2
      columnfun = result
      return
      end
C
C
C************************************************************************
      subroutine shellcount(rad, sh, dsh)
C************************************************************************
C
C     Calculates continuous "shell count" function and derivative
C     Shell radii determined by solving sh = k, k=2, 3, ..., nshell-1
C
      real*8 rad
      real*8 sh, dsh
C
      include 'grid.inc'
C
      real*8 colmax         ! column density at rmax
      real*8 shell          ! gets subtracted for root-finding
C
      common/shellcnt/ colmax, shell
C
      real*8 columnfun
      external columnfun
C
      sh = (nshell-2)*columnfun(rad)/colmax + 2.
      sh = sh - shell
      dsh = (nshell-2)*h2denfun(rad)*rscalecm/colmax
      return
      end
C
C************************************************************************
      subroutine calcgrid
C************************************************************************
C
C     Calculates a radial grid which has equal column density intervals
C
C     Evaluates physical parameters (density, temperatures, etc.)
C     at radii midway between shell boundaries
C
      real*8 ACC
      parameter (ACC=1.E-6)     ! accuracy for root-finder
C
      include 'grid.inc'
C
      integer ir
C
      real*8 h2denfun
      real*8 Tgasfun
      real*8 Tdustfun
      real*8 abunfun
      real*8 columnfun
C
      external h2denfun
      external Tgasfun
      external Tdustfun
      external abunfun
      external columnfun
C
      external shellcount
C
      real*8 rtsafe              ! Numerical Recipes root-finder
      external rtsafe
C
      real*8 colmax              ! column density at rmax
      real*8 shell               ! gets subtracted for root-finding
C
      common/shellcnt/ colmax, shell
C
      colmax = columnfun(rmax)   ! get maximum column density
C
      rshell(1) = 0.             ! first radius = 0
      rshell(2) = rmin           ! second radius at rmin
      rshell(nshell) = rmax      ! last radius at rmax
C
      do 10 ir=3, nshell-1
         shell = ir              ! this is the shell we're solving for
         rshell(ir) = rtsafe(shellcount, rshell(ir-1), rmax, ACC)
10    continue
C
C     We assume that the core is uniform, with parameters
C     evaluated at the core boundary, rmin.
C
      h2den(1) = h2denfun(rshell(2))
      Tgas(1) = Tgasfun(rshell(2))
      Tdust(1) = Tdustfun(rshell(2))
      abun(1) = abunfun(rshell(2))
      Ncol(1) = columnfun(rshell(2))   ! should be zero
C
C     Calculate other parameters midway between these radii
C
      do 20 ir=2, nshell-1
         h2den(ir) = h2denfun(0.5*(rshell(ir) + rshell(ir+1)))
         Tgas(ir) = Tgasfun(0.5*(rshell(ir) + rshell(ir+1)))
         Tdust(ir) = Tdustfun(0.5*(rshell(ir) + rshell(ir+1)))
         abun(ir) = abunfun(0.5*(rshell(ir) + rshell(ir+1)))
C
C        Except for column density, which is evaluated from
C        rmin to the shell radius rshell(ir+1)
C
         Ncol(ir) = columnfun(rshell(ir+1))
20    continue
C
      return
      end
C
C
C     Numerical Recipes routines...
C
C************************************************************************
      REAL*8 FUNCTION rtsafe(funcd,x1,x2,xacc)
C************************************************************************
      INTEGER MAXIT
      REAL*8 result,x1,x2,xacc
      EXTERNAL funcd
      PARAMETER (MAXIT=100)
      INTEGER j
      REAL*8 df,dx,dxold,f,fh,fl,temp,xh,xl
C
      call funcd(x1,fl,df)
      call funcd(x2,fh,df)
      if((fl.gt.0..and.fh.gt.0.).or.(fl.lt.0..and.fh.lt.0.))pause
     *'root must be bracketed in rtsafe'
      if(fl.eq.0.)then
        rtsafe=x1
        return
      else if(fh.eq.0.)then
        rtsafe=x2
        return
      else if(fl.lt.0.)then
        xl=x1
        xh=x2
      else
        xh=x1
        xl=x2
      endif
      result=.5*(x1+x2)
      dxold=abs(x2-x1)
      dx=dxold
      call funcd(result,f,df)
      do 11 j=1,MAXIT
        if(((result-xh)*df-f)*((result-xl)*df-f).ge.0..or. abs(2.*
     *f).gt.abs(dxold*df) ) then
          dxold=dx
          dx=0.5*(xh-xl)
          result=xl+dx
          if(xl.eq.result) then
             rtsafe = result
             return
          endif
        else
          dxold=dx
          dx=f/df
          temp=result
          result=result-dx
          if(temp.eq.result) then
             rtsafe = result
             return
          endif
        endif
        if(abs(dx).lt.xacc) then
           rtsafe = result
           return
        endif
        call funcd(result,f,df)
        if(f.lt.0.) then
          xl=result
        else
          xh=result
        endif
11    continue
      pause 'rtsafe exceeding maximum iterations'
      return
      END
