C************************************************************************
      program molndust
C************************************************************************
C
C     J. Zmuidzinas  12-12-93
C
C     Radiative transfer and statistical equilibrium program
C     for arbitrary spherical distributions of gas and dust
C
C************************************************************************
C
      character*80 savefile
C
      call getgrid                         ! read in radial grid
      call getspec                         ! read in energy levels, etc.
      call getcollrate                     ! collision rates
      call getdust                         ! dust parameters
C
      write(*,*) ' Output file ? '         ! get file name before
      read(*,10) savefile                  ! grinding through the
10    format(A)                            ! calculation
C
      call selfconsistent                  ! iterate level populations
C                                          ! and radiative transfer
      call saveresults(savefile)           ! write results to disk
      stop
      end
C
C
C************************************************************************
      subroutine saveresults(fname)
C************************************************************************
C
      character*(*) fname
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
      include 'cgs_constants.inc'
C
      integer ir, ilev, itran, li, lf
C
      open(UNIT=2, NAME=fname, STATUS='UNKNOWN')
C
      write(2,*) ' Defining parameters for grid: '
      write(2,20) rscalecm,
     +           '! rscalecm'
20    format(1X, E12.5, T60, A)
      write(2,30) rmin, rmax,
     +           '! rmin, rmax'
30    format(1X, E12.5, 1X, E12.5, T60, A)
      write(2,40) nshell,
     +           '! nshell'
40    format(1X, I10, T60, A)
      write(2,20) linewidth, '! FWHM linewidth'
      write(2,40) numvel, '! # velocities'
      write(2,50) h2den0, h2den1, h2denradius, h2denpower,
     +           '! h2den'
      write(2,50) Tgas0, Tgas1, Tgasradius, Tgaspower,
     +           '! Tgas'
      write(2,50) Tdust0, Tdust1, Tdustradius, Tdustpower,
     +           '! Tdust'
      write(2,50) abun0, abun1, abunradius, abunpower,
     +           '! abundance'
50    format(1X, E12.5, 1X, E12.5, 1X, E12.5, 1X, E12.5,
     +       T60, A)
C
C
      do 100 ir=1, nshell-1
         write(2,*)
         write(2,*) 'Radial shell #: ', ir
         write(2,55) 'rshell', 'h2den', 'Tgas', 'Tdust',
     +               'abun', 'Ncol'
55       format(3X, A, T17, A, T30, A, T43, A, T56, A, T69, A)
         write(2,60) rshell(ir)*rmax, h2den(ir),
     +              Tgas(ir), Tdust(ir), abun(ir), Ncol(ir)
60       format(1X, E12.5, 1X, E12.5, 1X, E12.5, 1X, E12.5,
     +       1X, E12.5, 1X, E12.5)
C
         write(2,62) 'Level', 'Frac. pop.', 'Label'
62       format(A, T7, A, T22, A)
         do 70 ilev=1, numlev
            write(2,65) ilev, fracpop(ir, ilev), elabel(ilev)
65          format(I5, 1X, E12.5, 3X, A10)
70       continue
         write(2,72) 'Transition', 'Frequency', 'Avg. Inten. (K)'
72       format(T5, A, T24, A, T37, A)
         do 80 itran=1, numtran
            li = linitial(itran)
            lf = lfinal(numtran)
            write(2,75) elabel(li), elabel(lf), frequency(itran),
     +                  Jtot(ir, itran)
75          format(A10, '-', A10, 3X, F10.0, 3X, E12.5)
80       continue
100   continue
C
      close(2)
      return
      end
C
C************************************************************************
      subroutine getgrid
C************************************************************************
C
      include 'grid.inc'
C
      character*80 infile
      integer ir
C
      write(*,*) ' Input file name for radial grid ? '
      read(*,10) infile
10    format(A)
C
      open(UNIT=1, FILE=infile, STATUS='UNKNOWN')
C
      read(1,*)
      read(1,*) rscalecm
      read(1,*) rmin, rmax
      read(1,*) nshell
      read(1,*) linewidth               ! this is FWHM in km/s
C
C     calculate 1/e half-width of line
C
      ewidth = linewidth/(2.*sqrt(log(2.)))
C
      read(1,*) numvel
      read(1,*)  h2den0, h2den1, h2denradius, h2denpower
      read(1,*)  Tgas0, Tgas1, Tgasradius, Tgaspower
      read(1,*)  Tdust0, Tdust1, Tdustradius, Tdustpower
      read(1,*)  abun0, abun1, abunradius, abunpower
      read(1,*)
      read(1,*)
C
C     Read in parameters as a function of radius
C     Note that radius is not normalized to 1 yet
C
      do 200 ir=1, nshell-1
         read(1,*)  rshell(ir), h2den(ir),
     +              Tgas(ir), Tdust(ir), abun(ir), Ncol(ir)
200   continue
C
      close(1)
C
C     Normalize shell radii now
C
      do 300 ir=1, nshell-1
         rshell(ir) = rshell(ir)/rmax
300   continue
      rshell(nshell) = 1.
C
      return
      end
C
C
C**********************************************************************
      subroutine getcollrate
C**********************************************************************
C
C     reads collisional rate table from disk
C
      include 'collrate.inc'
C
      character*80 collfile
      character*200 textline
      integer i, levi, levf
C
      write(*,*) ' Collisional excitation rates file ? '
      read(*,10) collfile
10    format(A)
C
      open(UNIT=1, FILE=collfile, STATUS='OLD')
C
C     zero out collisional rates
C
      do 40 i=1, MAXT
         tcoll(i) = 0.
         do 30 levi=1, MAXL
            do 20 levf=1, MAXL
               collrate(i, levi, levf) = 0.
20          continue
30       continue
40    continue
C
      read(1,*) numtcoll
      if(numtcoll .GT. MAXT) then
         write(*,*) ' Too many temperatures for collisional rates'
         write(*,*) ' Number of Temps = ', numtcoll
         write(*,*) ' Recompile with larger MAXT if necessary'
         close(1)
         stop
      endif
C
      do 50 i=1, numtcoll
         read(1,*) tcoll(i)
50    continue
C
      ncollev = 0
60    continue
C
C
         read(1,10, END=70) textline
         read(textline,*) levi, levf
         if(ncollev.LT.levi)  ncollev = levi
         if(ncollev.LT.levf)  ncollev = levf
         if(ncollev.GT.MAXL) then
            write(*,*)
     +         ' Too many energy levels in collisional rates file'
            write(*,*) ' Number of levels = ', ncollev
            write(*,*) ' Encountered in rate file on line '
            write(*,*) '*** ', textline, ' ***'
            write(*,*) ' Recompile with larger MAXL if necessary'
            close(1)
            stop
         endif
         if(levi.LT.1 .OR. levf.LT.1) then
            write(*,*) ' Error in collisional rate file'
            write(*,*) ' Energy level labels must be >= 1'
            write(*,*) '*** ', textline, ' ***'
            close(1)
            stop
         endif
C
C        All seems to be OK, so read the rates
C
         read(textline, *)
     +        levi, levf, (collrate(i,levi,levf), i=1, numtcoll)
      goto 60
C
70    continue
      close(1)
      return
      end
C
C**********************************************************************
      real*4 function collratecalc(li, lf, temp)
C**********************************************************************
C
C     interpolates collisional rate table
C
C     li = initial level
C     lf = final level
C     temp = temperature
C
      integer li, lf
      real*8 temp
C
      include 'collrate.inc'
C
      integer indx
      real result, x, x1, x2, y1, y2
C
      if(li.GT.ncollev .OR. lf.GT.ncollev) then
         result = 0.0
         goto 900
      endif
C
      if(temp.LE.tcoll(1)) then
         result = collrate(1, li, lf)
         goto 900
      endif
C
      if(temp.GE.tcoll(numtcoll)) then
         result = collrate(numtcoll, li, lf)
         goto 900
      endif
C
      indx = 1
10    continue
         indx = indx+1
      if(temp.GT.tcoll(indx) .AND. indx.LT.numtcoll) goto 10
C
C     now do a power law interpolation
C
      x = log(temp)
      x1 = log(tcoll(indx-1))
      x2 = log(tcoll(indx))
      y1 = log(collrate(indx-1, li, lf))
      y2 = log(collrate(indx, li, lf))
C
      result = (y1*(x2-x) + y2*(x-x1))/(x2-x1)
      result = exp(result)
C
900   continue
      collratecalc = result
      return
      end
C
C********************************************************************
      subroutine getspec
C********************************************************************
C
C     read energy levels, A coefficients from disk
C
      include 'spectrum.inc'
      include 'cgs_constants.inc'
C
      character*120 specfile, textline
      integer i, j, li, lf
      real*4 x
C
      write(*,*) ' Molecular transition data file ? '
      read(*,1) specfile
1     format(A)
C
      open(UNIT=1, FILE=specfile, STATUS='OLD')
C
      read(1,*)                 ! skip comment line
      read(1,*) numlev
      if(numlev.GT.MAXLEV) then
         write(*,*)
     +     ' Too many energy levels in molecular transitions file'
         write(*,*) ' Number of levels = ', numlev
         write(*,*) ' You may need to recompile with larger MAXLEV'
         close(1)
         stop
      endif
C
      read(1,*)                 ! skip comment line
      do 20 i=1, numlev
         read(1,1) elabel(i)
20    continue
C
      read(1,*)                 ! skip comment line
C
      do 30 i=1, numlev
         read(1,1) textline
         read(textline,*) j, energy(i), degen(i)
30    continue
C
      read(1,*)                 ! skip comment line
C
C
C     Clear A-coefficient matrix
C
      do 50 i=1, MAXTRAN
         acoeff(i) = 0.
50    continue
C
      numtran = 0                            ! no transitions yet
C
60    continue
         read(1,1,END=70) textline
         read(textline, *) li, lf, x

         if(li.LT.0 .OR. li.GT.numlev .OR.    ! subscripts out of bounds
     +      lf.LT.0 .OR. lf.GT.numlev) then
C
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Bad energy level number:'
            write(*,1) textline
            close(1)
            stop
         endif
C
         if(energy(li).LE.energy(lf)) then
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Cannot have spontaneous upwards transitions'
            write(*,1) textline
            close(1)
            stop
         endif
C
         numtran = numtran + 1     
         acoeff(numtran) = x         ! looks OK, so stuff it
         linitial(numtran) = li
         lfinal(numtran) = lf
         x = energy(li) - energy(lf)
         tphoton(numtran) = x            ! photon temperature in K
         x = x*kboltz/(hplanck*amegahz)  ! convert to MHz
         frequency(numtran) = x
C
      if(numtran .LT. MAXTRAN) goto 60   ! process next transition
      write(*,*) ' Warning: may have too many transitions in file'
      write(*,*) ' Maximum number is ', MAXTRAN
C
70    continue
C
C     now done reading transitions...
C
      close(1)
C
C     so sort them by ascending frequency, including
C     the initial and final state indices
C
      call PIKSR3(numtran, frequency, linitial, lfinal)
C
      return
      end
C
C********************************************************************
      SUBROUTINE PIKSR3(N,ARR,BRR,CRR)
C********************************************************************
      REAL*8 ARR(N), A
      INTEGER BRR(N), CRR(N), B, C
      DO 12 J=2,N
        A=ARR(J)
        B=BRR(J)
        C=CRR(J)
        DO 11 I=J-1,1,-1
          IF(ARR(I).LE.A)GO TO 10
          ARR(I+1)=ARR(I)
          BRR(I+1)=BRR(I)
          CRR(I+1)=CRR(I)
11      CONTINUE
        I=0
10      ARR(I+1)=A
        BRR(I+1)=B
        CRR(I+1)=C
12    CONTINUE
      RETURN
      END
C
C**********************************************************************
      subroutine getdust
C**********************************************************************
C
C     reads dust parameters from disk
C
      include 'dust.inc'
C
      character*80 dustfile
C
      write(*,*) ' Dust parameters file ? '
      read(*,10) dustfile
10    format(A)
C
      open(UNIT=1, FILE=dustfile, STATUS='OLD')
      read(1,*) Qdust0
      read(1,*) wavdust0
      read(1,*) slopedust
      read(1,*) adust
      read(1,*) rhodust
      read(1,*) Rgasdust
      read(1,*) gasmu
      close(1)
      return
      end
C
C**********************************************************************
      real*8 function sigmadust(wavlen)
C**********************************************************************
C
C     Calculates dust cross-section (in cm-2) per H2 molecule
C
      real*8 wavlen                     ! wavelength in microns
C
      include 'dust.inc'
      include 'cgs_constants.inc'       ! for amicron = 1.E-4
C
      real*8 Qdust, result
C
      Qdust = Qdust0*(wavdust0/wavlen)**slopedust    ! efficiency
      result = 3*gasmu*Qdust/(4.*adust*amicron*rhodust*Rgasdust)
      sigmadust = result
      return
      end
C
C
C********************************************************************
      subroutine selfconsistent
C********************************************************************
C
C     Use accelerated lambda iteration method of
C     Rybicki and Hummer, A+A 245, 171  (1991)
C     to simultaneously solve for statistical equilibrium and
C     radiative transfer.
C
C********************************************************************
C
      logical done, first
      integer itercount
C
      call initialize
      done = .FALSE.                ! convergence flag
      first = .TRUE.                ! first time thru
      itercount = 0
C
10    continue
         if(.NOT.first) then        ! don't do transfer first time
            write(*,*) ' Solving transfer for iteration #', itercount
            call solvetransfer
         endif
C
         write(*,*) ' Solving rates for iteration #', itercount
         call solverates            ! solve for fractional populations
C
         if(.NOT.first) then        ! don't check convergence 1st time
            call checkconvergence(done)
         else
            first = .FALSE.
         endif
C
         itercount = itercount+1
      if(.NOT.done) goto 10
C
      return
      end
C
C********************************************************************
      subroutine initialize
C********************************************************************
C
C     Initialize radiative transfer calculation
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
C
      integer ishell, itran
C
C     ***first, zero lambda-* and Jeff
C
      do 20 itran=1, numtran
         do 10 ishell = 1, nshell-1
            Jeff(ishell, itran) = 0.
            lamstar(ishell, itran) = 0.
10       continue
20    continue
C
C     ***then calculate abscissas and weights for Gauss-Hermite integration
C        Since the line is symmetric, we only do positive velocities
C
      call gauher(xinteg, winteg, 2*numvel)
C
      return
      end
C
C********************************************************************
      subroutine solverates
C********************************************************************
C
C     ***solve statistical equilibrium equations for each
C        radial shell, given lambda-* and Jeff from radiative
C        transfer calculation
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
C
      real*4 collratecalc              ! function to calculate coll rates
      external collratecalc
C
      real*4 rates(MAXLEV, MAXLEV)     ! rate matrix
      real*4 cpyrates(MAXLEV, MAXLEV)  ! copy of rate matrix
      real*4 fpop(MAXLEV)              ! temporary storage for frac. pop.
      real*4 rhs(MAXLEV)               ! right-hand side of rate eqn
      real*4 ratesdet                  ! for determinant sign
      integer ratesindex(MAXLEV)       ! for LU decomposition
C
      integer ishell, itran, ilev, jlev
      real*8 xtmp
C
      do 100 ishell=1, nshell-1        ! solve for each shell
C
C        ***zero rate matrix
C
         do 20 ilev=1, numlev
            do 10 jlev=1, numlev
               rates(ilev, jlev) = 0.
10          continue
20       continue
C
C        ***now add in collisional rates
C
         do 40 ilev=1, numlev
            do 30 jlev=1, numlev
               if(jlev.NE.ilev) then
                  xtmp = collratecalc(ilev, jlev, Tgas(ishell))
                  xtmp = xtmp*h2den(ishell)
                  rates(ilev, ilev) = rates(ilev, ilev) + xtmp
                  rates(jlev, ilev) = rates(jlev, ilev) - xtmp
               endif
30          continue
40       continue
C
C        ***now add in radiative rates
C
         do 50 itran=1, numtran           ! loop over all transitions
C
            ilev = linitial(itran)        ! get initial and final levels
            jlev = lfinal(itran)
C
C           ***do corrected spontaneous emission rate
C
            xtmp = acoeff(itran)*(1.-lamstar(ishell, itran))
            rates(ilev, ilev) = rates(ilev, ilev) + xtmp
            rates(jlev, ilev) = rates(jlev, ilev) - xtmp
C
C           ***next, do corrected stimulated emission rate
C              notice that Jeff is in units of Kelvin since it
C              is related to the antenna temperature; thus we
C              divide by the photon temp to get rate in units of Aij
C
            xtmp = acoeff(itran)*Jeff(ishell, itran)/tphoton(itran)
            rates(ilev, ilev) = rates(ilev, ilev) + xtmp
            rates(jlev, ilev) = rates(jlev, ilev) - xtmp
C
C           ***finally, do corrected absorption rate
C
            xtmp = xtmp*degen(ilev)/degen(jlev)
            rates(jlev, jlev) = rates(jlev, jlev) + xtmp
            rates(ilev, jlev) = rates(ilev, jlev) - xtmp
C
50       continue
C
C        ***the rate equations are redundant, since rates sum to zero
C           However, we need to impose normalization condition, so
C           we overwrite last row of rate matrix with 1's
C           We also need to make up the right-hand side of eqn
C           and to make a copy of the rate matrix.
C
         do 70 ilev=1, numlev
            rates(numlev, ilev) = 1.
            rhs(ilev) = 0.
            fpop(ilev) = 0.
            do 60 jlev=1, numlev
               cpyrates(jlev, ilev) = rates(jlev,ilev)
60          continue
70       continue
         rhs(numlev) = 1.       ! fractional populations add up to 1.
         fpop(numlev) = 1.
C
C        ***Now solve the linear system
C
         call ludcmp(cpyrates, numlev, MAXLEV, rateindex, ratesdet)
         call lubksb(cpyrates, numlev, MAXLEV, rateindex, fpop)
C
C        ***and polish up solution a bit
C
         call mprove(rates, cpyrates, numlev, MAXLEV,
     +               rateindex, rhs, fpop)
C
C        ***now save the solution
C
         do 90 ilev=1, numlev
            oldpop(ishell, ilev) = fracpop(ishell, ilev)
            fracpop(ishell, ilev) = fpop(ilev)
90       continue
C
C     ***go on to next radial shell
C
100   continue
C
      return
      end
C
C********************************************************************
      subroutine solvetransfer
C********************************************************************
C
C     solve the radiative transfer for each transition
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
      include 'cgs_constants.inc'
C
      real*8 sigmadust
      external sigmadust
C
      integer itran, ishell, ivel, levi, levf
C
      real*8 lamvel(MAXGRID, MAXVEL)     ! angle-averaged lambda-*
      real*8 uvel(MAXGRID, MAXVEL)       ! and Feautrier u before
C                                        ! velocity averaging
C
      real*8 opacity(MAXGRID, MAXVEL)    ! total opacity
      real*8 gasopacity(MAXGRID, MAXVEL) ! gas opacity
      real*8 sourcefn(MAXGRID, MAXVEL)   ! gas+dust source function
C
      real*8 sgas(MAXGRID)               ! gas source function
      real*8 sdust(MAXGRID)              ! dust source function
      real*8 dustopacity(MAXGRID)        ! dust opacity
C
      real*8 velprof(MAXVEL)             ! normalized velocity profile
C
      real*8 wavelen                     ! wavelength in cm
      real*8 xsectdust                   ! dust absorption cross-section
      real*8 xsectgas                    ! gas absorption cross-section
C
      real*8 x1, x2, xi, xf, gi, gf, tph
C
C     ***Calculate normalized velocity profile function in cgs units
C
      x1 = sqrt(PI)*ewidth*akm           ! ewidth is km/s; convert to cm/s
      do 10 ivel=1, numvel
         velprof(ivel) = exp(-xinteg(ivel)**2)/x1
10    continue
C
C     ***Loop over transitions
C
      do 100 itran=1, numtran
C
         levi = linitial(itran)
         levf = lfinal(itran)
         gi = degen(levi)
         gf = degen(levf)
C
C        ***transition wavelength in cm
C
         wavelen = clight/(frequency(itran)*amegahz)
C
C        ***dust absorption cross-section at this wavelength
C
         xsectdust = sigmadust(wavelen/amicron)
C
C        ***now calculate opacities in radial shells
C
         do 30 ishell=1, nshell-1
            dustopacity(ishell) = xsectdust*h2den(ishell)
     +                            *rmax*rscalecm
C
            xi = fracpop(ishell, levi)
            xf = fracpop(ishell, levf)
C
C           ***transition absorption cross-section per H2 molecule,
C              in cm-2 * Hz   (i.e. integrated over frequency)
C
            xsectgas = wavelen*wavelen*acoeff(itran)/(8.*PI)
            xsectgas = xsectgas*abun(ishell)
     +                 *xf*(gi/gf)*(1. - gf*xi/(gi*xf))
C
C           ***now multiply by frequency profile
C
            do 20 ivel=1, numvel
               gasopacity(ishell, ivel) = xsectgas*rmax*rscalecm
     +               *h2den(ishell)*velprof(ivel)*wavelen
C
C              ***and add to get total opacity
C
               opacity(ishell, ivel) = gasopacity(ishell, ivel) 
     +                                 + dustopacity(ishell)
20          continue
30       continue
C
C        ***now calculate gas and dust source functions
C
         sdust(nshell) = 0.
         sgas(nshell) = 0.
C
         tph = tphoton(itran)
C
         do 40 ishell=1, nshell-1
            x1 = tph/(exp(tph/Tdust(ishell))-1.)
            sdust(ishell) = x1
C
            xi = fracpop(ishell, levi)
            xf = fracpop(ishell, levf)
            x1 = tph/(gi*xf/(gf*xi) -1.)
            sgas(ishell) = x1
C
40       continue
C
C        ***now calculate total source function
C
         do 60 ishell=1, nshell-1
            x1 = dustopacity(ishell)*sdust(ishell)
            do 50 ivel=1, numvel
               x2 = gasopacity(ishell, ivel)*sgas(ishell)
               sourcefn(ishell, ivel) = (x1 + x2)
     +                                  /opacity(ishell, ivel)
50          continue
60       continue

C
C        ***we're ready to do the radiative transfer
C
         do 70 ivel=1, numvel
            call solver(rshell, nshell, opacity(1,ivel),
     +           sourcefn(1,ivel), uvel(1,ivel), lamvel(1,ivel))
70       continue
C
C        ***now sum intensities and lambda over velocity
C           note that lambda-operator gets multiplied by the
C           ratio of gas to total opacity, as in R+H, Eqn. 2.25
C
         do 90 ishell=1, nshell-1
C
            x1 = 0.
            x2 = 0.
            do 80 ivel=1, numvel
               x1 = x1 + winteg(ivel)*uvel(ishell,ivel)
               x2 = x2 + winteg(ivel)*lamvel(ishell, ivel)
     +             *gasopacity(ishell,ivel)/opacity(ishell,ivel)
80          continue
C
C           ***Normalize and save results
C              factor of 2 comes from doing only positive velocities
C               sqrt(PI) comes from normalization of exp(-x**2)
C
            Jtot(ishell, itran) = x1*2/sqrt(PI)
C
C            lamstar(ishell, itran) = x2*2/sqrt(PI)
C
C
C           ***Subtract off the part of intensity which comes
C              from the diagonal part of lambda to get Jeff
C              (R+H, eqn. 2.26)
C
C           Jeff(ishell, itran) = Jtot(ishell, itran) -
C    +                           lamstar(ishell,itran)*sgas(ishell)

            Jeff(ishell, itran) = Jtot(ishell, itran) 
C
90       continue
C
100   continue
C
      return
      end
c
C*******************************************************************
      subroutine solver(r,nr,chi,s,avu,avl)
C*******************************************************************
c
c     12-7-93   Darek Lis
c     12-15-93  J. Zmuidzinas (minor mods + comments)
c
c               Feautrier radiative transfer solver based on
c               Rybicki+Hummer, A+A 245, 171 (1991)
c
c     input:
c       r     - normalized shell radii (rgrid(1)=0, rgrid(nr)=1)
c       nr    - number of shells
c       chi   - absorption coefficient
c       s     - source function
c     output:
c       avu   - angle averaged u
c       avl   - angle averaged lambda operator
c
c     All input and output physical quantities (chi, s, avu, avl)
c     are evaluated AT THE SHELL MIDPOINTS, i.e. midway between
c     two shell radii. The convention is that chi(1) is the value
c     of chi midway between r(1) and r(2), etc., up to chi(nr-1)
c     which is chi between r(nr-1) and r(nr).
c
c     The routine calculates the Feautrier intensity u (avu)
c     and the diagonal part of the lambda operator (avl)
c     evaluated AT THE SHELL RADII. To do this, it must interpolate
c     the source function supplied to it at the shell midpoints
c     onto the shell boundaries. This is done at the beginning
c     of the routine. Also, the source function at r(1) and r(nr) is
c     set to zero.
c
c     When the calculation is done, avu and avl which were
c     calculated at the shell radii must be interpolated back
c     to the shell midpoints, so they can be directly used in the rate
c     equations for the level populations at the shell midpoints.
c     
C*******************************************************************
c
      include 'grid.inc'            ! for MAXGRID
C
      implicit real*8 (a-h,o-z)
      parameter (np=MAXGRID+1,cut=0.01)
      dimension r(nr),chi(nr),s(nr)
      dimension avu(nr),avl(nr)
      dimension dtau(np),a(np),b(np),c(np),d(np),e(np),h(np),
     1     f(np),z(np),u(np),rl(np)
C
C
C
      r(nr+1) = r(nr)               ! to avoid trouble in
C                                   ! angular integrations
c
c     interpolate source function onto radial shell boundaries
c
      do ir=1, nr-1
         a(ir) = s(ir)             ! copy into a() temporarily
      enddo
c
      s(1) = 0.                    ! source function vanishes at
      s(nr) = 0.                   ! the two boundaries
c
      do ir=2, nr-1                ! do the interpolation
         s(ir) = (chi(ir-1)*a(ir) + chi(ir)*a(ir-1))
     +           /(chi(ir-1) + chi(ir))
      enddo
c
c     initialize variables
c
      do ir=1,nr
         avu(ir)=0.
         avl(ir)=0.
      enddo
c
c     loop over impacts
      do ip=1,nr
         p=r(ip)
         tau=0.
c
c     calculate optical depth grid
         do ir=ip,nr-1
            dr=r(ir+1)-r(ir)
            if(ip.eq.1)then
               ds=dr
            else
               if(ir.eq.ip)then
                  fac=1.0
               else
                  fac=(r(ir+1)+r(ir))*dr/(r(ir+1)+p)/(r(ir+1)-p)
                  if(fac.lt.cut)then
                     fac=0.5*fac+0.125*fac*fac
                  else
                     fac=1.0-sqrt(1.0-fac)
                  endif
               endif
               ds=sqrt((r(ir+1)+p)*(r(ir+1)-p))*fac
            endif
            dtau(ir)=ds*chi(ir)
            tau=tau+dtau(ir)
         enddo
c
c     calculate a,b,c, and h coefficients - equations (A3)
         b(ip)=1./dtau(ip)
         c(ip)=1./dtau(ip)
         h(ip)=b(ip)-c(ip)
         do ir=ip+1,nr-1
            a(ir)=2./dtau(ir-1)/(dtau(ir-1)+dtau(ir))
            b(ir)=1.+2./dtau(ir)/dtau(ir-1)
            c(ir)=2./dtau(ir)/(dtau(ir-1)+dtau(ir))
            h(ir)=-a(ir)+b(ir)-c(ir)
         enddo
         a(nr)=1./dtau(nr-1)
         b(nr)=1.+1./dtau(nr-1)
         h(nr)=-a(nr)+b(nr)
c
c     calculate f and z coefficients - equations (A8)
         f(ip)=h(ip)/c(ip)
         z(ip)=0.
         do ir=ip+1,nr-1
            f(ir)=(h(ir)+a(ir)*f(ir-1)/(1.+f(ir-1)))/c(ir)
            z(ir)=(s(ir)+a(ir)*z(ir-1))/c(ir)/(1.+f(ir))
         enddo
c
c     watch out: c(nr)=0 - use equation (A4) instead of (A8)
         z(nr)=a(nr)*z(nr-1)/(b(nr)-a(nr)/(1.+f(nr-1)))
c
c     solve for u
         u(nr)=z(nr)
         do ir=nr-1,ip,-1
            u(ir)=u(ir+1)/(1.+f(ir))+z(ir)
         enddo
c
c     solve for lambda
c
         d(ip)=c(ip)/b(ip)
         do ir=ip+1,nr
            d(ir)=c(ir)/(b(ir)-a(ir)*d(ir-1))
         enddo
         e(nr)=a(nr)/b(nr)
         do ir=nr-1,ip,-1
            e(ir)=a(ir)/(b(ir)-c(ir)*e(ir+1))
         enddo
c
         rl(ip)=1./(1.-d(ip)*e(ip+1))/b(ip)
         do ir=ip+1,nr-1
            rl(ir)=1./(1.-d(ir)*e(ir+1))/(b(ir)-a(ir)*d(ir-1))
         enddo
         rl(nr)=1./(b(nr)-a(nr)*d(nr-1))
c
c     calculate angle averages
c
         if(ip.eq.1)then
            avu(1)=u(1)
            avl(1)=rl(1)
            do ir=2,nr
               dmu=0.5*(1.-sqrt(1.-(r(2)/r(ir))**2))
               avu(ir)=avu(ir)+u(ir)*dmu
               avl(ir)=avl(ir)+rl(ir)*dmu
            enddo
         else
            do ir=ip,nr
               if(ip.eq.ir)then
                  dmu=0.5*sqrt(1.-(r(ir-1)/r(ir))**2)
               else
c                  dmu=0.5*(sqrt(1.-(r(ip-1)/r(ir))**2)-
c     1                 sqrt(1.-(r(ip+1)/r(ir))**2))
c
                  fac=(r(ip+1)+r(ip-1))*(r(ip+1)-r(ip-1))/
     1                 (r(ir)+r(ip-1))/(r(ir)-r(ip-1))
                  if(fac.GE.1.0D0) then
                     fac = 1.0D0 - 1.E-6
                  endif
                  if(fac.lt.cut)then
                     fac=0.5*fac+0.125*fac*fac
                  else
                     fac=1.0-sqrt(1.0-fac)
                  endif
                  dmu=0.5*sqrt(1.-(r(ip-1)/r(ir))**2)*fac
c
               endif
               avu(ir)=avu(ir)+u(ir)*dmu
               avl(ir)=avl(ir)+rl(ir)*dmu
            enddo
         endif
      enddo
c
c     Now interpolate avu and avl from shell boundaries
c     onto shell midpoints
c
      do ir=1, nr           ! copy temporarily
         a(ir) = avu(ir)
         b(ir) = avl(ir)
      enddo
      avu(nr) = 0.          ! Only have nr-1 midpoints
      avl(nr) = 0.
      do ir=1, nr-1
         avu(ir) = 0.5*(a(ir)+a(ir+1))
         avl(ir) = 0.5*(b(ir)+b(ir+1))
      enddo
C
      return
      end
C
C********************************************************************
      subroutine checkconvergence(done)
C********************************************************************
C
C     check for convergence of the level populations
C
      logical done
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer.inc'
C
      integer ilev, ishell
      real*4 maxchange, change
C
      parameter(EPS=1.E-6)     ! for calculating fractional change
      parameter(CHMAX=1.E-4)   ! maximum relative change for convergence
C
C     find maximum relative change
C
      maxchange = 0.
      do 20 ishell = 1, nshell
         do 10 ilev = 1, numlev
            change = abs(fracpop(ishell, ilev) - oldpop(ishell, ilev))
            change = change/(abs(fracpop(ishell, ilev)) + EPS)
            if(maxchange.LT.change) then
               maxchange = change
            endif
10       continue
20    continue
C
C     Convergence test
C
      write(*,*) ' Maximum fractional population change is',
     +           maxchange
      done = (maxchange.LT.CHMAX)
C
      return
      end
C
C********************************************************************
      SUBROUTINE gauher(x,w,n)
C********************************************************************
      INTEGER n,MAXIT
      REAL*4 w(n),x(n)
      REAL*8 EPS,PIM4
      PARAMETER (EPS=3.D-14,PIM4=.7511255444649425D0,MAXIT=10)
      INTEGER i,its,j,m
      REAL*8 p1,p2,p3,pp,z,z1
      m=(n+1)/2
      do 13 i=1,m
        if(i.eq.1)then
          z=sqrt(float(2*n+1))-1.85575*(2*n+1)**(-.16667)
        else if(i.eq.2)then
          z=z-1.14*n**.426/z
        else if (i.eq.3)then
          z=1.86*z-.86*x(1)
        else if (i.eq.4)then
          z=1.91*z-.91*x(2)
        else
          z=2.*z-x(i-2)
        endif
        do 12 its=1,MAXIT
          p1=PIM4
          p2=0.d0
          do 11 j=1,n
            p3=p2
            p2=p1
            p1=z*sqrt(2.d0/j)*p2-sqrt(dble(j-1)/dble(j))*p3
11        continue
          pp=sqrt(2.d0*n)*p2
          z1=z
          z=z1-p1/pp
          if(abs(z-z1).le.EPS)goto 1
12      continue
        pause 'too many iterations in gauher'
1       x(i)=z
        x(n+1-i)=-z
        w(i)=2.d0/(pp*pp)
        w(n+1-i)=w(i)
13    continue
      return
      END
C
C********************************************************************
      SUBROUTINE ludcmp(a,n,np,indx,d)
C********************************************************************
      INTEGER n,np,indx(n),NMAX
      REAL d,a(np,np),TINY
      PARAMETER (NMAX=500,TINY=1.0e-20)
      INTEGER i,imax,j,k
      REAL aamax,dum,sum,vv(NMAX)
      d=1.
      do 12 i=1,n
        aamax=0.
        do 11 j=1,n
          if (abs(a(i,j)).gt.aamax) aamax=abs(a(i,j))
11      continue
        if (aamax.eq.0.) pause 'singular matrix in ludcmp'
        vv(i)=1./aamax
12    continue
      do 19 j=1,n
        do 14 i=1,j-1
          sum=a(i,j)
          do 13 k=1,i-1
            sum=sum-a(i,k)*a(k,j)
13        continue
          a(i,j)=sum
14      continue
        aamax=0.
        do 16 i=j,n
          sum=a(i,j)
          do 15 k=1,j-1
            sum=sum-a(i,k)*a(k,j)
15        continue
          a(i,j)=sum
          dum=vv(i)*abs(sum)
          if (dum.ge.aamax) then
            imax=i
            aamax=dum
          endif
16      continue
        if (j.ne.imax)then
          do 17 k=1,n
            dum=a(imax,k)
            a(imax,k)=a(j,k)
            a(j,k)=dum
17        continue
          d=-d
          vv(imax)=vv(j)
        endif
        indx(j)=imax
        if(a(j,j).eq.0.)a(j,j)=TINY
        if(j.ne.n)then
          dum=1./a(j,j)
          do 18 i=j+1,n
            a(i,j)=a(i,j)*dum
18        continue
        endif
19    continue
      return
      END
C
C********************************************************************
      SUBROUTINE lubksb(a,n,np,indx,b)
C********************************************************************
      INTEGER n,np,indx(n)
      REAL a(np,np),b(n)
      INTEGER i,ii,j,ll
      REAL sum
      ii=0
      do 12 i=1,n
        ll=indx(i)
        sum=b(ll)
        b(ll)=b(i)
        if (ii.ne.0)then
          do 11 j=ii,i-1
            sum=sum-a(i,j)*b(j)
11        continue
        else if (sum.ne.0.) then
          ii=i
        endif
        b(i)=sum
12    continue
      do 14 i=n,1,-1
        sum=b(i)
        do 13 j=i+1,n
          sum=sum-a(i,j)*b(j)
13      continue
        b(i)=sum/a(i,i)
14    continue
      return
      END
C
C********************************************************************
      SUBROUTINE mprove(a,alud,n,np,indx,b,x)
C********************************************************************
      INTEGER n,np,indx(n),NMAX
      REAL a(np,np),alud(np,np),b(n),x(n)
      PARAMETER (NMAX=500)
CU    USES lubksb
      INTEGER i,j
      REAL r(NMAX)
      DOUBLE PRECISION sdp
      do 12 i=1,n
        sdp=-b(i)
        do 11 j=1,n
          sdp=sdp+dble(a(i,j))*dble(x(j))
11      continue
        r(i)=sdp
12    continue
      call lubksb(alud,n,np,indx,r)
      do 13 i=1,n
        x(i)=x(i)-r(i)
13    continue
      return
      END
