C************************************************************************
      program observ
C************************************************************************
C
C     J. Zmuidzinas  1-4-94
C
C     Calculates emergent spectrum of arbitrary spherical
C     gas and dust cloud
C
C************************************************************************
C
      character*80 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
C
      write(*,*) ' Name of file created by molnd ? '
10    format(A)
      read(*,10) molndfile
      write(*,*) ' File name giving observation list ? '
      read(*,10) observfile
      write(*,*) ' Output file ? '
      read(*,10) outfile
C
      call getmolnd(molndfile)  
      call getobserv(observfile)
      call calcobserv(outfile)
C
      stop
      end
C
C************************************************************************
      subroutine getobserv(fname)
C************************************************************************
C
      character*(*) fname
      character*100 textline
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer_obs.inc'
      include 'cgs_constants.inc'
C
      integer itran, li, lf
      real*8 bw, bo
C
      open(UNIT=1, NAME=fname, STATUS='UNKNOWN')
C
      read(1,*) nspatial       ! no. of spatial integration pts in x and y
      read(1,*) velmin, velmax, velstep      ! velocity grid
C
10    format(A)
C
      numobs = 0
20    continue
         read(1,10, END=50) textline
         read(textline,*) li, lf, bw, bo
         do 30 itran=1,numtran                 ! see if initial, final
            if(li.EQ.linitial(itran) .AND.     ! levels match up with a
     +         lf.EQ.lfinal(itran)) then       ! known transition
                  numobs = numobs+1
                  transnum(numobs) = itran     ! yes, it's a good one...
                  beamsize(numobs) = bw
                  beamoffs(numobs) = bo
                  goto 40
            else
               write(*,*)
     +         ' WARNING - skipping unknown transition:  ',
     +         textline
            endif
30       continue
40       continue
      if(numobs.LT.MAXOBS) then
         goto 20
      endif
50    continue
C
      close(1)
      return
      end
C
C************************************************************************
      subroutine getmolnd(fname)
C************************************************************************
C
      character*(*) fname
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer_obs.inc'
      include 'cgs_constants.inc'
C
      character*80 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
      character*80 textline
C
      integer ir, ilev, ii
C
      open(UNIT=3, NAME=fname, STATUS='UNKNOWN')
C
      read(3,10) gridfile
10    format(A)
      read(3,10) specfile
      read(3,10) collfile
      read(3,10) dustfile
      read(3,10) textline
C
      call getgrid(gridfile)                ! read in radial grid
      call getspec(specfile)                ! read in energy levels, etc.
      call getdust(dustfile)                ! dust parameters
C
      read(3,10) textline                   ! skip nshell, numlev...
      do 100 ir=1, nshell
         read(3,10) textline                ! skip blank line
         read(3,10) textline                ! skip rshell(ir)
         do 70 ilev=1, numlev
            read(3,10) textline
            read(textline,*) ii, fracpop(ir, ilev)
70       continue
100   continue
C
      close(3)
      return
      end
C
C************************************************************************
      subroutine getgrid(infile)
C************************************************************************
C
      include 'grid.inc'
C
      character*(*) infile
      integer ir
C
      open(UNIT=1, FILE=infile, STATUS='UNKNOWN')
C
      read(1,*)
      read(1,*) igridver               ! grid version number
      if(igridver.NE.2) then           ! not version 2 like we need
         write(*,*) ' Incorrect version number in radial grid file',
     +              infile
         write(*,*) ' Need version 2; got version no.', igridver
         stop
      endif
C
      read(1,*)
      read(1,*)                        ! skip grid parameters file name
C
      read(1,*)
      read(1,*) rscalecm
      read(1,*) rmin, rmax
      read(1,*) nshell
      read(1,*) linewidth               ! this is FWHM in km/s
C
C     calculate 1/e half-width of line
C
      ewidth = linewidth/(2.*sqrt(log(2.)))
C
      read(1,*) numvel
      read(1,*)  h2den0, h2den1, h2denradius, h2denpower
      read(1,*)  Tgas0, Tgas1, Tgasradius, Tgaspower
      read(1,*)  Tdust0, Tdust1, Tdustradius, Tdustpower
      read(1,*)  abun0, abun1, abunradius, abunpower
      read(1,*)
      read(1,*)
C
C     Read in parameters as a function of radius
C     Note that radius is not normalized to 1 yet
C
      do 200 ir=1, nshell
         read(1,*)  rshell(ir), h2den(ir),
     +              Tgas(ir), Tdust(ir), abun(ir), Ncol(ir)
200   continue
C
      close(1)
C
C     Normalize shell radii now
C
      do 300 ir=1, nshell
         rshell(ir) = rshell(ir)/rmax
300   continue
C
      return
      end
C
C********************************************************************
      subroutine getspec(infile)
C********************************************************************
C
C     read energy levels, A coefficients from disk
C
      include 'spectrum.inc'
      include 'cgs_constants.inc'
C
      character*(*) infile
      character*120 textline
      integer i, j, li, lf
      real*4 x
C
1     format(A)
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
      read(1,*)                 ! skip comment line
      read(1,*) numlev
      if(numlev.GT.MAXLEV) then
         write(*,*)
     +     ' Too many energy levels in molecular transitions file'
         write(*,*) ' Number of levels = ', numlev
         write(*,*) ' You may need to recompile with larger MAXLEV'
         close(1)
         stop
      endif
C
      read(1,*)                 ! skip comment line
      do 20 i=1, numlev
         read(1,1) elabel(i)
20    continue
C
      read(1,*)                 ! skip comment line
C
      do 30 i=1, numlev
         read(1,1) textline
         read(textline,*) j, energy(i), degen(i)
30    continue
C
      read(1,*)                 ! skip comment line
C
C     Clear A-coefficient matrix
C
      do 50 i=1, MAXTRAN
         acoeff(i) = 0.
50    continue
C
      numtran = 0                            ! no transitions yet
C
60    continue
         read(1,1,END=70) textline
         read(textline, *) li, lf, x

         if(li.LT.0 .OR. li.GT.numlev .OR.    ! subscripts out of bounds
     +      lf.LT.0 .OR. lf.GT.numlev) then
C
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Bad energy level number:'
            write(*,1) textline
            close(1)
            stop
         endif
C
         if(energy(li).LE.energy(lf)) then
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Cannot have spontaneous upwards transitions'
            write(*,1) textline
            close(1)
            stop
         endif
C
         numtran = numtran + 1     
         acoeff(numtran) = x         ! looks OK, so stuff it
         linitial(numtran) = li
         lfinal(numtran) = lf
         x = energy(li) - energy(lf)
         tphoton(numtran) = x            ! photon temperature in K
         x = x*kboltz/(hplanck*amegahz)  ! convert to MHz
         frequency(numtran) = x
C
      if(numtran .LT. MAXTRAN) goto 60   ! process next transition
      write(*,*) ' Warning: may have too many transitions in file'
      write(*,*) ' Maximum number is ', MAXTRAN
C
70    continue
C
C     now done reading transitions...
C
      close(1)
C
C     so sort them by ascending frequency, including
C     the initial and final state indices
C
      call PIKSR3(numtran, frequency, linitial, lfinal)
C
      return
      end
C
C********************************************************************
      SUBROUTINE PIKSR3(N,ARR,BRR,CRR)
C********************************************************************
      REAL*8 ARR(N), A
      INTEGER BRR(N), CRR(N), B, C
      DO 12 J=2,N
        A=ARR(J)
        B=BRR(J)
        C=CRR(J)
        DO 11 I=J-1,1,-1
          IF(ARR(I).LE.A)GO TO 10
          ARR(I+1)=ARR(I)
          BRR(I+1)=BRR(I)
          CRR(I+1)=CRR(I)
11      CONTINUE
        I=0
10      ARR(I+1)=A
        BRR(I+1)=B
        CRR(I+1)=C
12    CONTINUE
      RETURN
      END
C
C**********************************************************************
      subroutine getdust(infile)
C**********************************************************************
C
C     reads dust parameters from disk
C
      include 'dust.inc'
C
      character*80 infile
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
      read(1,*) Qdust0
      read(1,*) wavdust0
      read(1,*) slopedust
      read(1,*) adust
      read(1,*) rhodust
      read(1,*) Rgasdust
      read(1,*) gasmu
      close(1)
      return
      end
C
C**********************************************************************
      real*8 function sigmadust(wavlen)
C**********************************************************************
C
C     Calculates dust cross-section (in cm-2) per H2 molecule
C
      real*8 wavlen                     ! wavelength in microns
C
      include 'dust.inc'
      include 'cgs_constants.inc'       ! for amicron = 1.E-4
C
      real*8 Qdust, result
C
      Qdust = Qdust0*(wavdust0/wavlen)**slopedust    ! efficiency
      result = 3*gasmu*Qdust/(4.*adust*amicron*rhodust*Rgasdust)
      sigmadust = result
      return
      end
C
C********************************************************************
      subroutine pathlen(r, nr, xp, ds, mr)
C********************************************************************
C
C     ***calculates pathlength thru shells 
C
C        Inputs:
C        r               - array of radii, in ASCENDING ORDER
C        nr              - no. of radii
C        xp              - impact parameter
C
C        Output:
C        ds(nr)          - pathlengths
C        mr              - index of minimum radius penetrated by xp
C********************************************************************
C
      real*8 r(nr), xp
      real*8 ds(nr)
      integer mr
C
      integer ir
      real*8 x1, x2
C
C     ***calculate mr
C
      ir = 1                  ! start from smallest radius
C
10    continue
      if(r(ir).LT.xp .AND. ir.LT.nr) then
         ir = ir+1            ! impact ip doesn't penetrate radius ir
         goto 10              ! so go to next larger radius
      endif
C
      if(r(ir).GE.xp) then
         mr = ir          ! smallest radius that ip penetrates
      else
         mr = nr+1        ! impact out of range - ip doesn't 
      endif               ! penetrate any radii
C
C     Calculate pathlengths
C
      ir = mr
      if(ir.LE.nr) then        ! this ray actually passes thru cloud
         x2 = xp/r(ir)
         x2 = x2*x2            ! calculate pathlength from ray center
C                              ! to first intersecting shell
         ds(ir) = r(ir)*sqrt(1.-x2)
C
         do 50 ir=mr+1, nr    ! now do pathlengths between shells
            x1 = r(ir)/r(ir-1)
            x1 = x1*x1
            x2 = xp/r(ir-1)
            x2 = x2*x2
            ds(ir) = r(ir-1)*(x1-1.)
     +               /(sqrt(x1-x2) + sqrt(1.-x2))
50       continue
      endif
C
      return
      end
C
C********************************************************************
      subroutine calcobserv(fname)
C********************************************************************
C
C     solve the radiative transfer for each observation and write
C     spectrum to output file
C
      character*(*) fname
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer_obs.inc'
      include 'cgs_constants.inc'
C
      real*8 sigmadust
      external sigmadust
C
      character*80 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
C
      integer itran, ishell, levi, levf
      integer iobs, ix, iy
C
      real*8 outinten                    ! emergent intensity
      real*8 ximpact                     ! impact parameter
      real*8 ebeamrad                    ! 1/e beam radius
C
      real*8 opacity(MAXGRID)            ! total opacity
      real*8 gasopacity(MAXGRID)         ! gas opacity
      real*8 sourcefn(MAXGRID)           ! gas+dust source function
C
      real*8 sgas(MAXGRID)               ! gas source function
      real*8 sdust(MAXGRID)              ! dust source function
      real*8 dustopacity(MAXGRID)        ! dust opacity
C
      real*8 vel                         ! velocity
      real*8 velprof                     ! normalized velocity profile
C
      real*8 wavelen                     ! wavelength in cm
      real*8 xsectdust                   ! dust absorption cross-section
      real*8 xsectgas                    ! gas absorption cross-section
C
      real*8 x1, x2, xi, xf, gi, gf, tph
C
C     ***weights for spatial integration over beam
C
      call gauher(sabs, swgt, nspatial)
C
C     ***Open output file
C
      open(UNIT=2, FILE=fname, STATUS='UNKNOWN')
C
1     format(A)
      write(2,1) observfile
      write(2,1) molndfile
      write(2,1) gridfile
      write(2,1) specfile
      write(2,1) collfile
      write(2,1) dustfile
      write(2,1) outfile
      write(2,*) numobs
C
C     ***Loop over observations
C
      do 100 iobs=1, numobs
C
         itran = transnum(iobs)          ! get transition number
C
         ebeamrad = beamsize(iobs)/(2.*sqrt(log(2.)))
C
         levi = linitial(itran)
         levf = lfinal(itran)
         gi = degen(levi)
         gf = degen(levf)
C
C        Write observation info to file
C
         write(2,*)
         write(2,16) elabel(levi), elabel(levf), frequency(itran),
     +                  beamsize(iobs), beamoffs(iobs)
C
16       format(A10, '-', A10, 3X, F10.0, ' GHz',
     +          3X, F12.4, 3X, F12.4)
C
C        ***transition wavelength in cm
C
         wavelen = clight/(frequency(itran)*amegahz)
C
C        ***dust absorption cross-section at this wavelength
C
         xsectdust = sigmadust(wavelen/amicron)
C
C        ***now calculate opacities in radial shells
C
         do 20 ishell=1, nshell
            dustopacity(ishell) = xsectdust*h2den(ishell)
     +                            *rmax*rscalecm
20       continue
C
C        ***now calculate gas and dust source functions
C
         tph = tphoton(itran)
C
         do 30 ishell=1, nshell
            x1 = tph/(exp(tph/Tdust(ishell))-1.)
            sdust(ishell) = x1
C
            xi = fracpop(ishell, levi)
            xf = fracpop(ishell, levf)
            x1 = tph/(gi*xf/(gf*xi) -1.)
            sgas(ishell) = x1
C
30       continue
C
C        ***LOOP OVER VELOCITIES***
C
         vel = velmin                    ! start with minimum velocity
40       continue                        ! this is for velocity loop
C
C           ***Calculate normalized velocity profile function in cgs units
C
            x1 = sqrt(PI)*ewidth*akm     ! ewidth is km/s; convert to cm/s
            velprof = exp(-(vel/ewidth)**2)/x1
C
C
            do 50 ishell=1, nshell
               xi = fracpop(ishell, levi)
               xf = fracpop(ishell, levf)
C
C              ***transition absorption cross-section per molecule,
C                 in cm-2 * Hz   (i.e. integrated over frequency)
C
               xsectgas = wavelen*wavelen*acoeff(itran)/(8.*PI)
               xsectgas = xsectgas
     +                    *xf*(gi/gf)*(1. - gf*xi/(gi*xf))
C
C           ***now multiply by frequency profile and abundance
C
               gasopacity(ishell) = xsectgas*rmax*rscalecm
     +                  *h2den(ishell)*velprof*wavelen*abun(ishell)
C
C              ***and add to get total opacity
C
               opacity(ishell) = gasopacity(ishell) 
     +                         + dustopacity(ishell)
50          continue
C
C           ***now calculate total source function
C
            do 60 ishell=1, nshell
               x1 = dustopacity(ishell)*sdust(ishell)
               x2 = gasopacity(ishell)*sgas(ishell)
               sourcefn(ishell) = (x1 + x2)
     +                            /opacity(ishell)
60          continue
C
C        ***we're ready to do the radiative transfer
C
            x1 = beamoffs(iobs)
            spectrum = 0.
            do 80 ix=1, nspatial
               do 70 iy=1, nspatial
                  ximpact = (sabs(ix)*ebeamrad + x1)**2
     +                     +(sabs(iy)*ebeamrad)**2
                  ximpact = sqrt(ximpact)/rmax
                  call doray(rshell, nshell, ximpact,
     +                 opacity, sourcefn, outinten)
                  spectrum = spectrum
     +                           + outinten*swgt(ix)*swgt(iy)
70             continue
80          continue
C
C           ***Normalize results and write to disk
C               PI comes from normalization of exp(-x**2)*exp(-y**2)
C
            spectrum = spectrum/PI
            write(2,*) vel, spectrum
C
            vel = vel + velstep    ! next velocity
         if(vel.LE.velmax) then
            goto 40
         endif
C
100   continue                  ! the loop over observations
C
      close(2)
C
      return
      end
c
C*******************************************************************
      subroutine doray(r,nr,xp,chi,ss,out)
C*******************************************************************
c
c     1-4-94  J. Zmuidzinas
c
c     Radiative transfer solver for spherical geometry
c     Based directly on 1-st order transfer eqn.
c     Solves for intensity along ray of given impact parameters
c     Assumes that opacity and source function vary linearly
c     between radial shells.
c
c     input:
c       r     - normalized shell radii (rgrid(1)=0, rgrid(nr)=1)
c       nr    - number of shells
c       xp    - impact parameter
c       chi   - absorption coefficient
c       ss    - source function
c     output:
c       out   - emergent intensity
c
c       **all are real*8 
c
c     All physical quantities (chi, s) are evaluated AT THE SHELL RADII.
c
c     The routine calculates the emergent intensity at the
c     outermost shell.
c     
C*******************************************************************
c
      include 'grid.inc'            ! for MAXGRID
C
      implicit real*8 (a-h,o-z)
C
C     Inputs:
      integer nr
      real*8 r(nr), xp, chi(nr), ss(nr)
C
C     Output:
      real*8 out
C
C     local variables:
C
      integer  mr
      real*8 dtau(MAXGRID), ifor(MAXGRID), iback(MAXGRID),
     +       e(MAXGRID), f(MAXGRID), g(MAXGRID),
     +       ds(MAXGRID)
      real*8 x1
C
      parameter (cut=1.e-2)
C
c     calculate pathlengths
c
      call pathlen(r, nr, xp, ds, mr)
c
c     calculate optical depth grid; e, f, g, h coefficients
c
      ifor(nr) = 0.               ! no radiation yet...
C
      ir = mr
c
      if(ir.LE.nr) then           ! ray actually passes thru cloud,
c                                 ! so go ahead...
c
      x1 = ds(ir)*chi(ir)         ! assume chi is constant in
      dtau(ir) = x1               ! innermost shell
      e(ir) = exp(-x1)
      if(abs(x1).LE.cut) then
         g(ir) = (x1/2.)*(1.-x1/3.*(1.-x1/4.*(1.-x1/5.*
     +                   (1.-x1/6.*(1.-x1/7.*(1.-x1/8.*
     +                   (1.-x1/9.*(1.-x1/10.))))))))
         f(ir) = x1*(1.-g(ir))
      else
         f(ir) = 1.-e(ir)
         g(ir)  =1. - f(ir)/x1
      endif
      do ir=mr+1, nr
         x1 = ds(ir)*(chi(ir)+chi(ir-1))*0.5
         dtau(ir) = x1
         e(ir) = exp(-x1)
         if(abs(x1).LE.cut) then
            g(ir) = (x1/2.)*(1.-x1/3.*(1.-x1/4.*(1.-x1/5.*
     +                      (1.-x1/6.*(1.-x1/7.*(1.-x1/8.*
     +                      (1.-x1/9.*(1.-x1/10.))))))))
            f(ir) = x1*(1.-g(ir))
         else
            f(ir) = 1.-e(ir)
            g(ir)  =1. - f(ir)/x1
         endif
      enddo
c
c     do the transfer
c
c     first, loop backwards to get ingoing ray
c
      iback(nr) = 0.              ! no background radiation incident
      do ir=nr-1, mr, -1
         iback(ir) = e(ir+1)*iback(ir+1)
     +               + f(ir+1)*ss(ir+1)
     +               + g(ir+1)*(ss(ir)-ss(ir+1))
      enddo
c
c     now loop forwards to get outgoing ray
c
      ir = mr
      x1 = e(ir)*e(ir)
      ifor(ir) = x1*iback(ir) + (1.-x1)*ss(ir)
      do ir=mr+1, nr
         ifor(ir) = e(ir)*ifor(ir-1)
     +              + f(ir)*ss(ir-1)
     +              + g(ir)*(ss(ir)-ss(ir-1))
      enddo
C
      endif                 ! if(mr.LE.nr)
C
      out = ifor(nr)        ! this is the emergent intensity
C
      return
      end
C
C********************************************************************
      SUBROUTINE gauher(x,w,n)
C********************************************************************
      INTEGER n,MAXIT
      REAL*4 w(n),x(n)
      REAL*8 EPS,PIM4
      PARAMETER (EPS=3.D-14,PIM4=.7511255444649425D0,MAXIT=10)
      INTEGER i,its,j,m
      REAL*8 p1,p2,p3,pp,z,z1
      m=(n+1)/2
      do 13 i=1,m
        if(i.eq.1)then
          z=sqrt(float(2*n+1))-1.85575*(2*n+1)**(-.16667)
        else if(i.eq.2)then
          z=z-1.14*n**.426/z
        else if (i.eq.3)then
          z=1.86*z-.86*x(1)
        else if (i.eq.4)then
          z=1.91*z-.91*x(2)
        else
          z=2.*z-x(i-2)
        endif
        do 12 its=1,MAXIT
          p1=PIM4
          p2=0.d0
          do 11 j=1,n
            p3=p2
            p2=p1
            p1=z*sqrt(2.d0/j)*p2-sqrt(dble(j-1)/dble(j))*p3
11        continue
          pp=sqrt(2.d0*n)*p2
          z1=z
          z=z1-p1/pp
          if(abs(z-z1).le.EPS)goto 1
12      continue
        pause 'too many iterations in gauher'
1       x(i)=z
        x(n+1-i)=-z
        w(i)=2.d0/(pp*pp)
        w(n+1-i)=w(i)
13    continue
      return
      END
C
