C**********************************************************************
C
C     Dust model for SgrB2
C
C     R-2 + const. density law
C
C**********************************************************************
      program Trad
C**********************************************************************
C
C     General parameters and conversion factors to cgs
C
      real*8 PI
      real*8 Msol, pc, mu
      real mm, micron
      parameter (PI=3.1415926535)
      parameter (Msol=2E33, pc=3.09E18, mu=4.58E-24)
      parameter (mm=0.1, micron=1E-4 ) 
C
C     Sgr B2 density law
C
      real n1, n2, r1, Rmax
      parameter (n1=5.5E4, n2=2.2E3, r1=1.25, Rmax=22.5)
C
C     Angle (arcsec) subtended by 1pc at SgrB2 (D=8.5kpc)
C
      real theta1
      parameter (theta1=180*60*60/(8500*PI))
C
      real Tantenna
      real amu, radius1, lambd1
      common/TAint/ amu, radius1, lambd1
C
      real Tr
      external Tr
C
      real radmin, radmax, radstep, ncrit, Tphot, den
C
      write(*,*) ' Radius from center (pc; min, max, step) ?'
      read(*,*) radmin, radmax, radstep
      write(*,*) ' Wavelength (microns) ?'
      read(*,*) lambd1
      Tphot = 23.6*609./lambd1
      write(*,*) ' Transition critical density, cm-3 ?'
      read(*,*) ncrit
      write(*,*) ' Lambda = ', lambd1, ' microns'
      write(*,*)
      write(*,*) ' Radius (pc)   TA (K)   Density   TA/Tphot  n/ncrit'
C
C     loop over positions
C
      radius1 = radmin
10    continue
C
         den = n1*(r1/radius1)**2 + n2
         Tantenna = Tr(radius1)
C
         write(*,*) radius1, Tantenna, den, Tantenna/Tphot, den/ncrit
         radius1 = radius1 + radstep
      if(radius1.LE.radmax) goto 10
      stop
      end
C
C**********************************************************************
      real function TAinteg(z)
C**********************************************************************
C
C     This calculates the integrand needed in the integration
C     through the cloud determine the antenna temperature at
C     wavelength *** lambd1 *** and at impact parameter *** b1 ***.
C     The integrand is evaluated at some depth *** z ***
C     into the cloud.
C
      real z
      real amu, radius1, lambd1
      common/TAint/ amu, radius1, lambd1
C
C     variables we need to calculate
C
      real b1
      real radius, Tdust, planck
      real  density, Ncol1, Ncol2
      real  Qdust, sigdust, tau
      real integrand
C
C     General parameters and conversion factors to cgs
C
      real*8 PI
      real*8 Msol, pc, mu
      real mm, micron
      parameter (PI=3.1415926535)
      parameter (Msol=2E33, pc=3.09E18, mu=4.58E-24)
      parameter (mm=0.1, micron=1E-4 ) 
C
C     dust parameters
C
      real adust, rhodust, Rgasdust
      real Q0dust, lam0dust, slopedust
      parameter(adust=0.1, rhodust=3, Rgasdust=100)
      parameter(Q0dust=2.E-5, lam0dust=1300, slopedust=1.5)
C
C     SgrB2-specific parameters
C
C     dust temperature profile (Lis&G, ApJ 356, 195, 1990)
C
      real Tdust0, rdust0, slopetemp
      parameter (Tdust0=398. , rdust0=0.01, slopetemp=-0.5)
C
C     Density law (after Lis&G, ApJ 337, 704, 1989 C18O data)
C
      real n1, n2, r1, Rmax
      parameter (n1=5.5E4, n2=2.2E3, r1=1.25, Rmax=22.5)
C
C     Stuff for Planck function
C
      real lambdaT
      parameter (lambdaT=11605.9*1.23963) ! in micron-K
C
C
C     Start calculating stuff
C
C     radius, temperature, Planck function, density:
C
      b1 = sqrt(1.-amu*amu)*radius1
      radius = sqrt(b1*b1+z*z)
      Tdust = Tdust0*(radius/rdust0)**slopetemp
      planck = lambdaT/(lambd1*(exp(lambdaT/(lambd1*Tdust))-1))
      density = n1*(r1/radius)**2 + n2
C
C     dust absorption cross-section per H2 molecule
C
      Qdust=Q0dust*(lam0dust/lambd1)**slopedust
      sigdust = 3*mu*Qdust/(4*adust*micron*rhodust*Rgasdust)
C
C     H2 column densities of the n1 and n2 density components
C
      Ncol1 = n1*((r1*pc)**2)/(b1*pc)
      Ncol1 = Ncol1*(atan(z/b1) -atan(amu*radius1/b1))
      Ncol2 = n2*Rmax*pc
      Ncol2 = Ncol2*(z/Rmax - amu*radius1/Rmax)
C
C     Dust optical depth
C
      tau = (Ncol1+Ncol2)*sigdust
C
C     Put this all together in the integrand
C
      integrand = planck*sigdust*density*pc*exp(-tau)
      TAinteg = integrand
      return
      end
C
C
C**********************************************************************
      real function Tray(amu1)
C**********************************************************************
C
      real amu1
      real TAinteg
      external TAinteg, QROMB1, MIDPT1
C
      real result, zmin, zmax
      real amu, radius1, lambd1
      common/TAint/ amu, radius1, lambd1
C
C     Only need this for Rmax
C
      real n1, n2, r1, Rmax
      parameter (n1=5.5E4, n2=2.2E3, r1=1.25, Rmax=22.5)
C
C     stuff the direction cosine into the common block
C
      amu = amu1
C
C     do the integration
C
      zmax = sqrt(Rmax*Rmax-(1.-amu*amu)*radius1*radius1)
      zmin = amu*radius1
      call QROMB1(TAinteg, zmin, zmax, result, MIDPT1)
      Tray = result
      return
      end
C
C**********************************************************************
      real function Tr(rad)
C**********************************************************************
C
      real rad
      real Tray
      external Tray, QROMB2, MIDPT2
C
      real amu, radius1, lambd1
      common/TAint/ amu, radius1, lambd1
C
      real result, ymin, ymax
      real EPS
      parameter (EPS=1.E-3)
C
C     stuff radius into common block
C
      radius1 = rad
C
      ymin = -1.+EPS
      ymax = 1.-EPS
      call QROMB2(Tray, ymin, ymax, result, MIDPT2)
      Tr = result/2.
      return
      end
C
C**********************************************************************
      SUBROUTINE QROMB1(FUNC,A,B,SS,CHOOSE)
C**********************************************************************
      PARAMETER (EPS=1.E-5,JMAX=14,JMAXP=JMAX+1,KM=4,K=KM+1)
      DIMENSION S(JMAXP),H(JMAXP)
      REAL FUNC
      EXTERNAL FUNC, CHOOSE
      H(1)=1.
      DO 11 J=1,JMAX
        CALL CHOOSE(FUNC,A,B,S(J),J)
        IF (J.GE.K) THEN
          CALL POLINT(H(J-KM),S(J-KM),K,0.0,SS,DSS)
          IF (ABS(DSS).LT.EPS*ABS(SS)) RETURN
        ENDIF
        S(J+1)=S(J)
        H(J+1)=H(J)/9.
11    CONTINUE
      PAUSE 'Too many steps.'
      END
C
C**********************************************************************
      SUBROUTINE QROMB2(FUNC,A,B,SS,CHOOSE)
C**********************************************************************
      PARAMETER (EPS=1.E-5,JMAX=14,JMAXP=JMAX+1,KM=4,K=KM+1)
      DIMENSION S(JMAXP),H(JMAXP)
      REAL FUNC
      EXTERNAL FUNC, CHOOSE
      H(1)=1.
      DO 11 J=1,JMAX
        CALL CHOOSE(FUNC,A,B,S(J),J)
        IF (J.GE.K) THEN
          CALL POLINT(H(J-KM),S(J-KM),K,0.0,SS,DSS)
          IF (ABS(DSS).LT.EPS*ABS(SS)) RETURN
        ENDIF
        S(J+1)=S(J)
        H(J+1)=H(J)/9.
11    CONTINUE
      PAUSE 'Too many steps.'
      END
C
C**********************************************************************
      SUBROUTINE MIDSQU(FUNK,AA,BB,S,N)
C**********************************************************************
      REAL FUNK
      EXTERNAL FUNK
      FUNC(X) = 2.*X*FUNK(BB-X**2)
      B=SQRT(BB-AA)
      A=0.
      IF (N.EQ.1) THEN
        S=(B-A)*FUNC(0.5*(A+B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/(3.*TNM)
        DDEL=DEL+DEL
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DDEL
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=(S+(B-A)*SUM/TNM)/3.
        IT=3*IT
      ENDIF
      RETURN
      END
C
C**********************************************************************
      SUBROUTINE MIDSQL(FUNK,AA,BB,S,N)
C**********************************************************************
      REAL FUNK
      EXTERNAL FUNK
      FUNC(X) = 2.*X*FUNK(AA+X**2)
      B=SQRT(BB-AA)
      A=0.
      IF (N.EQ.1) THEN
        S=(B-A)*FUNC(0.5*(A+B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/(3.*TNM)
        DDEL=DEL+DEL
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DDEL
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=(S+(B-A)*SUM/TNM)/3.
        IT=3*IT
      ENDIF
      RETURN
      END
C
C**********************************************************************
      SUBROUTINE MIDPT1(FUNC,A,B,S,N)
C**********************************************************************
      REAL FUNC, A, B, S
      INTEGER N
      EXTERNAL FUNC
      IF (N.EQ.1) THEN
        S=(B-A)*FUNC(0.5*(A+B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/(3.*TNM)
        DDEL=DEL+DEL
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DDEL
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=(S+(B-A)*SUM/TNM)/3.
        IT=3*IT
      ENDIF
      RETURN
      END
C**********************************************************************
      SUBROUTINE MIDPT2(FUNC,A,B,S,N)
C**********************************************************************
      REAL FUNC, A, B, S
      INTEGER N
      EXTERNAL FUNC
      IF (N.EQ.1) THEN
        S=(B-A)*FUNC(0.5*(A+B))
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/(3.*TNM)
        DDEL=DEL+DEL
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
          SUM=SUM+FUNC(X)
          X=X+DDEL
          SUM=SUM+FUNC(X)
          X=X+DEL
11      CONTINUE
        S=(S+(B-A)*SUM/TNM)/3.
        IT=3*IT
      ENDIF
      RETURN
      END
C
C**********************************************************************
      SUBROUTINE POLINT(XA,YA,N,X,Y,DY)
C**********************************************************************
      PARAMETER (NMAX=10) 
      DIMENSION XA(NMAX),YA(NMAX),C(NMAX),D(NMAX)
      NS=1
      DIF=ABS(X-XA(1))
      DO 11 I=1,N 
        DIFT=ABS(X-XA(I))
        IF (DIFT.LT.DIF) THEN
          NS=I
          DIF=DIFT
        ENDIF
        C(I)=YA(I)
        D(I)=YA(I)
11    CONTINUE
      Y=YA(NS)
      NS=NS-1
      DO 13 M=1,N-1
        DO 12 I=1,N-M
          HO=XA(I)-X
          HP=XA(I+M)-X
          W=C(I+1)-D(I)
          DEN=HO-HP
          IF(DEN.EQ.0.)PAUSE
          DEN=W/DEN
          D(I)=HP*DEN
          C(I)=HO*DEN
12      CONTINUE
        IF (2*NS.LT.N-M)THEN
          DY=C(NS+1)
        ELSE
          DY=D(NS)
          NS=NS-1
        ENDIF
        Y=Y+DY
13    CONTINUE
      RETURN
      END
C
