C************************************************************************
      program radprof 
C************************************************************************
C
C     J. Zmuidzinas  4-12-95
C
C     Calculate radial variation of antenna temperature, etc.
C
C
C************************************************************************
C
      character*80 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
C
      real*8 abunfac
      common/ab/abunfac
C
      write(*,*) ' Name of file created by molnd ? '
10    format(A)
      read(*,10) molndfile
      write(*,*) ' File name giving observation list ? '
      read(*,10) observfile
      write(*,*) ' Abundance factor (normally 1.0) ?'
      read(*,*) abunfac
      write(*,*) ' Output file ? '
      read(*,10) outfile
C
      call getmolnd(molndfile)  
      call getobserv(observfile)
      call calcobserv(outfile)
C
      stop
      end
C
C************************************************************************
      subroutine getobserv(fname)
C************************************************************************
C
      character*(*) fname
      character*100 textline
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer_obs.inc'
      include 'cgs_constants.inc'
C
      integer itran, li, lf
      real*8 bw, bo
C
      open(UNIT=1, NAME=fname, STATUS='UNKNOWN')
C
      read(1,*) velmin, velmax, velstep      ! velocity grid
C
10    format(A)
C
      numobs = 0
20    continue
         read(1,10, END=50) textline
         read(textline,*) li, lf, bw, bo
         do 30 itran=1,numtran                 ! see if initial, final
            if(li.EQ.linitial(itran) .AND.     ! levels match up with a
     +         lf.EQ.lfinal(itran)) then       ! known transition
                  numobs = numobs+1
                  transnum(numobs) = itran     ! yes, it's a good one...
                  beamsize(numobs) = bw
                  beamoffs(numobs) = bo
                  goto 40
            endif
30       continue
         write(*,*)
     +         ' WARNING - skipping unknown transition:  ',
     +         textline
40       continue
      if(numobs.LT.MAXOBS) then
         goto 20
      endif
50    continue
C
      close(1)
      return
      end
C
C************************************************************************
      subroutine getmolnd(fname)
C************************************************************************
C
      character*(*) fname
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer_obs.inc'
      include 'cgs_constants.inc'
C
      character*80 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
      character*80 textline
C
      integer ir, ilev, ii
C
      open(UNIT=3, NAME=fname, STATUS='UNKNOWN')
C
      read(3,10) gridfile
10    format(A)
      read(3,10) specfile
      read(3,10) collfile
      read(3,10) dustfile
      read(3,10) textline
C
      call getgrid(gridfile)                ! read in radial grid
      call getspec(specfile)                ! read in energy levels, etc.
      call getdust(dustfile)                ! dust parameters
C
      read(3,10) textline                   ! skip nshell, numlev...
      do 100 ir=1, nshell
         read(3,10) textline                ! skip blank line
         read(3,10) textline                ! skip rshell(ir)
         do 70 ilev=1, numlev
            read(3,10) textline
            read(textline,*) ii, fracpop(ir, ilev)
70       continue
100   continue
C
      close(3)
      return
      end
C
C************************************************************************
      subroutine getgrid(infile)
C************************************************************************
C
      include 'grid.inc'
C
      character*(*) infile
      integer ir
C
      open(UNIT=1, FILE=infile, STATUS='UNKNOWN')
C
      read(1,*)
      read(1,*) igridver               ! grid version number
      if(igridver.NE.2) then           ! not version 2 like we need
         write(*,*) ' Incorrect version number in radial grid file',
     +              infile
         write(*,*) ' Need version 2; got version no.', igridver
         stop
      endif
C
      read(1,*)
      read(1,*)                        ! skip grid parameters file name
C
      read(1,*)
      read(1,*) rscalecm
      read(1,*) rmin, rmax
      read(1,*) nshell
      read(1,*) linewidth               ! this is FWHM in km/s
C
C     calculate 1/e half-width of line
C
      ewidth = linewidth/(2.*sqrt(log(2.)))
C
      read(1,*) numvel
      read(1,*)  h2den0, h2den1, h2denradius, h2denpower
      read(1,*)  Tgas0, Tgas1, Tgasradius, Tgaspower
      read(1,*)  Tdust0, Tdust1, Tdustradius, Tdustpower
      read(1,*)  abun0, abun1, abunradius, abunpower
      read(1,*)
      read(1,*)
C
C     Read in parameters as a function of radius
C     Note that radius is not normalized to 1 yet
C
      do 200 ir=1, nshell
         read(1,*)  rshell(ir), h2den(ir),
     +              Tgas(ir), Tdust(ir), abun(ir), Ncol(ir)
200   continue
C
      close(1)
C
C     Normalize shell radii now
C
      do 300 ir=1, nshell
         rshell(ir) = rshell(ir)/rmax
300   continue
C
      return
      end
C
C********************************************************************
      subroutine getspec(infile)
C********************************************************************
C
C     read energy levels, A coefficients from disk
C
      include 'spectrum.inc'
      include 'cgs_constants.inc'
C
      character*(*) infile
      character*120 textline
      integer i, j, li, lf
      real*4 x
C
1     format(A)
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
      read(1,*)                 ! skip comment line
      read(1,*) numlev
      if(numlev.GT.MAXLEV) then
         write(*,*)
     +     ' Too many energy levels in molecular transitions file'
         write(*,*) ' Number of levels = ', numlev
         write(*,*) ' You may need to recompile with larger MAXLEV'
         close(1)
         stop
      endif
C
      read(1,*)                 ! skip comment line
      do 20 i=1, numlev
         read(1,1) elabel(i)
20    continue
C
      read(1,*)                 ! skip comment line
C
      do 30 i=1, numlev
         read(1,1) textline
         read(textline,*) j, energy(i), degen(i)
30    continue
C
      read(1,*)                 ! skip comment line
C
C     Clear A-coefficient matrix
C
      do 50 i=1, MAXTRAN
         acoeff(i) = 0.
50    continue
C
      numtran = 0                            ! no transitions yet
C
60    continue
         read(1,1,END=70) textline
         read(textline, *) li, lf, x

         if(li.LT.0 .OR. li.GT.numlev .OR.    ! subscripts out of bounds
     +      lf.LT.0 .OR. lf.GT.numlev) then
C
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Bad energy level number:'
            write(*,1) textline
            close(1)
            stop
         endif
C
         if(energy(li).LE.energy(lf)) then
            write(*,*) ' Error in molecular transition file'
            write(*,*) ' Cannot have spontaneous upwards transitions'
            write(*,1) textline
            close(1)
            stop
         endif
C
         numtran = numtran + 1     
         acoeff(numtran) = x         ! looks OK, so stuff it
         linitial(numtran) = li
         lfinal(numtran) = lf
         x = energy(li) - energy(lf)
         tphoton(numtran) = x            ! photon temperature in K
         x = x*kboltz/(hplanck*amegahz)  ! convert to MHz
         frequency(numtran) = x
C
      if(numtran .LT. MAXTRAN) goto 60   ! process next transition
      write(*,*) ' Warning: may have too many transitions in file'
      write(*,*) ' Maximum number is ', MAXTRAN
C
70    continue
C
C     now done reading transitions...
C
      close(1)
      return
      end
C
C**********************************************************************
      subroutine getdust(infile)
C**********************************************************************
C
C     reads dust parameters from disk
C
      include 'dust.inc'
C
      character*80 infile
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
      read(1,*) Qdust0
      read(1,*) wavdust0
      read(1,*) slopedust
      read(1,*) adust
      read(1,*) rhodust
      read(1,*) Rgasdust
      read(1,*) gasmu
      close(1)
      return
      end
C
C**********************************************************************
      real*8 function sigmadust(wavlen)
C**********************************************************************
C
C     Calculates dust cross-section (in cm-2) per H2 molecule
C
      real*8 wavlen                     ! wavelength in microns
C
      include 'dust.inc'
      include 'cgs_constants.inc'       ! for amicron = 1.E-4
C
      real*8 Qdust, result
C
      Qdust = Qdust0*(wavdust0/wavlen)**slopedust    ! efficiency
      result = 3*gasmu*Qdust/(4.*adust*amicron*rhodust*Rgasdust)
      sigmadust = result
      return
      end
C
C********************************************************************
      subroutine pathlen(r, nr, xp, ds, mr)
C********************************************************************
C
C     ***calculates pathlength thru shells 
C
C        Inputs:
C        r               - array of radii, in ASCENDING ORDER
C        nr              - no. of radii
C        xp              - impact parameter
C
C        Output:
C        ds(nr)          - pathlengths
C        mr              - index of minimum radius penetrated by xp
C********************************************************************
C
      real*8 r(nr), xp
      real*8 ds(nr)
      integer mr
C
      integer ir
      real*8 x1, x2
C
C     ***calculate mr
C
      ir = 1                  ! start from smallest radius
C
10    continue
      if(r(ir).LT.xp .AND. ir.LT.nr) then
         ir = ir+1            ! impact ip doesn't penetrate radius ir
         goto 10              ! so go to next larger radius
      endif
C
      if(r(ir).GE.xp) then
         mr = ir          ! smallest radius that ip penetrates
      else
         mr = nr+1        ! impact out of range - ip doesn't 
      endif               ! penetrate any radii
C
C     Calculate pathlengths
C
      ir = mr
      if(ir.LE.nr) then        ! this ray actually passes thru cloud
         x2 = xp/r(ir)
         x2 = x2*x2            ! calculate pathlength from ray center
C                              ! to first intersecting shell
         ds(ir) = r(ir)*sqrt(1.-x2)
C
         do 50 ir=mr+1, nr    ! now do pathlengths between shells
            x1 = r(ir)/r(ir-1)
            x1 = x1*x1
            x2 = xp/r(ir-1)
            x2 = x2*x2
            ds(ir) = r(ir-1)*(x1-1.)
     +               /(sqrt(x1-x2) + sqrt(1.-x2))
50       continue
      endif
C
      return
      end
C
C********************************************************************
      subroutine calcobserv(fname)
C********************************************************************
C
C     solve the radiative transfer for each observation
C     write radial profiles to output file
C
      character*(*) fname
C
      include 'spectrum.inc'
      include 'grid.inc'
      include 'transfer_obs.inc'
      include 'intensity.inc'
      include 'cgs_constants.inc'
C
      real*8 sigmadust
      external sigmadust
C
      character*80 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
      common/filenames/
     +                 gridfile, specfile, collfile, dustfile,
     +                 molndfile, observfile, outfile
C
      real*8 abunfac
      common/ab/abunfac
C
      integer itran, ishell, levi, levf
      integer iobs
C
      real*8 gasopacity(MAXGRID)         ! gas opacity
C
      real*8 sgas(MAXGRID)               ! gas source function
      real*8 sdust(MAXGRID)              ! dust source function
      real*8 dustopacity(MAXGRID)        ! dust opacity
      real*8 outinten(MAXGRID)           ! outgoing intensity vs radius
      real*8 optdepth(MAXGRID)           ! optical depth vs radius
C
      real*8 vel                         ! velocity
      real*8 velprof                     ! normalized velocity profile
C
      real*8 wavelen                     ! wavelength in cm
      real*8 xsectdust                   ! dust absorption cross-section
      real*8 xsectgas                    ! gas absorption cross-section
C
      real*8 x1, x2, xi, xf, gi, gf, tph, denom
      real*8 sp, xp
C
      real*8 tiny
      parameter(tiny=1.e-10)
C
C     ***Open output file
C
      open(UNIT=2, FILE=fname, STATUS='UNKNOWN')
C
1     format(A)
      write(2,1) observfile
      write(2,1) molndfile
      write(2,1) gridfile
      write(2,1) specfile
      write(2,1) collfile
      write(2,1) dustfile
      write(2,1) outfile
      write(2,*) numobs
C
C     ***Loop over observations
C
      do 100 iobs=1, numobs
C
         itran = transnum(iobs)          ! get transition number
C
         levi = linitial(itran)
         levf = lfinal(itran)
         gi = degen(levi)
         gf = degen(levf)
C
C        Write observation info to file
C
         write(2,*)
         write(2,*)
         write(2,16) elabel(levi), elabel(levf), frequency(itran)
C
16       format(A10, '-', A10, 3X, F10.0, ' MHz')
         write(2,*)
         write(2,17)
17          format(' ishl', '   Radius   ', ' Mol src fn ',
     +             'Dust src fn ', ' Tot src fn ',
     +             '  Opacity   ',
     +             'Optical depth', '  Ant Temp  ')
C
C        ***transition wavelength in cm
C
         wavelen = clight/(frequency(itran)*amegahz)
C
C        ***dust absorption cross-section at this wavelength
C
         xsectdust = sigmadust(wavelen/amicron)
C
C        ***now calculate opacities in radial shells
C
         do 20 ishell=1, nshell
            dustopacity(ishell) = xsectdust*h2den(ishell)
     +                            *rmax*rscalecm
20       continue
C
C        ***now calculate gas and dust source functions
C
         tph = tphoton(itran)
C
         do 30 ishell=1, nshell
            x1 = tph/(exp(tph/Tdust(ishell))-1.)
            sdust(ishell) = x1
C
            xi = fracpop(ishell, levi)
            xf = fracpop(ishell, levf)
            denom = gi*xf - gf*xi
            if(abs(denom).LE.tiny) then
               x1 = 0.
            else
               x1 = tph*gf*xi/denom
            endif
            sgas(ishell) = x1
C
30       continue
C
C        ***SET VELOCITY = 0***
C
         vel = 0                      
C
C           ***Calculate normalized velocity profile function in cgs units
C
            x1 = sqrt(PI)*ewidth*akm     ! ewidth is km/s; convert to cm/s
            velprof = exp(-(vel/ewidth)**2)/x1
C
C
            do 50 ishell=1, nshell
               xi = fracpop(ishell, levi)
               xf = fracpop(ishell, levf)
C
C              ***transition absorption cross-section per molecule,
C                 in cm-2 * Hz   (i.e. integrated over frequency)
C
               xsectgas = wavelen*wavelen*acoeff(itran)/(8.*PI)
               xsectgas = xsectgas
     +                    *(1./gf)*(gi*xf - gf*xi)
C
C           ***now multiply by frequency profile and abundance
C
               gasopacity(ishell) = xsectgas*rmax*rscalecm
     +                  *h2den(ishell)*velprof*wavelen*abun(ishell)
     +                  *abunfac
C
C              ***and add to get total opacity
C
               opacity(ishell) = gasopacity(ishell) 
     +                         + dustopacity(ishell)
50          continue
C
C           ***now calculate total source function
C
            do 60 ishell=1, nshell
               x1 = dustopacity(ishell)*sdust(ishell)
               x2 = gasopacity(ishell)*sgas(ishell)
               sourcefn(ishell) = (x1 + x2)
     +                            /opacity(ishell)
60          continue
C
C        ***we're ready to do the radiative transfer
C
            xp = 0.    ! impact parameter is zero
            call doray(rshell, nshell, xp, opacity,
     +                 sourcefn, outinten, optdepth)
C
            do 70 ishell = 1, nshell
               write(2,65) ishell, rshell(ishell)*rmax,
     +              sgas(ishell), sdust(ishell), sourcefn(ishell),
     +              opacity(ishell),
     +              optdepth(ishell), outinten(ishell)
65             format(' ', I4, 7(E11.4, 1X))
70          continue
C
C
100   continue                  ! the loop over observations
C
      close(2)
C
      return
      end
c
C*******************************************************************
      subroutine doray(r,nr,xp,chi,ss,out,tau)
C*******************************************************************
c
c     1-4-94  J. Zmuidzinas
c
c     Radiative transfer solver for spherical geometry
c     Based directly on 1-st order transfer eqn.
c     Solves for intensity along ray of given impact parameters
c     Assumes that opacity and source function vary linearly
c     between radial shells.
c
c     input:
c       r     - normalized shell radii (rgrid(1)=0, rgrid(nr)=1)
c       nr    - number of shells
c       xp    - impact parameter
c       chi   - absorption coefficient
c       ss    - source function
c     output:
c       out   - outgoing intensity vs. radius
c       tau   - optical depth vs. radius
c
c       **all are real*8 
c
c     All physical quantities (chi, s) are evaluated AT THE SHELL RADII.
c
c     The routine calculates the outgoing intensity for each
c     radial shell, and optical depth vs. radius.
c     
C*******************************************************************
c
      implicit real*8 (a-h,o-z)
C
      include 'grid.inc'            ! for MAXGRID
C
C
C     Inputs:
      integer nr
      real*8 r(nr), xp, chi(nr), ss(nr)
C
C     Output:
      real*8 out(nr), tau(nr)
C
C     local variables:
C
      integer  mr
      real*8 dtau(MAXGRID), ifor(MAXGRID), iback(MAXGRID),
     +       e(MAXGRID), f(MAXGRID), g(MAXGRID),
     +       ds(MAXGRID)
      real*8 x1
C
      parameter (cut=1.e-2)
C
C
C     zero outgoing intensities and optical depth...
C
      do ir = 1, nr
         out(ir) = 0.
         tau(ir) = 0.
      enddo
C
c     calculate pathlengths
c
      call pathlen(r, nr, xp, ds, mr)
c
c     calculate optical depth grid; e, f, g, h coefficients
c
      ifor(nr) = 0.               ! no radiation yet...
C
      ir = mr
c
      if(ir.LE.nr) then           ! ray actually passes thru cloud,
c                                 ! so go ahead...
c
      x1 = ds(ir)*chi(ir)         ! assume chi is constant in
      dtau(ir) = x1               ! innermost shell
      e(ir) = exp(-x1)
      if(abs(x1).LE.cut) then
         g(ir) = (x1/2.)*(1.-x1/3.*(1.-x1/4.*(1.-x1/5.*
     +                   (1.-x1/6.*(1.-x1/7.*(1.-x1/8.*
     +                   (1.-x1/9.*(1.-x1/10.))))))))
         f(ir) = x1*(1.-g(ir))
      else
         f(ir) = 1.-e(ir)
         g(ir)  =1. - f(ir)/x1
      endif
      do ir=mr+1, nr
         x1 = ds(ir)*(chi(ir)+chi(ir-1))*0.5
         dtau(ir) = x1
         tau(ir) = x1
         e(ir) = exp(-x1)
         if(abs(x1).LE.cut) then
            g(ir) = (x1/2.)*(1.-x1/3.*(1.-x1/4.*(1.-x1/5.*
     +                      (1.-x1/6.*(1.-x1/7.*(1.-x1/8.*
     +                      (1.-x1/9.*(1.-x1/10.))))))))
            f(ir) = x1*(1.-g(ir))
         else
            f(ir) = 1.-e(ir)
            g(ir)  =1. - f(ir)/x1
         endif
      enddo
c
c     do the transfer
c
c     first, loop backwards to get ingoing ray
c
      iback(nr) = 0.              ! no background radiation incident
      tau(nr) = dtau(nr)
      do ir=nr-1, mr, -1
         iback(ir) = e(ir+1)*iback(ir+1)
     +               + f(ir+1)*ss(ir+1)
     +               + g(ir+1)*(ss(ir)-ss(ir+1))
         tau(ir) = tau(ir+1) + dtau(ir)
      enddo
c
c     now loop forwards to get outgoing ray
c
      ir = mr
      x1 = e(ir)*e(ir)
      ifor(ir) = x1*iback(ir) + (1.-x1)*ss(ir)
      out(ir) = ifor(ir)
      do ir=mr+1, nr
         ifor(ir) = e(ir)*ifor(ir-1)
     +              + f(ir)*ss(ir-1)
     +              + g(ir)*(ss(ir)-ss(ir-1))
         out(ir) = ifor(ir)
      enddo
C
      endif                 ! if(mr.LE.nr)
C
      return
      end
C
